@@ -12,20 +12,21 @@
    Changes by Robert Millan <rmh@aybabtu.com>.
 */
 
-#include "os.h"
-#include "common.h"
-#include "extern_data.h"
-
-#ifdef SYS_WINDOWS
-# include "os_windows.h"
-#endif
+#include "common/os.h"
 
+#include <algorithm>
 #if HAVE_MAGIC_H
 extern "C" {
 #include <magic.h>
 };
 #endif
 
+#include "common/common.h"
+#include "common/extern_data.h"
+#include "common/fs_sys_helpers.h"
+#include "common/mm_io.h"
+#include "common/strings/editing.h"
+
 #ifndef NULL
 # define NULL 0
 #endif
@@ -2609,9 +2610,9 @@ const char *cctlds[] = {
   NULL
 };
 
-static string
-guess_mime_type_by_ext(string ext) {
-  vector<string> extensions;
+static std::string
+guess_mime_type_by_ext(std::string ext) {
+  std::vector<std::string> extensions;
   int i, j;
 
   /* chop off basename */
@@ -2634,11 +2635,46 @@ guess_mime_type_by_ext(string ext) {
   return "";
 }
 
-string
-guess_mime_type(string ext,
+#if HAVE_MAGIC_H
+static std::string
+guess_mime_type_by_content(magic_t &m,
+                           const std::string &file_name) {
+  try {
+    mm_file_io_c file(file_name);
+    int64_t file_size = file.get_size();
+    int buffer_size   = 0;
+    memory_cptr buf   = memory_c::alloc(1024 * 1024);
+    int i;
+
+    for (i = 1; 3 >= i; ++i) {
+      int64_t bytes_to_read = std::min(file_size - buffer_size, (int64_t)1024 * 1024);
+
+      if (0 == bytes_to_read)
+        break;
+
+      if (buf->get_size() < (buffer_size + bytes_to_read))
+        buf->resize(buffer_size + bytes_to_read);
+
+      int64_t bytes_read     = file.read(buf->get_buffer() + buffer_size, bytes_to_read);
+      buffer_size           += bytes_read;
+
+      std::string mime_type  = magic_buffer(m, buf->get_buffer(), buffer_size);
+
+      if (!mime_type.empty())
+        return mime_type;
+    }
+  } catch (...) {
+  }
+
+  return "";
+}
+#endif  // HAVE_MAGIC_H
+
+std::string
+guess_mime_type(std::string ext,
                 bool is_file) {
 #if HAVE_MAGIC_H
-  string ret;
+  std::string ret;
   magic_t m;
 
   if (!is_file)
@@ -2662,7 +2698,7 @@ guess_mime_type(string ext,
     return guess_mime_type_by_ext(ext);
 #endif
 
-  ret = magic_file(m, ext.c_str());
+  ret = guess_mime_type_by_content(m, ext);
   magic_close(m);
 
   if (ret == "")
@@ -2686,7 +2722,7 @@ guess_mime_type(string ext,
 }
 
 bool
-is_valid_cctld(const string &s) {
+is_valid_cctld(const std::string &s) {
   int i;
 
   for (i = 0; NULL != cctlds[i]; ++i)
