@@ -12,8 +12,8 @@
    Modified by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-#ifndef __MATROSKA_H
-#define __MATROSKA_H
+#ifndef __MTX_COMMON_MATROSKA_H
+#define __MTX_COMMON_MATROSKA_H
 
 // see http://www.matroska.org/technical/specs/codecid/index.html
 
@@ -36,6 +36,7 @@
 #define MKV_A_PCM_FLOAT  "A_PCM/FLOAT/IEEE"
 #define MKV_A_VORBIS     "A_VORBIS"
 #define MKV_A_ACM        "A_MS/ACM"
+#define MKV_A_QUICKTIME  "A_QUICKTIME"
 #define MKV_A_QDMC       "A_QUICKTIME/QDMC"
 #define MKV_A_QDMC2      "A_QUICKTIME/QDM2"
 #define MKV_A_REAL_14_4  "A_REAL/14_4"
@@ -44,6 +45,8 @@
 #define MKV_A_REAL_SIPR  "A_REAL/SIPR"
 #define MKV_A_REAL_ATRC  "A_REAL/ATRC"
 #define MKV_A_FLAC       "A_FLAC"
+#define MKV_A_MLP        "A_MLP"
+#define MKV_A_TRUEHD     "A_TRUEHD"
 #define MKV_A_TTA        "A_TTA1"
 #define MKV_A_WAVPACK4   "A_WAVPACK4"
 
@@ -71,6 +74,7 @@
 #define MKV_S_VOBSUB     "S_VOBSUB"
 #define MKV_S_VOBSUBZLIB "S_VOBSUB/ZLIB"
 #define MKV_S_KATE       "S_KATE"
+#define MKV_S_HDMV_PGS   "S_HDMV/PGS"
 
 #define MKV_B_VOBBTN     "B_VOBBTN"
 
@@ -129,4 +133,4 @@
 
 #define CHAPTER_PHYSEQUIV_INDEX        10
 
-#endif // __MATROSKA_H
+#endif // __MTX_COMMON_MATROSKA_H
