@@ -6,20 +6,17 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: ogmstreams.h 2711 2005-01-12 15:03:47Z mosu $
-
    definitions for the OGM file format
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#ifndef __OGGSTREAMS_H
-#define __OGGSTREAMS_H
+#ifndef __MTX_COMMON_OGGSTREAMS_H
+#define __MTX_COMMON_OGGSTREAMS_H
 
 /*
    Taken from http://tobias.everwicked.com/packfmt.htm
 
-
  First packet (header)
  ---------------------
 
@@ -30,7 +27,6 @@
  0x0001 | stream_header           | the size is indicated in the
         |                         | size member
 
-
  Second packet (comment)
  -----------------------
 
@@ -61,8 +57,6 @@
  0x0001+  | data                    | packet contents
  lenbytes |                         |
 
-
-
 */
 
 //// OggDS headers
