@@ -5,15 +5,13 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: os.h 3940 2008-09-26 14:58:46Z mosu $
-
    Cross platform compatibility definitions
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#ifndef __OS_H
-#define __OS_H
+#ifndef __MTX_COMMON_OS_H
+#define __MTX_COMMON_OS_H
 
 #include "config.h"
 
@@ -133,14 +131,4 @@ typedef UINT64_TYPE uint64_t;
 # define ARCH_LITTLEENDIAN
 #endif
 
-// MSVC doesn't provide vsscanf. So let's use our own frontend.
-#if !defined(HAVE_VSSCANF) || (HAVE_VSSCANF != 1)
-# include <stdio.h>
-int vsscanf(const char *, const char *, va_list);
-#endif // !HAVE_VSSCANF...
-
-int MTX_DLL_API fs_entry_exists(const char *path);
-void MTX_DLL_API create_directory(const char *path);
-int64_t MTX_DLL_API get_current_time_millis();
-
 #endif
