@@ -6,18 +6,16 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: segmentinfo_parser.cpp 3579 2007-08-19 08:53:32Z mosu $
-
    XML segment info parser functions
 
    Written by Steve Lhomme <steve.lhomme@free.fr> and
    Moritz Bunkus <moritz@bunkus.org>.
 */
 
+#include "common/os.h"
+
 #include <expat.h>
 #include <ctype.h>
-#include <setjmp.h>
-#include <stdarg.h>
 
 #include <string>
 
@@ -25,21 +23,17 @@
 #include <matroska/KaxInfoData.h>
 #include <matroska/KaxSegment.h>
 
-#include "commonebml.h"
-#include "error.h"
-#include "iso639.h"
-#include "mm_io.h"
-#include "segmentinfo.h"
-#include "xml_element_mapping.h"
-#include "xml_element_parser.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/error.h"
+#include "common/iso639.h"
+#include "common/mm_io.h"
+#include "common/segmentinfo.h"
+#include "common/xml/element_mapping.h"
+#include "common/xml/element_parser.h"
 
-using namespace std;
 using namespace libmatroska;
 
-// {{{ XML chapters
-
-#define CPDATA (parser_data_t *)pdata
-
 static void
 end_segmentinfo_data(void *) {
 /* TODO verify that the FamilyUID is set
@@ -72,11 +66,7 @@ end_segmentinfo_links(void *) {
 KaxInfo *
 parse_xml_segmentinfo(mm_text_io_c *in,
                       bool exception_on_error) {
-  KaxInfo *info;
-  EbmlMaster *m;
-  string error;
   int i;
-
   for (i = 0; segmentinfo_elements[i].name != NULL; i++) {
     segmentinfo_elements[i].start_hook = NULL;
     segmentinfo_elements[i].end_hook   = NULL;
@@ -86,27 +76,24 @@ parse_xml_segmentinfo(mm_text_io_c *in,
   segmentinfo_elements[chapter_element_map_index("FamilyUID")].end_hook = end_segmentinfo_family;
   segmentinfo_elements[chapter_element_map_index("Links")].end_hook     = end_segmentinfo_links;
 
+  std::string error;
+  KaxInfo *info = NULL;
   try {
-    m = parse_xml_elements("Info", segmentinfo_elements, in);
-    info = dynamic_cast<KaxInfo *>(sort_ebml_master(m));
-    assert(info != NULL);
+    EbmlMaster *m = parse_xml_elements("Info", segmentinfo_elements, in);
+    info          = dynamic_cast<KaxInfo *>(sort_ebml_master(m));
+    assert(NULL != info);
 
   } catch (error_c e) {
     if (!exception_on_error)
-      mxerror("%s", e.get_error().c_str());
+      mxerror(e.get_error());
     error = e.get_error();
-    info = NULL;
+    info  = NULL;
   }
 
-  if ((info != NULL) && (verbose > 1))
-    debug_dump_elements(info, 0);
-
-  if (error.length() > 0)
+  if (!error.empty())
     throw error_c(error);
 
   fix_mandatory_segmentinfo_elements(info);
 
   return info;
 }
-
-// }}}
