@@ -11,8 +11,9 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
+#include <algorithm>
 #include <boost/regex.hpp>
 #if HAVE_NL_LANGINFO
 # include <langinfo.h>
@@ -23,38 +24,49 @@
 # include <libintl.h>
 #endif
 #include <locale.h>
+#include <stdlib.h>
 
-#include "common.h"
-#include "locale_string.h"
-#include "translation.h"
+#include "common/common.h"
+#include "common/locale_string.h"
+#include "common/strings/editing.h"
+#include "common/translation.h"
 
 #if defined(SYS_WINDOWS)
 # include <windows.h>
-# include "os_windows.h"
+# include "common/fs_sys_helpers.h"
+# include "common/memory.h"
 #endif
 
 std::vector<translation_c> translation_c::ms_available_translations;
+int translation_c::ms_active_translation_idx = 0;
 
 translation_c::translation_c(const std::string &unix_locale,
                              const std::string &windows_locale,
                              const std::string &windows_locale_sysname,
                              const std::string &english_name,
-                             const std::string &translated_name)
+                             const std::string &translated_name,
+                             bool line_breaks_anywhere)
   : m_unix_locale(unix_locale)
   , m_windows_locale(windows_locale)
   , m_windows_locale_sysname(windows_locale_sysname)
   , m_english_name(english_name)
   , m_translated_name(translated_name)
+  , m_line_breaks_anywhere(line_breaks_anywhere)
 {
 }
 
 void
 translation_c::initialize_available_translations() {
   ms_available_translations.clear();
+  ms_available_translations.push_back(translation_c("en_US", "en",    "english",  "English",             "English",  false));
 #if defined(HAVE_LIBINTL_H)
-  ms_available_translations.push_back(translation_c("en_US", "en", "english", "English", "English"));
-  ms_available_translations.push_back(translation_c("de_DE", "de", "german",  "German",  "Deutsch"));
+  ms_available_translations.push_back(translation_c("de_DE", "de",    "german",   "German",              "Deutsch",  false));
+  ms_available_translations.push_back(translation_c("ja_JP", "ja",    "japanese", "Japanese",            "日本語",   true));
+  ms_available_translations.push_back(translation_c("zh_CN", "zh_CN", "chinese",  "Chinese Simplified",  "简体中文", true));
+  ms_available_translations.push_back(translation_c("zh_TW", "zh_TW", "chinese",  "Chinese Traditional", "繁體中文", true));
 #endif
+
+  ms_active_translation_idx = 0;
 }
 
 int
@@ -87,6 +99,14 @@ translation_c::get_default_ui_locale() {
 
 #if defined(HAVE_LIBINTL_H)
 # if defined(SYS_WINDOWS)
+  std::string env_var = get_environment_variable("LC_MESSAGES");
+  if (!env_var.empty() && (-1 != look_up_translation(env_var)))
+    return env_var;
+
+  env_var = get_environment_variable("LANG");
+  if (!env_var.empty() && (-1 != look_up_translation(env_var)))
+    return env_var;
+
   char *data;
   int len = GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SENGLANGUAGE, NULL, 0);
   if (0 < len) {
@@ -106,21 +126,24 @@ translation_c::get_default_ui_locale() {
   char *data = setlocale(LC_MESSAGES, NULL);
   if (NULL != data) {
     std::string previous_locale = data;
+    if (debugging_requested("locale"))
+      mxinfo(boost::format("[get_default_ui_locale previous %1%]\n") % previous_locale);
     setlocale(LC_MESSAGES, "");
     data = setlocale(LC_MESSAGES, NULL);
 
     if (NULL != data)
       locale = data;
 
+    if (debugging_requested("locale"))
+      mxinfo(boost::format("[get_default_ui_locale new %1%]\n") % locale);
+
     setlocale(LC_MESSAGES, previous_locale.c_str());
-  }
+  } else if (debugging_requested("locale"))
+    mxinfo(boost::format("[get_default_ui_locale get previous failed]\n"));
 
 # endif // SYS_WINDOWS
 #endif  // HAVE_LIBINTL_H
 
-  if (!locale.empty() && (-1 == look_up_translation(locale)))
-    locale = "";
-
   return locale;
 }
 
@@ -133,28 +156,79 @@ translation_c::get_locale() {
 #endif
 }
 
+translation_c &
+translation_c::get_active_translation() {
+  return ms_available_translations[ms_active_translation_idx];
+}
+
+void
+translation_c::set_active_translation(const std::string &locale) {
+  int idx                   = look_up_translation(locale);
+  ms_active_translation_idx = std::max(idx, 0);
+
+  if (debugging_requested("locale"))
+    mxinfo(boost::format("[translation_c::set_active_translation() active_translation_idx %1% for locale %2%]\n") % ms_active_translation_idx % locale);
+}
+
+// ------------------------------------------------------------
+
+translatable_string_c::translatable_string_c()
+{
+}
+
+translatable_string_c::translatable_string_c(const std::string &untranslated_string)
+  : m_untranslated_string(untranslated_string)
+{
+}
+
+translatable_string_c::translatable_string_c(const char *untranslated_string)
+  : m_untranslated_string(untranslated_string)
+{
+}
+
+std::string
+translatable_string_c::get_translated()
+  const
+{
+  return m_untranslated_string.empty() ? "" : Y(m_untranslated_string.c_str());
+}
+
+std::string
+translatable_string_c::get_untranslated()
+  const
+{
+  return m_untranslated_string;
+}
+
+// ------------------------------------------------------------
+
 #if defined(HAVE_LIBINTL_H)
 
 void
 init_locales(std::string locale) {
   translation_c::initialize_available_translations();
 
-  mxverb(4, boost::format("[init_locales start: locale %1%]\n") % locale);
+  if (debugging_requested("locale"))
+    mxinfo(boost::format("[init_locales start: locale %1%]\n") % locale);
 
   std::string locale_dir;
   std::string default_locale = translation_c::get_default_ui_locale();
 
   if (-1 == translation_c::look_up_translation(locale)) {
-    mxverb(4, boost::format("[init_locales lookup failed; clearing locale]\n"));
+    if (debugging_requested("locale"))
+      mxinfo(boost::format("[init_locales lookup failed; clearing locale]\n"));
     locale = "";
   }
 
   if (locale.empty()) {
     locale = default_locale;
-    mxverb(4, boost::format("[init_locales setting to default locale %1%]\n") % locale);
+    if (debugging_requested("locale"))
+      mxinfo(boost::format("[init_locales setting to default locale %1%]\n") % locale);
   }
 
 # if defined(SYS_WINDOWS)
+  set_environment_variable("LANGUAGE", "");
+
   if (!locale.empty()) {
     // The Windows system headers define LC_MESSAGES but
     // Windows itself doesn't know it and treats "set_locale(LC_MESSAGES, ...)"
@@ -166,6 +240,8 @@ init_locales(std::string locale) {
 
     set_environment_variable("LANG",        locale);
     set_environment_variable("LC_MESSAGES", locale);
+
+    translation_c::set_active_translation(locale);
   }
 
   locale_dir = get_installation_path() + "\\locale";
@@ -190,17 +266,27 @@ init_locales(std::string locale) {
     }
 
   } catch (locale_string_format_error_c &error) {
-    mxverb(4, boost::format("[init_locales format error in %1%]\n") % error.m_format);
+    if (debugging_requested("locale"))
+      mxinfo(boost::format("[init_locales format error in %1%]\n") % error.m_format);
   }
 
-  mxverb(4, boost::format("[init_locales chosen locale %1%]\n") % chosen_locale);
+  if (debugging_requested("locale"))
+    mxinfo(boost::format("[init_locales chosen locale %1%]\n") % chosen_locale);
 
   if (chosen_locale.empty())
     mxerror(Y("The locale could not be set properly. Check the LANG, LC_ALL and LC_MESSAGES environment variables.\n"));
 
+  translation_c::set_active_translation(chosen_locale);
+
   locale_dir = MTX_LOCALE_DIR;
 # endif  // SYS_WINDOWS
 
+# if defined(SYS_APPLE)
+  int result = setenv("LC_MESSAGES", chosen_locale.c_str(), 1);
+  if (debugging_requested("locale"))
+    mxinfo(boost::format("[init_locales setenv() return code: %1%]\n") % result);
+# endif
+
   bindtextdomain("mkvtoolnix", locale_dir.c_str());
   textdomain("mkvtoolnix");
   bind_textdomain_codeset("mkvtoolnix", "UTF-8");
@@ -210,6 +296,7 @@ init_locales(std::string locale) {
 
 void
 init_locales(std::string) {
+  translation_c::initialize_available_translations();
 }
 
 #endif  // HAVE_LIBINTL_H
