@@ -10,7 +10,7 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <cassert>
 #include <iostream>
@@ -31,18 +31,16 @@ extern "C" {
 #include <matroska/KaxAttachments.h>
 #include <matroska/KaxSegment.h>
 
-#include "common.h"
-#include "commonebml.h"
-#include "mkvextract.h"
-#include "mm_io.h"
-#include "quickparser.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/kax_analyzer.h"
+#include "extract/mkvextract.h"
 
 using namespace libmatroska;
-using namespace std;
 
 static void
 handle_attachments(KaxAttachments *atts,
-                   vector<track_spec_t> &tracks) {
+                   std::vector<track_spec_t> &tracks) {
   static int64_t attachment_ui_id = 0;
 
   int i;
@@ -50,7 +48,7 @@ handle_attachments(KaxAttachments *atts,
     KaxAttached  *att = dynamic_cast<KaxAttached *>((*atts)[i]);
     assert(NULL != att);
 
-    string name, type;
+    std::string name, type;
     int64_t size       = -1;
     int64_t id         = -1;
     KaxFileData *fdata = NULL;
@@ -60,10 +58,10 @@ handle_attachments(KaxAttachments *atts,
       EbmlElement *e = (*att)[k];
 
       if (EbmlId(*e) == KaxFileName::ClassInfos.GlobalId)
-        name = UTFstring_to_cstr(UTFstring(*static_cast<KaxFileName *>(e)));
+        name = UTFstring_to_cstrutf8(UTFstring(*static_cast<KaxFileName *>(e)));
 
       else if (EbmlId(*e) == KaxMimeType::ClassInfos.GlobalId)
-        type = string(*static_cast<KaxMimeType *>(e));
+        type = std::string(*static_cast<KaxMimeType *>(e));
 
       else if (EbmlId(*e) == KaxFileUID::ClassInfos.GlobalId)
         id = uint32(*static_cast<KaxFileUID *>(e));
@@ -88,10 +86,8 @@ handle_attachments(KaxAttachments *atts,
 
       if (found && !tracks[k].done) {
         // check for output name
-        if (strlen(tracks[k].out_name) == 0) {
-          safefree(tracks[k].out_name);
-          tracks[k].out_name = safestrdup(name.c_str());
-        }
+        if (tracks[k].out_name.empty())
+          tracks[k].out_name = name;
 
         mxinfo(boost::format(Y("The attachment #%1%, ID %2%, MIME type %3%, size %4%, is written to '%5%'.\n")) % attachment_ui_id % id % type % size % tracks[k].out_name);
         mm_io_c *out = NULL;
@@ -109,30 +105,27 @@ handle_attachments(KaxAttachments *atts,
 }
 
 void
-extract_attachments(const char *file_name,
-                    vector<track_spec_t> &tracks,
-                    bool parse_fully) {
-  mm_io_c *in;
-  kax_quickparser_c *qp;
+extract_attachments(const std::string &file_name,
+                    std::vector<track_spec_t> &tracks,
+                    kax_analyzer_c::parse_mode_e parse_mode) {
+  kax_analyzer_cptr analyzer;
 
   // open input file
   try {
-    in = new mm_file_io_c(file_name);
-    qp = new kax_quickparser_c(*in, parse_fully);
+    analyzer = kax_analyzer_cptr(new kax_analyzer_c(file_name));
+    if (!analyzer->process(parse_mode, MODE_READ))
+      throw false;
   } catch (...) {
     show_error(boost::format(Y("The file '%1%' could not be opened for reading (%2%).")) % file_name % strerror(errno));
     return;
   }
 
-  KaxAttachments *attachments = dynamic_cast<KaxAttachments *>(qp->read_all(KaxAttachments::ClassInfos));
-  if (attachments != NULL) {
+  KaxAttachments *attachments = dynamic_cast<KaxAttachments *>(analyzer->read_all(KaxAttachments::ClassInfos));
+  if (NULL != attachments) {
     handle_attachments(attachments, tracks);
     delete attachments;
   }
 
-  delete in;
-  delete qp;
-
   int i;
   for (i = 0; i < tracks.size(); i++)
     if (!tracks[i].done)
