@@ -5,14 +5,12 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: tracks.cpp 3986 2008-10-11 16:46:03Z mosu $
-
    extracts tracks from Matroska files into other files
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <cassert>
 #include <string>
@@ -36,24 +34,22 @@
 #include <matroska/KaxTrackAudio.h>
 #include <matroska/KaxTrackVideo.h>
 
-#include "common.h"
-#include "commonebml.h"
-#include "matroska.h"
-#include "mkvextract.h"
-#include "mm_io.h"
-
-#include "xtr_base.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/matroska.h"
+#include "common/mm_io.h"
+#include "extract/mkvextract.h"
+#include "extract/xtr_base.h"
 
 using namespace libmatroska;
-using namespace std;
 
-static vector<xtr_base_c *> extractors;
+static std::vector<xtr_base_c *> extractors;
 
 // ------------------------------------------------------------------------
 
 static void
 create_extractors(KaxTracks &kax_tracks,
-                  vector<track_spec_t> &tracks) {
+                  std::vector<track_spec_t> &tracks) {
   int i;
 
   for (i = 0; i < kax_tracks.ListSize(); i++) {
@@ -90,7 +86,7 @@ create_extractors(KaxTracks &kax_tracks,
       continue;
 
     // Let's find the codec ID and create an extractor for it.
-    string codec_id = kt_get_codec_id(track);
+    std::string codec_id = kt_get_codec_id(track);
     if (codec_id.empty())
       mxerror(boost::format(Y("The track number %1% does not have a valid CodecID.\n")) % tnum);
 
@@ -251,25 +247,25 @@ close_extractors() {
 static void
 write_all_cuesheets(KaxChapters &chapters,
                     KaxTags &tags,
-                    vector<track_spec_t> &tspecs) {
+                    std::vector<track_spec_t> &tspecs) {
   int i;
   mm_io_c *out = NULL;
 
   for (i = 0; i < tspecs.size(); i++) {
     if (tspecs[i].extract_cuesheet) {
-      string file_name = tspecs[i].out_name;
-      int pos          = file_name.rfind('/');
-      int pos2         = file_name.rfind('\\');
+      std::string file_name = tspecs[i].out_name;
+      int pos               = file_name.rfind('/');
+      int pos2              = file_name.rfind('\\');
 
       if (pos2 > pos)
         pos = pos2;
       if (pos >= 0)
         file_name.erase(0, pos2);
 
-      string cue_file_name = (string)tspecs[i].out_name;
-      pos                  = cue_file_name.rfind('.');
-      pos2                 = cue_file_name.rfind('/');
-      int pos3             = cue_file_name.rfind('\\');
+      std::string cue_file_name = tspecs[i].out_name;
+      pos                       = cue_file_name.rfind('.');
+      pos2                      = cue_file_name.rfind('/');
+      int pos3                  = cue_file_name.rfind('\\');
 
       if ((0 <= pos) && (pos > pos2) && (pos > pos3))
         cue_file_name.erase(pos);
@@ -282,7 +278,7 @@ write_all_cuesheets(KaxChapters &chapters,
       }
 
       mxinfo(boost::format(Y("The CUE sheet for track %1% will be written to '%2%'.\n")) % tspecs[i].tid % cue_file_name);
-      write_cuesheet(file_name.c_str(), chapters, tags, tspecs[i].tuid, *out);
+      write_cuesheet(file_name, chapters, tags, tspecs[i].tuid, *out);
       delete out;
     }
   }
@@ -290,7 +286,7 @@ write_all_cuesheets(KaxChapters &chapters,
 
 static void
 find_track_uids(KaxTracks &tracks,
-                vector<track_spec_t> &tspecs) {
+                std::vector<track_spec_t> &tspecs) {
   int t;
 
   for (t = 0; t < tracks.ListSize(); t++) {
@@ -310,12 +306,12 @@ find_track_uids(KaxTracks &tracks,
 }
 
 bool
-extract_tracks(const char *file_name,
-               vector<track_spec_t> &tspecs) {
+extract_tracks(const std::string &file_name,
+               std::vector<track_spec_t> &tspecs) {
   // open input file
   mm_io_c *in;
   try {
-    in = new mm_file_io_c(file_name);
+    in = new mm_file_io_c(file_name, MODE_READ);
   } catch (...) {
     show_error(boost::format(Y("The file '%1%' could not be opened for reading (%2%).\n")) % file_name % strerror(errno));
     return false;
@@ -422,7 +418,7 @@ extract_tracks(const char *file_name,
         cluster = (KaxCluster *)l1;
 
         if (0 == verbose)
-          mxinfo(boost::format(Y("Progress: %1%%%\r")) % (int)(in->getFilePointer() * 100 / file_size));
+          mxinfo(boost::format(Y("Progress: %1%%%%2%")) % (int)(in->getFilePointer() * 100 / file_size) % "\r");
 
         upper_lvl_el = 0;
         l2           = es->FindNextElement(l1->Generic().Context, upper_lvl_el, 0xFFFFFFFFL, true, 1);
