@@ -10,34 +10,34 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <matroska/KaxTracks.h>
 #include <matroska/KaxTrackEntryData.h>
 #include <matroska/KaxTrackAudio.h>
 #include <matroska/KaxTrackVideo.h>
 
-#include "common.h"
-#include "commonebml.h"
-#include "matroska.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/matroska.h"
+#include "common/strings/editing.h"
+#include "extract/xtr_aac.h"
+#include "extract/xtr_avc.h"
+#include "extract/xtr_avi.h"
+#include "extract/xtr_base.h"
+#include "extract/xtr_cpic.h"
+#include "extract/xtr_mpeg1_2.h"
+#include "extract/xtr_ogg.h"
+#include "extract/xtr_pgs.h"
+#include "extract/xtr_rmff.h"
+#include "extract/xtr_textsubs.h"
+#include "extract/xtr_tta.h"
+#include "extract/xtr_vobsub.h"
+#include "extract/xtr_wav.h"
 
-#include "xtr_aac.h"
-#include "xtr_avc.h"
-#include "xtr_avi.h"
-#include "xtr_base.h"
-#include "xtr_cpic.h"
-#include "xtr_mpeg1_2.h"
-#include "xtr_ogg.h"
-#include "xtr_rmff.h"
-#include "xtr_textsubs.h"
-#include "xtr_tta.h"
-#include "xtr_vobsub.h"
-#include "xtr_wav.h"
-
-using namespace std;
 using namespace libmatroska;
 
-xtr_base_c::xtr_base_c(const string &codec_id,
+xtr_base_c::xtr_base_c(const std::string &codec_id,
                        int64_t tid,
                        track_spec_t &tspec,
                        const char *container_name)
@@ -86,7 +86,7 @@ xtr_base_c::handle_frame(memory_cptr &frame,
                          bool discardable,
                          bool references_valid) {
   m_content_decoder.reverse(frame, CONTENT_ENCODING_SCOPE_BLOCK);
-  m_out->write(frame->get(), frame->get_size());
+  m_out->write(frame);
   m_bytes_written += frame->get_size();
 }
 
@@ -122,14 +122,15 @@ xtr_base_c::init_content_decoder(KaxTrackEntry &track) {
 }
 
 xtr_base_c *
-xtr_base_c::create_extractor(const string &new_codec_id,
+xtr_base_c::create_extractor(const std::string &new_codec_id,
                              int64_t new_tid,
                              track_spec_t &tspec) {
   // Raw format
-  if (1 == tspec.extract_raw)
+  if (track_spec_t::tm_raw == tspec.target_mode)
     return new xtr_base_c(new_codec_id, new_tid, tspec);
-  else if (2 == tspec.extract_raw)
+  else if (track_spec_t::tm_full_raw == tspec.target_mode)
     return new xtr_fullraw_c(new_codec_id, new_tid, tspec);
+
   // Audio formats
   else if (new_codec_id == MKV_A_AC3)
     return new xtr_base_c(new_codec_id, new_tid, tspec, "Dolby Digital (AC3)");
@@ -146,12 +147,16 @@ xtr_base_c::create_extractor(const string &new_codec_id,
       return new xtr_oggflac_c(new_codec_id, new_tid, tspec);
     else
       return new xtr_flac_c(new_codec_id, new_tid, tspec);
-  } else if (new_codec_id == MKV_A_VORBIS)
+  }  else if (new_codec_id == MKV_A_VORBIS)
     return new xtr_oggvorbis_c(new_codec_id, new_tid, tspec);
   else if (starts_with_case(new_codec_id, "A_AAC"))
     return new xtr_aac_c(new_codec_id, new_tid, tspec);
   else if (starts_with_case(new_codec_id, "A_REAL/"))
     return new xtr_rmff_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_A_MLP)
+    return new xtr_base_c(new_codec_id, new_tid, tspec, "MLP");
+  else if (new_codec_id == MKV_A_TRUEHD)
+    return new xtr_base_c(new_codec_id, new_tid, tspec, "TrueHD");
   else if (new_codec_id == MKV_A_TTA)
     return new xtr_tta_c(new_codec_id, new_tid, tspec);
   else if (new_codec_id == MKV_A_WAVPACK4)
@@ -182,6 +187,8 @@ xtr_base_c::create_extractor(const string &new_codec_id,
     return new xtr_cpic_c(new_codec_id, new_tid, tspec);
   else if (new_codec_id == MKV_S_KATE)
     return new xtr_oggkate_c(new_codec_id, new_tid, tspec);
+  else if (new_codec_id == MKV_S_HDMV_PGS)
+    return new xtr_pgs_c(new_codec_id, new_tid, tspec);
 
   return NULL;
 }
@@ -196,12 +203,12 @@ xtr_fullraw_c::create_file(xtr_base_c *master,
   if ((NULL != priv) && (0 != priv->GetSize())) {
     memory_cptr mem(new memory_c(priv->GetBuffer(), priv->GetSize(), false));
     m_content_decoder.reverse(mem, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
-    m_out->write(mem->get(), mem->get_size());
+    m_out->write(mem);
   }
 }
 
 void
 xtr_fullraw_c::handle_codec_state(memory_cptr &codec_state) {
   m_content_decoder.reverse(codec_state, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
-  m_out->write(codec_state->get(), codec_state->get_size());
+  m_out->write(codec_state);
 }
