@@ -6,14 +6,12 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_avc.cpp 3854 2008-08-24 18:41:17Z mosu $
-
    AVC/h2.64 ES demultiplexer module
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -25,14 +23,14 @@
 #include <list>
 #include <memory>
 
-#include "byte_buffer.h"
-#include "common.h"
-#include "error.h"
-#include "output_control.h"
-#include "r_avc.h"
-#include "p_avc.h"
+#include "common/byte_buffer.h"
+#include "common/common.h"
+#include "common/error.h"
+#include "common/memory.h"
+#include "input/r_avc.h"
+#include "merge/output_control.h"
+#include "output/p_avc.h"
 
-using namespace std;
 
 #define PROBESIZE 4
 #define READ_SIZE 1024 * 1024
@@ -47,7 +45,7 @@ avc_es_reader_c::probe_file(mm_io_c *io,
     if (PROBESIZE > size)
       return 0;
 
-    memory_cptr buf(new memory_c((unsigned char *)safemalloc(READ_SIZE), READ_SIZE));
+    memory_cptr buf = memory_c::alloc(READ_SIZE);
     int num_read, i;
     bool first = true;
 
@@ -58,37 +56,37 @@ avc_es_reader_c::probe_file(mm_io_c *io,
 
     io->setFilePointer(0, seek_beginning);
     for (i = 0; MAX_PROBE_BUFFERS > i; ++i) {
-      num_read = io->read(buf->get(), READ_SIZE);
+      num_read = io->read(buf->get_buffer(), READ_SIZE);
       if (4 > num_read)
         return 0;
 
       // MPEG TS starts with 0x47.
-      if (first && (0x47 == buf->get()[0]))
+      if (first && (0x47 == buf->get_buffer()[0]))
         return 0;
       first = false;
 
-      parser.add_bytes(buf->get(), num_read);
+      parser.add_bytes(buf->get_buffer(), num_read);
 
       if (parser.headers_parsed())
         return 1;
     }
 
   } catch (error_c &err) {
-    mxinfo("err %s\n", err.get_error().c_str());
+    mxinfo(boost::format(Y("Error %1%\n")) % err.get_error());
 
   } catch (...) {
-    mxinfo("have an xcptn\n");
+    mxinfo(Y("have an xcptn\n"));
   }
 
   return 0;
 }
 
 avc_es_reader_c::avc_es_reader_c(track_info_c &n_ti)
-  throw (error_c):
-  generic_reader_c(n_ti),
-  m_bytes_processed(0),
-  m_buffer(new memory_c((unsigned char *)safemalloc(READ_SIZE), READ_SIZE)) {
-
+  throw (error_c)
+  : generic_reader_c(n_ti)
+  , m_bytes_processed(0)
+  , m_buffer(memory_c::alloc(READ_SIZE))
+{
   try {
     m_io   = counted_ptr<mm_io_c>(new mm_file_io_c(ti.fname));
     m_size = m_io->get_size();
@@ -105,10 +103,10 @@ avc_es_reader_c::avc_es_reader_c(track_info_c &n_ti)
     int num_read, i;
 
     for (i = 0; MAX_PROBE_BUFFERS > i; ++i) {
-      num_read = m_io->read(m_buffer->get(), READ_SIZE);
+      num_read = m_io->read(m_buffer->get_buffer(), READ_SIZE);
       if (0 == num_read)
-        throw error_c("avc_es_reader: Should not have happened.");
-      parser.add_bytes(m_buffer->get(), num_read);
+        throw error_c(Y("avc_es_reader: Should not have happened."));
+      parser.add_bytes(m_buffer->get_buffer(), num_read);
       if (parser.headers_parsed())
         break;
     }
@@ -123,11 +121,11 @@ avc_es_reader_c::avc_es_reader_c(track_info_c &n_ti)
     m_io->setFilePointer(0, seek_beginning);
 
   } catch (...) {
-    throw error_c("avc_es_reader: Could not open the source file.");
+    throw error_c(Y("avc_es_reader: Could not open the source file."));
   }
 
   if (verbose)
-    mxinfo(FMT_FN "Using the AVC/h.264 ES demultiplexer.\n", ti.fname.c_str());
+    mxinfo_fn(ti.fname, Y("Using the AVC/h.264 ES demultiplexer.\n"));
 }
 
 void
@@ -135,9 +133,9 @@ avc_es_reader_c::create_packetizer(int64_t) {
   if (NPTZR() != 0)
     return;
 
-  add_packetizer(new mpeg4_p10_es_video_packetizer_c(this, m_avcc, m_width, m_height, ti));
+  add_packetizer(new mpeg4_p10_es_video_packetizer_c(this, ti, m_avcc, m_width, m_height));
 
-  mxinfo(FMT_TID "Using the MPEG-4 part 10 ES video output module.\n", ti.fname.c_str(), (int64_t)0);
+  mxinfo_tid(ti.fname, 0, Y("Using the MPEG-4 part 10 ES video output module.\n"));
 }
 
 file_status_e
@@ -145,16 +143,14 @@ avc_es_reader_c::read(generic_packetizer_c *,
                       bool) {
   int num_read;
 
-//   mxinfo("read proc " LLD " size " LLD "\n", m_bytes_processed, m_size);
-
   if (m_bytes_processed >= m_size)
     return FILE_STATUS_DONE;
 
-  num_read = m_io->read(m_buffer->get(), READ_SIZE);
+  num_read = m_io->read(m_buffer->get_buffer(), READ_SIZE);
   if (0 >= num_read)
     return FILE_STATUS_DONE;
 
-  PTZR0->process(new packet_t(new memory_c(m_buffer->get(), num_read)));
+  PTZR0->process(new packet_t(new memory_c(m_buffer->get_buffer(), num_read)));
 
   m_bytes_processed += num_read;
 
