@@ -6,35 +6,29 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_corepicture.cpp 3861 2008-08-27 11:27:04Z mosu $
-
    CorePanorama video reader
 
    Written by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-#include "os.h"
-
-#include <errno.h>
-#include <expat.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
+#include "common/os.h"
 
 #include <algorithm>
 
-#include "matroska.h"
-#include "mm_io.h"
-#include "output_control.h"
-#include "pr_generic.h"
-#include "p_video.h"
-#include "r_corepicture.h"
+#include "common/endian.h"
+#include "common/matroska.h"
+#include "common/mm_io.h"
+#include "common/strings/parsing.h"
+#include "common/xml/xml.h"
+#include "input/r_corepicture.h"
+#include "merge/output_control.h"
+#include "merge/pr_generic.h"
+#include "output/p_video.h"
 
-using namespace std;
 
 class corepicture_xml_find_root_c: public xml_parser_c {
 public:
-  string m_root_element;
+  std::string m_root_element;
 
 public:
   corepicture_xml_find_root_c(mm_text_io_c *io):
@@ -75,7 +69,7 @@ corepicture_reader_c::corepicture_reader_c(track_info_c &_ti)
     m_xml_source = new mm_text_io_c(new mm_file_io_c(ti.fname));
 
     if (!corepicture_reader_c::probe_file(m_xml_source, 0))
-      throw error_c("corepicture_reader: Source is not a valid CorePanorama file.");
+      throw error_c(Y("corepicture_reader: Source is not a valid CorePanorama file."));
 
     parse_xml_file();
 
@@ -88,11 +82,11 @@ corepicture_reader_c::corepicture_reader_c(track_info_c &_ti)
     throw error_c(error.get_error());
 
   } catch (mm_io_error_c &error) {
-    throw error_c("corepicture_reader: Could not open the source file.");
+    throw error_c(Y("corepicture_reader: Could not open the source file."));
   }
 
   if (verbose)
-    mxinfo(FMT_FN "Using the CorePanorama subtitle reader.\n", ti.fname.c_str());
+    mxinfo_fn(ti.fname, Y("Using the CorePanorama subtitle reader.\n"));
 }
 
 corepicture_reader_c::~corepicture_reader_c() {
@@ -102,7 +96,7 @@ void
 corepicture_reader_c::start_element_cb(const char *name,
                                        const char **atts) {
   size_t i;
-  string node;
+  std::string node;
 
   m_parents.push_back(name);
 
@@ -141,7 +135,7 @@ corepicture_reader_c::start_element_cb(const char *name,
           new_picture.m_pic_type = COREPICTURE_TYPE_PNG;
 
         else
-          mxwarn(FMT_TID "The picture type '%s' is not recognized.\n", ti.fname.c_str(), (int64_t)0, atts[i + 1]);
+          mxwarn_tid(ti.fname, 0, boost::format(Y("The picture type '%1%' is not recognized.\n")) % atts[i + 1]);
 
       } else if (!strcasecmp(atts[i], "panorama") && (0 != atts[i + 1][0])) {
         if (!strcasecmp(atts[i + 1], "flat"))
@@ -157,7 +151,7 @@ corepicture_reader_c::start_element_cb(const char *name,
           new_picture.m_pan_type = COREPICTURE_PAN_SPHERICAL;
 
         else
-          mxwarn(FMT_TID "The panoramic mode '%s' is not recognized.\n", ti.fname.c_str(), (int64_t)0, atts[i + 1]);
+          mxwarn_tid(ti.fname, 0, boost::format(Y("The panoramic mode '%1%' is not recognized.\n")) % atts[i + 1]);
 
       } else if (!strcasecmp(atts[i], "url") && (0 != atts[i + 1][0]))
         new_picture.m_url = escape_xml(atts[i + 1]);
@@ -171,7 +165,7 @@ corepicture_reader_c::start_element_cb(const char *name,
 void
 corepicture_reader_c::end_element_cb(const char *name) {
   size_t i;
-  string node;
+  std::string node;
 
   // Generate the full path to this node.
   for (i = 0; m_parents.size() > i; ++i) {
@@ -187,7 +181,7 @@ void
 corepicture_reader_c::create_packetizer(int64_t tid) {
   uint8 private_buffer[5];
   uint32 codec_used = 0;
-  vector<corepicture_pic_t>::const_iterator picture;
+  std::vector<corepicture_pic_t>::const_iterator picture;
 
   private_buffer[0] = 0; // version 0
 
@@ -201,7 +195,7 @@ corepicture_reader_c::create_packetizer(int64_t tid) {
 
   ti.private_data = (unsigned char *)safememdup(private_buffer, sizeof(private_buffer));
   ti.private_size = sizeof(private_buffer);
-  m_ptzr          = add_packetizer(new video_packetizer_c(this, MKV_V_COREPICTURE, 0.0, m_width, m_height, ti));
+  m_ptzr          = add_packetizer(new video_packetizer_c(this, ti, MKV_V_COREPICTURE, 0.0, m_width, m_height));
 }
 
 file_status_e
@@ -210,7 +204,7 @@ corepicture_reader_c::read(generic_packetizer_c *ptzr,
 
   if (m_current_picture != m_pictures.end()) {
     try {
-      auto_ptr<mm_io_c> io(new mm_file_io_c(m_current_picture->m_url));
+      counted_ptr<mm_io_c> io(new mm_file_io_c(m_current_picture->m_url));
       uint64_t size  = io->get_size();
       binary *buffer = (binary *)safemalloc(7 + size);
 
@@ -226,7 +220,7 @@ corepicture_reader_c::read(generic_packetizer_c *ptzr,
       }
 
     } catch(...) {
-      mxerror(FMT_TID "Impossible to use file '%s': The file could not be opened for reading.\n", ti.fname.c_str(), (int64_t)0, m_current_picture->m_url.c_str());
+      mxerror_tid(ti.fname, 0, boost::format(Y("Impossible to use file '%1%': The file could not be opened for reading.\n")) % m_current_picture->m_url);
     }
     m_current_picture++;
   }
@@ -243,7 +237,7 @@ int
 corepicture_reader_c::get_progress() {
   if (m_pictures.size() == 0)
     return 0;
-  return 100 - distance(m_current_picture, (vector<corepicture_pic_t>::const_iterator)m_pictures.end()) * 100 / m_pictures.size();
+  return 100 - distance(m_current_picture, (std::vector<corepicture_pic_t>::const_iterator)m_pictures.end()) * 100 / m_pictures.size();
 }
 
 int64_t
@@ -251,7 +245,7 @@ corepicture_reader_c::try_to_parse_timecode(const char *s) {
   int64_t timecode;
 
   if (!parse_timecode(s, timecode))
-    throw xml_parser_error_c("Invalid start timecode", m_xml_parser);
+    throw xml_parser_error_c(Y("Invalid start timecode"), m_xml_parser);
 
   return timecode;
 }
