@@ -12,16 +12,7 @@
    Modified by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-// {{{ includes
-
-#include "os.h"
-
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include <typeinfo>
+#include "common/os.h"
 
 extern "C" {                    // for BITMAPINFOHEADER
 #include "avilib.h"
@@ -51,37 +42,44 @@ extern "C" {                    // for BITMAPINFOHEADER
 #include <matroska/KaxTrackAudio.h>
 #include <matroska/KaxTrackVideo.h>
 
-#include "chapters.h"
-#include "common.h"
-#include "commonebml.h"
-#include "hacks.h"
-#include "matroska.h"
-#include "output_control.h"
-#include "mm_io.h"
-#include "pr_generic.h"
-#include "r_matroska.h"
-
-#include "p_aac.h"
-#include "p_ac3.h"
-#include "p_avc.h"
-#include "p_dirac.h"
-#include "p_dts.h"
+#include "common/chapters/chapters.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/endian.h"
+#include "common/hacks.h"
+#include "common/math.h"
+#include "common/matroska.h"
+#include "common/mm_io.h"
+#include "common/strings/formatting.h"
+#include "common/strings/parsing.h"
+#include "common/tags/tags.h"
+#include "input/r_matroska.h"
+#include "merge/output_control.h"
+#include "merge/pr_generic.h"
+#include "output/p_aac.h"
+#include "output/p_ac3.h"
+#include "output/p_avc.h"
+#include "output/p_dirac.h"
+#include "output/p_dts.h"
 #if defined(HAVE_FLAC_FORMAT_H)
-#include "p_flac.h"
+# include "output/p_flac.h"
 #endif
-#include "p_mp3.h"
-#include "p_passthrough.h"
-#include "p_pcm.h"
-#include "p_textsubs.h"
-#include "p_tta.h"
-#include "p_video.h"
-#include "p_vobsub.h"
-#include "p_vobbtn.h"
-#include "p_vorbis.h"
-#include "p_wavpack.h"
-#include "p_kate.h"
+#include "output/p_kate.h"
+#include "output/p_mp3.h"
+#include "output/p_mpeg1_2.h"
+#include "output/p_mpeg4_p2.h"
+#include "output/p_mpeg4_p10.h"
+#include "output/p_passthrough.h"
+#include "output/p_pcm.h"
+#include "output/p_textsubs.h"
+#include "output/p_theora.h"
+#include "output/p_tta.h"
+#include "output/p_video.h"
+#include "output/p_vobbtn.h"
+#include "output/p_vobsub.h"
+#include "output/p_vorbis.h"
+#include "output/p_wavpack.h"
 
-using namespace std;
 using namespace libmatroska;
 
 #define MAP_TRACK_TYPE(c) (  (c) == 'a' ? track_audio   \
@@ -97,14 +95,34 @@ using namespace libmatroska;
 #define in_parent(p) \
   (!p->IsFiniteSize() || (in->getFilePointer() < (p->GetElementPosition() + p->HeadSize() + p->GetSize())))
 
-// }}}
-
 #define is_ebmlvoid(e) (EbmlId(*e) == EbmlVoid::ClassInfos.GlobalId)
 
-// {{{ FUNCTION kax_reader::probe_file()
-
 #define MAGIC_MKV 0x1a45dfa3
 
+void
+kax_track_t::handle_packetizer_display_dimensions() {
+  // If user hasn't set an aspect ratio via the command line and the
+  // source file provides display width/height paramaters then use
+  // these and signal the packetizer not to extract the dimensions
+  // from the bitstream.
+  if ((0 == v_dwidth) || (0 == v_dheight))
+    return;
+
+  ptzr_ptr->set_video_display_dimensions(v_dwidth, v_dheight, PARAMETER_SOURCE_CONTAINER);
+}
+
+void
+kax_track_t::handle_packetizer_pixel_cropping() {
+  if ((0 < v_pcleft) || (0 < v_pctop) || (0 < v_pcright) || (0 < v_pcbottom))
+    ptzr_ptr->set_video_pixel_cropping(v_pcleft, v_pctop, v_pcright, v_pcbottom, PARAMETER_SOURCE_CONTAINER);
+}
+
+void
+kax_track_t::handle_packetizer_stereo_mode() {
+  if (STEREO_MODE_UNSPECIFIED != v_stereo_mode)
+    ptzr_ptr->set_video_stereo_mode(v_stereo_mode, PARAMETER_SOURCE_CONTAINER);
+}
+
 /*
    Probes a file by simply comparing the first four bytes to the EBML
    head signature.
@@ -129,10 +147,6 @@ kax_reader_c::probe_file(mm_io_c *io,
   return 1;
 }
 
-// }}}
-
-// {{{ C'TOR
-
 kax_reader_c::kax_reader_c(track_info_c &_ti)
   throw (error_c)
   : generic_reader_c(_ti)
@@ -140,17 +154,17 @@ kax_reader_c::kax_reader_c(track_info_c &_ti)
   , first_timecode(-1)
   , writing_app_ver(-1)
   , m_attachment_id(0)
+  , m_tags(NULL)
 {
+  init_l1_position_storage(deferred_l1_positions);
+  init_l1_position_storage(handled_l1_positions);
+
   if (!read_headers())
     throw error_c(Y("matroska_reader: Failed to read the headers."));
   if (verbose)
     mxinfo_fn(ti.fname, Y("Using the Matroska demultiplexer.\n"));
 }
 
-// }}}
-
-// {{{ D'TOR
-
 kax_reader_c::~kax_reader_c() {
   int i;
 
@@ -161,11 +175,16 @@ kax_reader_c::~kax_reader_c() {
   delete in;
   delete es;
   delete segment;
+  delete m_tags;
 }
 
-// }}}
-
-// {{{ FUNCTIONS packets_available(), new_kax_track(), find_track*
+void
+kax_reader_c::init_l1_position_storage(deferred_positions_t &storage) {
+  storage[dl1t_attachments] = std::vector<int64_t>();
+  storage[dl1t_chapters]    = std::vector<int64_t>();
+  storage[dl1t_tags]        = std::vector<int64_t>();
+  storage[dl1t_tracks]      = std::vector<int64_t>();
+}
 
 int
 kax_reader_c::packets_available() {
@@ -183,14 +202,8 @@ kax_reader_c::packets_available() {
 
 kax_track_t *
 kax_reader_c::new_kax_track() {
-  // Set some default values.
-  kax_track_t *t   = new kax_track_t;
-  t->default_track = true;
-  t->ptzr          = -1;
-
-  tracks.push_back(t);
-
-  return t;
+  tracks.push_back(new kax_track_t);
+  return tracks.back();
 }
 
 kax_track_t *
@@ -217,10 +230,6 @@ kax_reader_c::find_track_by_uid(uint64_t uid,
   return NULL;
 }
 
-// }}}
-
-// {{{ FUNCTION kax_reader_c::verify_tracks()
-
 void
 kax_reader_c::verify_tracks() {
   int tnum, u;
@@ -239,7 +248,7 @@ kax_reader_c::verify_tracks() {
       memory_cptr private_data(new memory_c((unsigned char *)t->private_data, t->private_size, true));
       t->content_decoder.reverse(private_data, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
       private_data->lock();
-      t->private_data = private_data->get();
+      t->private_data = private_data->get_buffer();
       t->private_size = private_data->get_size();
     }
 
@@ -369,13 +378,13 @@ kax_reader_c::verify_tracks() {
 
             try {
               memory_cptr temp(new memory_c((unsigned char *)t->private_data, t->private_size, false));
-              vector<memory_cptr> blocks = unlace_memory_xiph(temp);
+              std::vector<memory_cptr> blocks = unlace_memory_xiph(temp);
               if (blocks.size() != 3)
                 throw false;
 
-              t->headers[0]      = blocks[0]->get();
-              t->headers[1]      = blocks[1]->get();
-              t->headers[2]      = blocks[2]->get();
+              t->headers[0]      = blocks[0]->get_buffer();
+              t->headers[1]      = blocks[1]->get_buffer();
+              t->headers[2]      = blocks[2]->get_buffer();
               t->header_sizes[0] = blocks[0]->get_size();
               t->header_sizes[1] = blocks[1]->get_size();
               t->header_sizes[2] = blocks[2]->get_size();
@@ -473,28 +482,25 @@ kax_reader_c::verify_tracks() {
   }
 }
 
-// }}}
+bool
+kax_reader_c::has_deferred_element_been_processed(kax_reader_c::deferred_l1_type_e type,
+                                                  int64_t position) {
+  foreach(int64_t test_position, handled_l1_positions[type])
+    if (position == test_position)
+      return true;
 
-// {{{ FUNCTION kax_reader_c::handle_attachments()
+  handled_l1_positions[type].push_back(position);
+
+  return false;
+}
 
 void
 kax_reader_c::handle_attachments(mm_io_c *io,
                                  EbmlElement *l0,
                                  int64_t pos) {
-  bool found = false;
-
-  int i;
-  for (i = 0; i < handled_attachments.size(); i++)
-    if (handled_attachments[i] == pos) {
-      found = true;
-      break;
-    }
-
-  if (found)
+  if (has_deferred_element_been_processed(dl1t_attachments, pos))
     return;
 
-  handled_attachments.push_back(pos);
-
   io->save_pos(pos);
   int upper_lvl_el;
   EbmlElement *l1 = es->FindNextElement(l0->Generic().Context, upper_lvl_el, 0xFFFFFFFFL, true);
@@ -506,13 +512,14 @@ kax_reader_c::handle_attachments(mm_io_c *io,
 
     atts->Read(*es, KaxAttachments::ClassInfos.Context, upper_lvl_el, l2, true);
 
+    int i;
     for (i = 0; i < atts->ListSize(); i++) {
       KaxAttached *att = (KaxAttached *)(*atts)[i];
 
       if (EbmlId(*att) == KaxAttached::ClassInfos.GlobalId) {
         UTFstring name        = L"";
         UTFstring description = L"";
-        string mime_type      =  "";
+        std::string mime_type =  "";
         int64_t size          = -1;
         int64_t id            = -1;
         unsigned char *data   = NULL;
@@ -531,7 +538,7 @@ kax_reader_c::handle_attachments(mm_io_c *io,
 
           } else if (EbmlId(*l2) == KaxMimeType::ClassInfos.GlobalId) {
             KaxMimeType &mtype = *static_cast<KaxMimeType *>(l2);
-            mime_type          = string(mtype);
+            mime_type          = std::string(mtype);
 
           } else if (EbmlId(*l2) == KaxFileUID::ClassInfos.GlobalId) {
             KaxFileUID &fuid = *static_cast<KaxFileUID *>(l2);
@@ -544,9 +551,9 @@ kax_reader_c::handle_attachments(mm_io_c *io,
           }
         }
 
+        ++m_attachment_id;
         attach_mode_e attach_mode;
-        if ((-1 != id) && (-1 != size) && !mime_type.empty() && (0 != name.length()) && ((attach_mode = attachment_requested(m_attachment_id + 1)) != ATTACH_MODE_SKIP)) {
-          ++m_attachment_id;
+        if ((-1 != id) && (-1 != size) && !mime_type.empty() && (0 != name.length()) && ((attach_mode = attachment_requested(m_attachment_id)) != ATTACH_MODE_SKIP)) {
 
           attachment_t matt;
           matt.name           = UTFstring_to_cstrutf8(name);
@@ -572,19 +579,9 @@ void
 kax_reader_c::handle_chapters(mm_io_c *io,
                               EbmlElement *l0,
                               int64_t pos) {
-  bool found = false;
-  int i;
-  for (i = 0; i < handled_chapters.size(); i++)
-    if (handled_chapters[i] == pos) {
-      found = true;
-      break;
-    }
-
-  if (found)
+  if (has_deferred_element_been_processed(dl1t_chapters, pos))
     return;
 
-  handled_chapters.push_back(pos);
-
   int upper_lvl_el = 0;
   io->save_pos(pos);
   EbmlElement *l1 = es->FindNextElement(l0->Generic().Context, upper_lvl_el, 0xFFFFFFFFL, true);
@@ -599,6 +596,7 @@ kax_reader_c::handle_chapters(mm_io_c *io,
     if (NULL == chapters)
       chapters = new KaxChapters;
 
+    int i;
     for (i = 0; i < tmp_chapters->ListSize(); i++)
       chapters->PushElement(*(*tmp_chapters)[i]);
     tmp_chapters->RemoveAll();
@@ -614,19 +612,9 @@ void
 kax_reader_c::handle_tags(mm_io_c *io,
                           EbmlElement *l0,
                           int64_t pos) {
-  bool found = false;
-  int i;
-  for (i = 0; i < handled_tags.size(); i++)
-    if (handled_tags[i] == pos) {
-      found = true;
-      break;
-    }
-
-  if (found)
+  if (has_deferred_element_been_processed(dl1t_tags, pos))
     return;
 
-  handled_tags.push_back(pos);
-
   int upper_lvl_el = 0;
   io->save_pos(pos);
   EbmlElement *l1 = es->FindNextElement(l0->Generic().Context, upper_lvl_el, 0xFFFFFFFFL, true);
@@ -645,23 +633,22 @@ kax_reader_c::handle_tags(mm_io_c *io,
         continue;
       }
 
-      bool is_global        = false;
+      bool delete_tag       = true;
+      bool is_global        = true;
       KaxTag *tag           = static_cast<KaxTag *>((*tags)[0]);
       KaxTagTargets *target = FINDFIRST(tag, KaxTagTargets);
 
       if (NULL != target) {
         KaxTagTrackUID *tuid = FINDFIRST(target, KaxTagTrackUID);
 
-        if (NULL == tuid)
-          is_global = true;
-        else {
-          found              = false;
+        if (NULL != tuid) {
+          is_global          = false;
           kax_track_t *track = find_track_by_uid(uint64(*tuid));
 
           if (NULL != track) {
-            found             = true;
             bool contains_tag = false;
 
+            int i;
             for (i = 0; i < tag->ListSize(); i++)
               if (dynamic_cast<KaxTagSimple *>((*tag)[i]) != NULL) {
                 contains_tag = true;
@@ -672,15 +659,19 @@ kax_reader_c::handle_tags(mm_io_c *io,
               if (NULL == track->tags)
                 track->tags = new KaxTags;
               track->tags->PushElement(*tag);
+
+              delete_tag = false;
             }
           }
         }
-      } else
-        is_global = true;
+      }
 
-      if (is_global)
-        add_tags(tag);
-      else if (!found)
+      if (is_global) {
+        if (NULL == m_tags)
+          m_tags = new KaxTags;
+        m_tags->PushElement(*tag);
+
+      } else if (delete_tag)
         delete tag;
 
       tags->Remove(0);
@@ -692,8 +683,6 @@ kax_reader_c::handle_tags(mm_io_c *io,
   io->restore_pos();
 }
 
-// }}}
-
 void
 kax_reader_c::read_headers_info(EbmlElement *&l1,
                                 EbmlElement *&l2,
@@ -733,7 +722,7 @@ kax_reader_c::read_headers_info(EbmlElement *&l1,
   // AVI-Mux GUI 1.16.8 MPEG test build 1, Aug 24 2004  12:42:57
   //
   // The idea is to first replace known application names that contain
-  // spaces with one that doesn't. Then split the whole string up on
+  // spaces with one that doesn't. Then split the whole std::string up on
   // spaces into at most three parts. If the result is at least two parts
   // long then try to parse the version number from the second and
   // store a lower case version of the first as the application's name.
@@ -759,14 +748,14 @@ void
 kax_reader_c::read_headers_info_writing_app(KaxWritingApp *&kwriting_app) {
   int idx;
 
-  string s = UTFstring_to_cstrutf8(UTFstring(*kwriting_app));
+  std::string s = UTFstring_to_cstrutf8(UTFstring(*kwriting_app));
   strip(s);
   mxverb(2, boost::format("matroska_reader: | + writing app: %1%\n") % s);
 
   if (starts_with_case(s, "avi-mux gui"))
     s.replace(0, strlen("avi-mux gui"), "avimuxgui");
 
-  vector<string> parts = split(s.c_str(), " ", 3);
+  std::vector<std::string> parts = split(s.c_str(), " ", 3);
   if (parts.size() < 2) {
     writing_app = "";
     for (idx = 0; idx < s.size(); idx++)
@@ -784,7 +773,7 @@ kax_reader_c::read_headers_info_writing_app(KaxWritingApp *&kwriting_app) {
       if (isdigit(parts[1][idx]) || (parts[1][idx] == '.'))
         s += parts[1][idx];
 
-    vector<string> ver_parts = split(s.c_str(), ".");
+    std::vector<std::string> ver_parts = split(s.c_str(), ".");
     for (idx = ver_parts.size(); idx < 4; idx++)
       ver_parts.push_back("0");
 
@@ -910,13 +899,29 @@ kax_reader_c::read_headers_track_video(kax_track_t *&track,
 }
 
 void
-kax_reader_c::read_headers_tracks(EbmlElement *&l1,
-                                  EbmlElement *&l2,
-                                  int &upper_lvl_el) {
+kax_reader_c::read_headers_tracks(mm_io_c *io,
+                                  EbmlElement *l0,
+                                  int64_t position) {
   // Yep, we've found our KaxTracks element. Now find all tracks
   // contained in this segment.
+  if (has_deferred_element_been_processed(dl1t_tracks, position))
+    return;
+
   mxverb(2, "matroska_reader: |+ segment tracks...\n");
 
+  int upper_lvl_el = 0;
+  io->save_pos(position);
+  EbmlElement *l1 = es->FindNextElement(l0->Generic().Context, upper_lvl_el, 0xFFFFFFFFL, true);
+
+  if ((l1 == NULL) || !is_id(l1, KaxTracks)) {
+    delete l1;
+    io->restore_pos();
+
+    return;
+  }
+
+  EbmlElement *l2 = NULL;
+  upper_lvl_el    = 0;
   l1->Read(*es, KaxTracks::ClassInfos.Context, upper_lvl_el, l2, true);
 
   KaxTrackEntry *ktentry = FINDFIRST(l1, KaxTrackEntry);
@@ -972,8 +977,8 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
 
     KaxCodecID *kcodecid = FINDFIRST(ktentry, KaxCodecID);
     if (NULL != kcodecid) {
-      mxverb(2, boost::format("matroska_reader: |  + Codec ID: %1%\n") % string(*kcodecid));
-      track->codec_id = string(*kcodecid);
+      mxverb(2, boost::format("matroska_reader: |  + Codec ID: %1%\n") % std::string(*kcodecid));
+      track->codec_id = std::string(*kcodecid);
     } else
       mxerror(Y("matroska_reader: The CodecID is missing.\n"));
 
@@ -1005,6 +1010,12 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
       track->default_track = uint64(*ktfdefault);
     }
 
+    KaxTrackFlagForced *ktfforced = FINDFIRST(ktentry, KaxTrackFlagForced);
+    if (NULL != ktfforced) {
+      mxverb(2, boost::format("matroska_reader: |  + Forced flag: %1%\n") % uint64(*ktfforced));
+      track->forced_track = uint64(*ktfforced);
+    }
+
     KaxTrackFlagLacing *ktflacing = FINDFIRST(ktentry, KaxTrackFlagLacing);
     if (NULL != ktflacing) {
       mxverb(2, boost::format("matroska_reader: |  + Lacing flag: %1%\n") % uint64(*ktflacing));
@@ -1021,8 +1032,8 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
 
     KaxTrackLanguage *ktlanguage = FINDFIRST(ktentry, KaxTrackLanguage);
     if (NULL != ktlanguage) {
-      mxverb(2, boost::format("matroska_reader: |  + Language: %1%\n") % string(*ktlanguage));
-      track->language = string(*ktlanguage);
+      mxverb(2, boost::format("matroska_reader: |  + Language: %1%\n") % std::string(*ktlanguage));
+      track->language = std::string(*ktlanguage);
     }
 
     KaxTrackName *ktname = FINDFIRST(ktentry, KaxTrackName);
@@ -1035,15 +1046,13 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
     ktentry = FINDNEXT(l1, KaxTrackEntry, ktentry);
   } // while (ktentry != NULL)
 
-  l1->SkipData(*es, l1->Generic().Context);
+  delete l1;
+  io->restore_pos();
 }
 
 void
 kax_reader_c::read_headers_seek_head(EbmlElement *&l0,
-                                     EbmlElement *&l1,
-                                     vector<int64_t> &deferred_tags,
-                                     vector<int64_t> &deferred_chapters,
-                                     vector<int64_t> &deferred_attachments) {
+                                     EbmlElement *&l1) {
   EbmlElement *el;
 
   KaxSeekHead &seek_head = *static_cast<KaxSeekHead *>(l1);
@@ -1055,45 +1064,35 @@ kax_reader_c::read_headers_seek_head(EbmlElement *&l0,
     if (EbmlId(*seek_head[i]) != KaxSeek::ClassInfos.GlobalId)
       continue;
 
-    KaxSeek &seek       = *static_cast<KaxSeek *>(seek_head[i]);
-    int64_t pos         = -1;
-    bool is_attachments = false;
-    bool is_chapters    = false;
-    bool is_tags        = false;
+    KaxSeek &seek           = *static_cast<KaxSeek *>(seek_head[i]);
+    int64_t pos             = -1;
+    deferred_l1_type_e type = dl1t_unknown;
     int k;
 
     for (k = 0; k < seek.ListSize(); k++)
       if (EbmlId(*seek[k]) == KaxSeekID::ClassInfos.GlobalId) {
         KaxSeekID &sid = *static_cast<KaxSeekID *>(seek[k]);
         EbmlId id(sid.GetBuffer(), sid.GetSize());
-        if (id == KaxAttachments::ClassInfos.GlobalId)
-          is_attachments = true;
-        else if (id == KaxChapters::ClassInfos.GlobalId)
-          is_chapters = true;
-        else if (id == KaxTags::ClassInfos.GlobalId)
-          is_tags = true;
+
+        type = id == KaxAttachments::ClassInfos.GlobalId ? dl1t_attachments
+          :    id == KaxChapters::ClassInfos.GlobalId    ? dl1t_chapters
+          :    id == KaxTags::ClassInfos.GlobalId        ? dl1t_tags
+          :    id == KaxTracks::ClassInfos.GlobalId      ? dl1t_tracks
+          :                                                dl1t_unknown;
 
       } else if (EbmlId(*seek[k]) == KaxSeekPosition::ClassInfos.GlobalId)
         pos = uint64(*static_cast<KaxSeekPosition *>(seek[k]));
 
-    if (-1 != pos) {
+    if ((-1 != pos) && (dl1t_unknown != type)) {
       pos = ((KaxSegment *)l0)->GetGlobalPosition(pos);
-      if (is_attachments)
-        deferred_attachments.push_back(pos);
-      else if (is_chapters)
-        deferred_chapters.push_back(pos);
-      else if (is_tags)
-        deferred_tags.push_back(pos);
+      deferred_l1_positions[type].push_back(pos);
     }
   }
 }
 
-// {{{ FUNCTION kax_reader_c::read_headers()
-
 int
 kax_reader_c::read_headers() {
   // Elements for different levels
-  vector<int64_t> deferred_tags, deferred_chapters, deferred_attachments;
 
   bool exit_loop = false;
   try {
@@ -1142,19 +1141,19 @@ kax_reader_c::read_headers() {
         read_headers_info(l1, l2, upper_lvl_el);
 
       else if (EbmlId(*l1) == KaxTracks::ClassInfos.GlobalId)
-        read_headers_tracks(l1, l2, upper_lvl_el);
+        deferred_l1_positions[dl1t_tracks].push_back(l1->GetElementPosition());
 
       else if (EbmlId(*l1) == KaxAttachments::ClassInfos.GlobalId)
-        deferred_attachments.push_back(l1->GetElementPosition());
+        deferred_l1_positions[dl1t_attachments].push_back(l1->GetElementPosition());
 
       else if (EbmlId(*l1) == KaxChapters::ClassInfos.GlobalId)
-        deferred_chapters.push_back(l1->GetElementPosition());
+        deferred_l1_positions[dl1t_chapters].push_back(l1->GetElementPosition());
 
       else if (EbmlId(*l1) == KaxTags::ClassInfos.GlobalId)
-        deferred_tags.push_back(l1->GetElementPosition());
+        deferred_l1_positions[dl1t_tags].push_back(l1->GetElementPosition());
 
       else if (EbmlId(*l1) == KaxSeekHead::ClassInfos.GlobalId)
-        read_headers_seek_head(l0, l1, deferred_tags, deferred_chapters, deferred_attachments);
+        read_headers_seek_head(l0, l1);
 
       else if (EbmlId(*l1) == KaxCluster::ClassInfos.GlobalId) {
         mxverb(2, "matroska_reader: |+ found cluster, headers are parsed completely\n");
@@ -1193,18 +1192,24 @@ kax_reader_c::read_headers() {
 
     } // while (l1 != NULL)
 
-    int i;
-    if (!ti.no_attachments)
-      for (i = 0; i < deferred_attachments.size(); i++)
-        handle_attachments(in, l0, deferred_attachments[i]);
+    foreach(int64_t position, deferred_l1_positions[dl1t_tracks])
+      read_headers_tracks(in, l0, position);
 
-    if (!ti.no_chapters)
-      for (i = 0; i < deferred_chapters.size(); i++)
-        handle_chapters(in, l0, deferred_chapters[i]);
+    if (!ti.no_attachments) {
+      foreach(int64_t position, deferred_l1_positions[dl1t_attachments])
+        handle_attachments(in, l0, position);
+    }
 
-    if (!ti.no_tags)
-      for (i = 0; i < deferred_tags.size(); i++)
-        handle_tags(in, l0, deferred_tags[i]);
+    if (!ti.no_chapters) {
+      foreach(int64_t position, deferred_l1_positions[dl1t_chapters])
+        handle_chapters(in, l0, position);
+    }
+
+    foreach(int64_t position, deferred_l1_positions[dl1t_tags])
+      handle_tags(in, l0, position);
+
+    if (!ti.no_global_tags)
+      process_global_tags();
 
   } catch (...) {
     mxerror(Y("matroska_reader: caught exception\n"));
@@ -1218,9 +1223,17 @@ kax_reader_c::read_headers() {
   return 1;
 }
 
-// }}}
+void
+kax_reader_c::process_global_tags() {
+  if ((NULL == m_tags) || g_identifying)
+    return;
 
-// {{{ FUNCTION kax_reader_c::create_packetizers()
+  int i;
+  for (i = 0; m_tags->ListSize() > i; ++i)
+    add_tags(static_cast<KaxTag *>((*m_tags)[i]));
+
+  m_tags->RemoveAll();
+}
 
 void
 kax_reader_c::init_passthrough_packetizer(kax_track_t *t) {
@@ -1235,6 +1248,7 @@ kax_reader_c::init_passthrough_packetizer(kax_track_t *t) {
 
   ptzr                    = new passthrough_packetizer_c(this, nti);
   t->ptzr                 = add_packetizer(ptzr);
+  t->ptzr_ptr             = ptzr;
   t->passthrough          = true;
   ptzr_to_track_map[ptzr] = t;
 
@@ -1252,16 +1266,11 @@ kax_reader_c::init_passthrough_packetizer(kax_track_t *t) {
   if ('v' == t->type) {
     ptzr->set_video_pixel_width(t->v_width);
     ptzr->set_video_pixel_height(t->v_height);
-    if (!ptzr->ti.aspect_ratio_given) { // The user hasn't set it.
-      if (0 != t->v_dwidth)
-        ptzr->set_video_display_width(t->v_dwidth);
-      if (0 != t->v_dheight)
-        ptzr->set_video_display_height(t->v_dheight);
-    }
-    if (!ptzr->ti.pixel_cropping_specified && ((t->v_pcleft > 0)  || (t->v_pctop > 0) || (t->v_pcright > 0) || (t->v_pcbottom > 0)))
-      ptzr->set_video_pixel_cropping(t->v_pcleft, t->v_pctop, t->v_pcright, t->v_pcbottom);
-    if (STEREO_MODE_UNSPECIFIED == ptzr->ti.stereo_mode)
-      ptzr->set_stereo_mode(t->v_stereo_mode);
+
+    t->handle_packetizer_display_dimensions();
+    t->handle_packetizer_pixel_cropping();
+    t->handle_packetizer_stereo_mode();
+
     if (CUE_STRATEGY_UNSPECIFIED == ptzr->get_cue_creation())
       ptzr->set_cue_creation(CUE_STRATEGY_IFRAMES);
 
@@ -1283,9 +1292,16 @@ void
 kax_reader_c::set_packetizer_headers(kax_track_t *t) {
   if (appending)
     return;
+
   if (t->default_track)
-    PTZR(t->ptzr)->set_as_default_track(t->type == 'v' ? DEFTRACK_TYPE_VIDEO : t->type == 'a' ? DEFTRACK_TYPE_AUDIO : DEFTRACK_TYPE_SUBS,
-                                        DEFAULT_TRACK_PRIORITY_FROM_SOURCE);
+    PTZR(t->ptzr)->set_as_default_track(t->type == 'v' ? DEFTRACK_TYPE_VIDEO : t->type == 'a' ? DEFTRACK_TYPE_AUDIO : DEFTRACK_TYPE_SUBS, DEFAULT_TRACK_PRIORITY_FROM_SOURCE);
+
+  else if (!boost::logic::indeterminate(t->default_track) && boost::logic::indeterminate(PTZR(t->ptzr)->ti.default_track))
+    PTZR(t->ptzr)->ti.default_track = false;
+
+  if (t->forced_track && boost::logic::indeterminate(PTZR(t->ptzr)->ti.forced_track))
+    PTZR(t->ptzr)->set_track_forced_flag(true);
+
   if ((0 != t->tuid) && !PTZR(t->ptzr)->set_uid(t->tuid))
     mxwarn(boost::format(Y("matroska_reader: Could not keep the track UID %1% because it is already allocated for the new file.\n")) % t->tuid);
 }
@@ -1331,18 +1347,11 @@ kax_reader_c::create_video_packetizer(kax_track_t *t,
       t->ptzr = add_packetizer(new video_packetizer_c(this, nti, t->codec_id.c_str(), t->v_frate, t->v_width, t->v_height));
     }
 
-    if (!PTZR(t->ptzr)->ti.aspect_ratio_given) {
-      // The user hasn't set it.
-      if (0 != t->v_dwidth)
-        PTZR(t->ptzr)->set_video_display_width(t->v_dwidth);
-      if (0 != t->v_dheight)
-        PTZR(t->ptzr)->set_video_display_height(t->v_dheight);
-    }
+    t->ptzr_ptr = PTZR(t->ptzr);
 
-    if (!PTZR(t->ptzr)->ti.pixel_cropping_specified && ((t->v_pcleft > 0) || (t->v_pctop > 0) || (t->v_pcright > 0) || (t->v_pcbottom > 0)))
-      PTZR(t->ptzr)->set_video_pixel_cropping(t->v_pcleft, t->v_pctop, t->v_pcright, t->v_pcbottom);
-    if (STEREO_MODE_UNSPECIFIED == PTZR(t->ptzr)->ti.stereo_mode)
-      PTZR(t->ptzr)->set_stereo_mode(t->v_stereo_mode);
+    t->handle_packetizer_display_dimensions();
+    t->handle_packetizer_pixel_cropping();
+    t->handle_packetizer_stereo_mode();
 
   } else
     init_passthrough_packetizer(t);
@@ -1401,7 +1410,7 @@ kax_reader_c::create_audio_packetizer(kax_track_t *t,
         if (sbr)
           profile        = AAC_PROFILE_SBR;
 
-      } else if (!parse_aac_codec_id(string(t->codec_id), id, profile))
+      } else if (!parse_aac_codec_id(std::string(t->codec_id), id, profile))
         mxerror_tid(ti.fname, t->tnum, boost::format(Y("Malformed codec id '%1%'.\n")) % t->codec_id);
 
     } else {
@@ -1487,7 +1496,7 @@ kax_reader_c::create_subtitle_packetizer(kax_track_t *t,
     t->sub_type = 'v';
 
   } else if (starts_with(t->codec_id, "S_TEXT", 6) || (t->codec_id == "S_SSA") || (t->codec_id == "S_ASS")) {
-    string new_codec_id = ((t->codec_id == "S_SSA") || (t->codec_id == "S_ASS")) ? string("S_TEXT/") + string(&t->codec_id[2]) : t->codec_id;
+    std::string new_codec_id = ((t->codec_id == "S_SSA") || (t->codec_id == "S_ASS")) ? std::string("S_TEXT/") + std::string(&t->codec_id[2]) : t->codec_id;
 
     t->ptzr = add_packetizer(new textsubs_packetizer_c(this, nti, new_codec_id.c_str(), t->private_data, t->private_size, false, true));
     mxinfo_tid(ti.fname, t->tnum, Y("Using the text subtitle output module.\n"));
@@ -1544,7 +1553,7 @@ kax_reader_c::create_packetizer(int64_t tid) {
     nti.language   = t->language;
   if (nti.track_name == "")
     nti.track_name = t->track_name;
-  if (NULL != t->tags)
+  if ((NULL != t->tags) && demuxing_requested('T', t->tnum))
     nti.tags       = dynamic_cast<KaxTags *>(t->tags->Clone());
 
   if (hack_engaged(ENGAGE_FORCE_PASSTHROUGH_PACKETIZER)) {
@@ -1598,7 +1607,7 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
                                                    track_info_c &nti) {
   try {
     read_first_frame(t);
-    if (NULL == t->first_frame_data.get())
+    if (!t->first_frame_data.is_set())
       throw false;
 
     avc_es_parser_c parser;
@@ -1610,7 +1619,7 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
 
     if (sizeof(alBITMAPINFOHEADER) < t->private_size)
       parser.add_bytes((unsigned char *)t->private_data + sizeof(alBITMAPINFOHEADER), t->private_size - sizeof(alBITMAPINFOHEADER));
-    parser.add_bytes(t->first_frame_data->get(), t->first_frame_data->get_size());
+    parser.add_bytes(t->first_frame_data->get_buffer(), t->first_frame_data->get_size());
     parser.flush();
 
     if (!parser.headers_parsed())
@@ -1632,11 +1641,9 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
   }
 }
 
-// }}}
-
 void
 kax_reader_c::read_first_frame(kax_track_t *t) {
-  if ((NULL != t->first_frame_data.get()) || (NULL == saved_l1))
+  if (t->first_frame_data.is_set() || (NULL == saved_l1))
     return;
 
   in->save_pos(saved_l1->GetElementPosition());
@@ -1668,7 +1675,7 @@ kax_reader_c::read_first_frame(kax_track_t *t) {
             if ((NULL == block_track) || (0 == block_simple->NumberFrames()))
               continue;
 
-            if (NULL == block_track->first_frame_data.get()) {
+            if (!block_track->first_frame_data.is_set()) {
               DataBuffer &data_buffer = block_simple->GetBuffer(0);
               block_track->first_frame_data = memory_cptr(new memory_c(data_buffer.Buffer(), data_buffer.Size()));
               block_track->content_decoder.reverse(block_track->first_frame_data, CONTENT_ENCODING_SCOPE_BLOCK);
@@ -1694,7 +1701,7 @@ kax_reader_c::read_first_frame(kax_track_t *t) {
             if ((NULL == block_track) || (0 == block->NumberFrames()))
               continue;
 
-            if (NULL == block_track->first_frame_data.get()) {
+            if (!block_track->first_frame_data.is_set()) {
               DataBuffer &data_buffer       = block->GetBuffer(0);
               block_track->first_frame_data = memory_cptr(new memory_c(data_buffer.Buffer(), data_buffer.Size()));
               block_track->content_decoder.reverse(block_track->first_frame_data, CONTENT_ENCODING_SCOPE_BLOCK);
@@ -1738,8 +1745,6 @@ kax_reader_c::read_first_frame(kax_track_t *t) {
   in->restore_pos();
 }
 
-// {{{ FUNCTION kax_reader_c::read()
-
 file_status_e
 kax_reader_c::read(generic_packetizer_c *requested_ptzr,
                    bool force) {
@@ -1859,10 +1864,10 @@ kax_reader_c::read(generic_packetizer_c *requested_ptzr,
                 block_track->content_decoder.reverse(data, CONTENT_ENCODING_SCOPE_BLOCK);
 
                 if (('s' == block_track->type) && ('t' == block_track->sub_type)) {
-                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get()) && (0 != *data->get()) && !iscr(*data->get()))) {
+                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get_buffer()) && (0 != *data->get_buffer()) && !iscr(*data->get_buffer()))) {
                     char *lines             = (char *)safemalloc(data->get_size() + 1);
                     lines[data->get_size()] = 0;
-                    memcpy(lines, data->get(), data->get_size());
+                    memcpy(lines, data->get_buffer(), data->get_size());
 
                     PTZR(block_track->ptzr)-> process(new packet_t(new memory_c((unsigned char *)lines, 0, true), last_timecode, block_duration, block_bref, block_fref));
                   }
@@ -1980,10 +1985,10 @@ kax_reader_c::read(generic_packetizer_c *requested_ptzr,
                 block_track->content_decoder.reverse(data, CONTENT_ENCODING_SCOPE_BLOCK);
 
                 if (('s' == block_track->type) && ('t' == block_track->sub_type)) {
-                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get()) && (0 != *data->get()) && !iscr(*data->get()))) {
+                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get_buffer()) && (0 != *data->get_buffer()) && !iscr(*data->get_buffer()))) {
                     char *lines             = (char *)safemalloc(data->get_size() + 1);
                     lines[data->get_size()] = 0;
-                    memcpy(lines, data->get(), data->get_size());
+                    memcpy(lines, data->get_buffer(), data->get_size());
 
                     packet_t *packet = new packet_t(new memory_c((unsigned char *)lines, 0, true), last_timecode, block_duration, block_bref, block_fref);
                     if (NULL != codec_state)
@@ -2065,10 +2070,6 @@ kax_reader_c::read(generic_packetizer_c *requested_ptzr,
   return FILE_STATUS_DONE;
 }
 
-// }}}
-
-// {{{ FUNCTIONS display_*
-
 int
 kax_reader_c::get_progress() {
   if (0 != segment_duration)
@@ -2087,13 +2088,9 @@ kax_reader_c::set_headers() {
       PTZR(tracks[i]->ptzr)->get_track_entry()->EnableLacing(tracks[i]->lacing_flag);
 }
 
-// }}}
-
-// {{{ FUNCTION kax_reader_c::identify()
-
 void
 kax_reader_c::identify() {
-  vector<string> verbose_info;
+  std::vector<std::string> verbose_info;
 
   if (!title.empty())
     verbose_info.push_back((boost::format("title:%1%") % escape(title)).str());
@@ -2121,17 +2118,21 @@ kax_reader_c::identify() {
     if (STEREO_MODE_UNSPECIFIED != tracks[i]->v_stereo_mode)
       verbose_info.push_back((boost::format("stereo_mode:%1%") % (int)tracks[i]->v_stereo_mode).str());
 
+    if ((0 != tracks[i]->v_pcleft) || (0 != tracks[i]->v_pctop) || (0 != tracks[i]->v_pcright) || (0 != tracks[i]->v_pcbottom))
+      verbose_info.push_back((boost::format("cropping:%1%,%2%,%3%,%4%") % tracks[i]->v_pcleft % tracks[i]->v_pctop % tracks[i]->v_pcright % tracks[i]->v_pcbottom).str());
+
     verbose_info.push_back((boost::format("default_track:%1%") % (tracks[i]->default_track ? 1 : 0)).str());
+    verbose_info.push_back((boost::format("forced_track:%1%")  % (tracks[i]->forced_track  ? 1 : 0)).str());
 
     if ((tracks[i]->codec_id == MKV_V_MSCOMP) && mpeg4::p10::is_avc_fourcc(tracks[i]->v_fourcc))
       verbose_info.push_back("packetizer:mpeg4_p10_es_video");
     else if (tracks[i]->codec_id == MKV_V_MPEG4_AVC)
       verbose_info.push_back("packetizer:mpeg4_p10_video");
 
-    string info = tracks[i]->codec_id;
+    std::string info = tracks[i]->codec_id;
 
     if (tracks[i]->ms_compat)
-      info += string(", ") +
+      info += std::string(", ") +
         (  tracks[i]->type        == 'v'    ? tracks[i]->v_fourcc
          : tracks[i]->a_formattag == 0x0001 ? "PCM"
          : tracks[i]->a_formattag == 0x0003 ? "PCM"
@@ -2150,9 +2151,17 @@ kax_reader_c::identify() {
 
   for (i = 0; i < g_attachments.size(); i++)
     id_result_attachment(g_attachments[i].ui_id, g_attachments[i].mime_type, g_attachments[i].data->get_size(), g_attachments[i].name, g_attachments[i].description);
-}
 
-// }}}
+  if (NULL != chapters)
+    id_result_chapters(count_chapter_atoms(*chapters));
+
+  if (NULL != m_tags)
+    id_result_tags(ID_RESULT_GLOBAL_TAGS_ID, count_simple_tags(*m_tags));
+
+  for (i = 0; tracks.size() > i; i++)
+    if (tracks[i]->ok && (NULL != tracks[i]->tags))
+      id_result_tags(tracks[i]->tnum, count_simple_tags(*tracks[i]->tags));
+}
 
 void
 kax_reader_c::add_available_track_ids() {
