@@ -15,18 +15,17 @@
 #ifndef __R_MATROSKA_H
 #define __R_MATROSKA_H
 
-#include "os.h"
-
-#include <stdio.h>
+#include "common/os.h"
 
+#include <boost/logic/tribool.hpp>
 #include <map>
 #include <vector>
 
-#include "common.h"
-#include "compression.h"
-#include "error.h"
-#include "mm_io.h"
-#include "pr_generic.h"
+#include "common/common.h"
+#include "common/compression.h"
+#include "common/error.h"
+#include "common/mm_io.h"
+#include "merge/pr_generic.h"
 
 #include <ebml/EbmlUnicodeString.h>
 
@@ -36,12 +35,11 @@
 
 using namespace libebml;
 using namespace libmatroska;
-using namespace std;
 
 struct kax_track_t {
   uint64_t tnum, tuid;
 
-  string codec_id;
+  std::string codec_id;
   bool ms_compat;
 
   char type; // 'v' = video, 'a' = audio, 't' = text subs, 'b' = buttons
@@ -70,12 +68,12 @@ struct kax_track_t {
   unsigned char *headers[3];
   uint32_t header_sizes[3];
 
-  bool default_track;
-  string language;
+  boost::logic::tribool default_track, forced_track;
+  std::string language;
 
   int64_t units_processed;
 
-  string track_name;
+  std::string track_name;
 
   bool ok;
 
@@ -86,51 +84,54 @@ struct kax_track_t {
   KaxTags *tags;
 
   int ptzr;
+  generic_packetizer_c *ptzr_ptr;
   bool headers_set;
 
   bool ignore_duration_hack;
 
   memory_cptr first_frame_data;
 
-  kax_track_t():
-    tnum(0),
-    tuid(0),
-    ms_compat(false),
-    type(' '),
-    sub_type(' '),
-    passthrough(false),
-    min_cache(0),
-    max_cache(0),
-    lacing_flag(false),
-    default_duration(0),
-    v_width(0),
-    v_height(0),
-    v_dwidth(0),
-    v_dheight(0),
-    v_pcleft(0),
-    v_pctop(0),
-    v_pcright(0),
-    v_pcbottom(0),
-    v_stereo_mode(STEREO_MODE_UNSPECIFIED),
-    v_frate(0.0),
-    v_bframes(false),
-    a_channels(0),
-    a_bps(0),
-    a_formattag(0),
-    a_sfreq(0.0),
-    a_osfreq(0.0),
-    private_data(NULL),
-    private_size(0),
-    default_track(false),
-    language("eng"),
-    units_processed(0),
-    ok(false),
-    previous_timecode(0),
-    tags(NULL),
-    ptzr(0),
-    headers_set(false),
-    ignore_duration_hack(false) {
-
+  kax_track_t()
+    : tnum(0)
+    , tuid(0)
+    , ms_compat(false)
+    , type(' ')
+    , sub_type(' ')
+    , passthrough(false)
+    , min_cache(0)
+    , max_cache(0)
+    , lacing_flag(false)
+    , default_duration(0)
+    , v_width(0)
+    , v_height(0)
+    , v_dwidth(0)
+    , v_dheight(0)
+    , v_pcleft(0)
+    , v_pctop(0)
+    , v_pcright(0)
+    , v_pcbottom(0)
+    , v_stereo_mode(STEREO_MODE_UNSPECIFIED)
+    , v_frate(0.0)
+    , v_bframes(false)
+    , a_channels(0)
+    , a_bps(0)
+    , a_formattag(0)
+    , a_sfreq(0.0)
+    , a_osfreq(0.0)
+    , private_data(NULL)
+    , private_size(0)
+    , default_track(false)
+    , forced_track(boost::logic::indeterminate)
+    , language("eng")
+    , units_processed(0)
+    , ok(false)
+    , previous_timecode(0)
+    , tags(NULL)
+    , ptzr(-1)
+    , ptzr_ptr(NULL)
+    , headers_set(false)
+    , ignore_duration_hack(false)
+  {
     memset(v_fourcc, 0, 5);
     memset(headers, 0, 3 * sizeof(unsigned char *));
     memset(header_sizes, 0, 3 * sizeof(uint32_t));
@@ -141,14 +142,26 @@ struct kax_track_t {
     if (NULL != tags)
       delete tags;
   }
+
+  void handle_packetizer_display_dimensions();
+  void handle_packetizer_pixel_cropping();
+  void handle_packetizer_stereo_mode();
 };
 
 class kax_reader_c: public generic_reader_c {
 private:
+  enum deferred_l1_type_e {
+    dl1t_unknown,
+    dl1t_attachments,
+    dl1t_chapters,
+    dl1t_tags,
+    dl1t_tracks,
+  };
+
   int act_wchar;
 
-  vector<kax_track_t *> tracks;
-  map<generic_packetizer_c *, kax_track_t *> ptzr_to_track_map;
+  std::vector<kax_track_t *> tracks;
+  std::map<generic_packetizer_c *, kax_track_t *> ptzr_to_track_map;
 
   int64_t tc_scale;
   int64_t cluster_tc;
@@ -162,15 +175,19 @@ private:
 
   int64_t segment_duration;
   int64_t last_timecode, first_timecode;
-  string title;
+  std::string title;
 
-  vector<int64_t> handled_tags, handled_attachments, handled_chapters;
+  // typedef std::map<deferred_l1_type_e, counted_ptr<std::vector<int64_t> > > deferred_positions_t;
+  typedef std::map<deferred_l1_type_e, std::vector<int64_t> > deferred_positions_t;
+  deferred_positions_t deferred_l1_positions, handled_l1_positions;
 
-  string writing_app, muxing_app;
+  std::string writing_app, muxing_app;
   int64_t writing_app_ver;
 
   int64_t m_attachment_id;
 
+  KaxTags *m_tags;
+
 public:
   kax_reader_c(track_info_c &_ti) throw (error_c);
   virtual ~kax_reader_c();
@@ -198,6 +215,7 @@ protected:
   virtual void handle_attachments(mm_io_c *io, EbmlElement *l0, int64_t pos);
   virtual void handle_chapters(mm_io_c *io, EbmlElement *l0, int64_t pos);
   virtual void handle_tags(mm_io_c *io, EbmlElement *l0, int64_t pos);
+  virtual void process_global_tags();
 
   virtual void create_video_packetizer(kax_track_t *t, track_info_c &nti);
   virtual void create_audio_packetizer(kax_track_t *t, track_info_c &nti);
@@ -209,10 +227,12 @@ protected:
   virtual void read_headers_info_writing_app(KaxWritingApp *&kwriting_app);
   virtual void read_headers_track_audio(kax_track_t *&track, KaxTrackAudio *&ktaudio);
   virtual void read_headers_track_video(kax_track_t *&track, KaxTrackVideo *&ktvideo);
-  virtual void read_headers_tracks(EbmlElement *&l1, EbmlElement *&l2, int &upper_lvl_el);
-  virtual void read_headers_seek_head(EbmlElement *&l0, EbmlElement *&l1,
-                                      vector<int64_t> &deferred_tags, vector<int64_t> &deferred_chapters, vector<int64_t> &deferred_attachments);
+  virtual void read_headers_tracks(mm_io_c *io, EbmlElement *l0, int64_t position);
+  virtual void read_headers_seek_head(EbmlElement *&l0, EbmlElement *&l1);
   virtual int  read_headers();
+
+  void init_l1_position_storage(deferred_positions_t &storage);
+  virtual bool has_deferred_element_been_processed(deferred_l1_type_e type, int64_t position);
 };
 
 #endif  // __R_MATROSKA_H
