@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_mpeg_es.h 3912 2008-09-20 14:00:11Z mosu $
-
    class definitions for the MPEG ES/PS demultiplexer module
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -16,13 +14,13 @@
 #ifndef __R_MPEG_ES_H
 #define __R_MPEG_ES_H
 
-#include "os.h"
+#include "common/os.h"
 
-#include "dts_common.h"
-#include "mpeg4_common.h"
-#include "pr_generic.h"
-#include "smart_pointers.h"
-#include "M2VParser.h"
+#include "common/dts.h"
+#include "common/mpeg1_2.h"
+#include "merge/pr_generic.h"
+#include "common/smart_pointers.h"
+#include "mpegparser/M2VParser.h"
 
 class error_c;
 class generic_reader_c;
