@@ -11,26 +11,28 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
-#include <algorithm>
-#include <memory>
-
-#include "ac3_common.h"
-#include "common.h"
-#include "error.h"
-#include "M2VParser.h"
-#include "mp3_common.h"
-#include "mpeg4_common.h"
-#include "r_mpeg_ps.h"
-#include "smart_pointers.h"
-#include "output_control.h"
-#include "p_ac3.h"
-#include "p_avc.h"
-#include "p_dts.h"
-#include "p_mp3.h"
-#include "p_vc1.h"
-#include "p_video.h"
+#include "common/ac3.h"
+#include "common/common.h"
+#include "common/endian.h"
+#include "common/error.h"
+#include "common/math.h"
+#include "common/mp3.h"
+#include "common/mpeg1_2.h"
+#include "common/mpeg4_p2.h"
+#include "common/smart_pointers.h"
+#include "common/truehd.h"
+#include "input/r_mpeg_ps.h"
+#include "merge/output_control.h"
+#include "mpegparser/M2VParser.h"
+#include "output/p_ac3.h"
+#include "output/p_avc.h"
+#include "output/p_dts.h"
+#include "output/p_mp3.h"
+#include "output/p_mpeg1_2.h"
+#include "output/p_truehd.h"
+#include "output/p_vc1.h"
 
 #define PS_PROBE_SIZE 10 * 1024 * 1024
 
@@ -45,7 +47,7 @@ mpeg_ps_reader_c::probe_file(mm_io_c *io,
                              int64_t size) {
   try {
     memory_c af_buf((unsigned char *)safemalloc(PS_PROBE_SIZE), 0, true);
-    unsigned char *buf = af_buf.get();
+    unsigned char *buf = af_buf.get_buffer();
     int num_read;
 
     io->setFilePointer(0, seek_beginning);
@@ -65,18 +67,18 @@ mpeg_ps_reader_c::probe_file(mm_io_c *io,
 }
 
 mpeg_ps_reader_c::mpeg_ps_reader_c(track_info_c &_ti)
-  throw (error_c):
-  generic_reader_c(_ti) {
-
+  throw (error_c)
+  : generic_reader_c(_ti)
+  , file_done(false)
+{
   init_reader();
 }
 
 void
 mpeg_ps_reader_c::init_reader() {
   try {
-    io        = new mm_file_io_c(ti.fname);
-    size      = io->get_size();
-    file_done = false;
+    io   = mm_multi_file_io_c::open_multi(ti.fname);
+    size = io->get_size();
 
   } catch (...) {
     throw error_c(Y("mpeg_ps_reader: Could not open the file."));
@@ -175,12 +177,13 @@ mpeg_ps_reader_c::init_reader() {
 
   io->setFilePointer(0, seek_beginning);
 
-  if (verbose)
+  if (verbose) {
     mxinfo_fn(ti.fname, Y("Using the MPEG PS demultiplexer.\n"));
+    io->display_other_file_info();
+  }
 }
 
 mpeg_ps_reader_c::~mpeg_ps_reader_c() {
-  delete io;
 }
 
 void
@@ -245,14 +248,17 @@ bool
 mpeg_ps_reader_c::read_timestamp(bit_cursor_c &bc,
                                  int64_t &timestamp) {
   bc.skip_bits(4);
-  timestamp = bc.get_bits(3);
-  bc.skip_bits(1);
-  timestamp = (timestamp << 15) | bc.get_bits(15);
-  bc.skip_bits(1);
-  timestamp = (timestamp << 15) | bc.get_bits(15);
-  bc.skip_bits(1);
+  int64_t temp_timestamp = bc.get_bits(3);
+  if (1 != bc.get_bit())
+    return false;
+  temp_timestamp = (temp_timestamp << 15) | bc.get_bits(15);
+  if (1 != bc.get_bit())
+    return false;
+  temp_timestamp = (temp_timestamp << 15) | bc.get_bits(15);
+  if (1 != bc.get_bit())
+    return false;
 
-  timestamp = timestamp * 100000ll / 9;
+  timestamp = temp_timestamp * 100000ll / 9;
 
   return true;
 }
@@ -335,12 +341,30 @@ mpeg_ps_reader_c::parse_packet(mpeg_ps_id_t &id,
 
   if (   (0xbc > id.id)
       || ((0xf0 <= id.id) && (0xfd != id.id))
-      || (0xbe == id.id)           // padding stream
       || (0xbf == id.id)) {        // private 2 stream
     io->skip(length);
     return false;
   }
 
+  if (0xbe == id.id) {        // padding stream
+    int64_t pos = io->getFilePointer();
+    io->skip(length);
+    uint32_t header = io->read_uint32_be();
+    if ( mpeg_is_start_code(header) ) {
+      io->setFilePointer(pos + length);
+    } else {
+      mxverb(2, boost::format("mpeg_ps: [begin] padding stream length incorrect at %1%, find next header...\n") % (pos - 6));
+      io->setFilePointer(pos);
+      header = 0xffffffff;
+      if (resync_stream(header)) {
+        full_length = io->getFilePointer() - pos - 4;
+        mxverb(2, boost::format("mpeg_ps: [end] padding stream length adjusted from %1% to %2%\n") % length % full_length);
+        io->setFilePointer(pos + full_length);
+      }
+    }
+    return false;
+  }
+
   if (0 == length)
     return false;
 
@@ -391,10 +415,10 @@ mpeg_ps_reader_c::parse_packet(mpeg_ps_id_t &id,
     length -= hdrlen;
 
     memory_c af_header(safemalloc(hdrlen), hdrlen, true);
-    if (io->read(af_header.get(), hdrlen) != hdrlen)
+    if (io->read(af_header.get_buffer(), hdrlen) != hdrlen)
       return false;
 
-    bit_cursor_c bc(af_header.get(), hdrlen);
+    bit_cursor_c bc(af_header.get_buffer(), hdrlen);
 
     try {
       // PTS
@@ -452,15 +476,11 @@ mpeg_ps_reader_c::parse_packet(mpeg_ps_id_t &id,
         return false;
 
       } else if (   ((0x80 <= id.sub_id) && (0x8f >= id.sub_id))
-                 || ((0x98 <= id.sub_id) && (0xaf >= id.sub_id))
-                 || ((0xc0 <= id.sub_id) && (0xcf >= id.sub_id))) {
-        io->skip(3);         // number of frames, startpos
-        length -= 3;
-
-        if ((0xa0 == (id.sub_id & 0xe0)) && (3 <= length)) { // LPCM
-          io->skip(3);
-          length -= 3;
-        }
+                 || ((0x98 <= id.sub_id) && (0xcf >= id.sub_id))) {
+        // Number of frames, startpos. MLP/TrueHD audio has a 4 byte header.
+        int audio_header_len = (0xb0 <= id.sub_id) && (0xbf >= id.sub_id) ? 4 : 3;
+        io->skip(audio_header_len);
+        length -= audio_header_len;
       }
     }
 
@@ -567,10 +587,10 @@ mpeg_ps_reader_c::new_stream_v_avc_or_mpeg_1_2(mpeg_ps_id_t id,
         continue;
 
       memory_cptr new_buf = memory_c::alloc(new_length);
-      if (io->read(new_buf->get(), new_length) != new_length)
+      if (io->read(new_buf->get_buffer(), new_length) != new_length)
         throw false;
 
-      buffer.add(new_buf->get(), new_length);
+      buffer.add(new_buf->get_buffer(), new_length);
     }
   } catch (...) {
   }
@@ -584,7 +604,7 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
                                         unsigned char *buf,
                                         int length,
                                         mpeg_ps_track_ptr &track) {
-  auto_ptr<M2VParser> m2v_parser(new M2VParser);
+  counted_ptr<M2VParser> m2v_parser(new M2VParser);
 
   m2v_parser->WriteData(buf, length);
 
@@ -599,10 +619,10 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
       break;
 
     memory_c new_buf((unsigned char *)safemalloc(length), 0, true);
-    if (io->read(new_buf.get(), length) != length)
+    if (io->read(new_buf.get_buffer(), length) != length)
       break;
 
-    m2v_parser->WriteData(new_buf.get(), length);
+    m2v_parser->WriteData(new_buf.get_buffer(), length);
 
     state = m2v_parser->GetState();
   }
@@ -614,8 +634,8 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
   }
 
   MPEG2SequenceHeader seq_hdr = m2v_parser->GetSequenceHeader();
-  auto_ptr<MPEGFrame> frame(m2v_parser->ReadFrame());
-  if (frame.get() == NULL)
+  counted_ptr<MPEGFrame> frame(m2v_parser->ReadFrame());
+  if (!frame.is_set())
     throw false;
 
   track->fourcc         = FOURCC('M', 'P', 'G', '0' + m2v_parser->GetMPEGVersion());
@@ -633,7 +653,7 @@ mpeg_ps_reader_c::new_stream_v_mpeg_1_2(mpeg_ps_id_t id,
 
   MPEGChunk *raw_seq_hdr = m2v_parser->GetRealSequenceHeader();
   if (NULL != raw_seq_hdr) {
-    track->raw_seq_hdr      = (unsigned char *) safememdup(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
+    track->raw_seq_hdr      = (unsigned char *)safememdup(raw_seq_hdr->GetPointer(), raw_seq_hdr->GetSize());
     track->raw_seq_hdr_size = raw_seq_hdr->GetSize();
   }
 
@@ -664,10 +684,10 @@ mpeg_ps_reader_c::new_stream_v_avc(mpeg_ps_id_t id,
     if (!parse_packet(id, timecode, length, full_length))
       break;
     memory_c new_buf((unsigned char *)safemalloc(length), 0, true);
-    if (io->read(new_buf.get(), length) != length)
+    if (io->read(new_buf.get_buffer(), length) != length)
       break;
 
-    parser.add_bytes(new_buf.get(), length);
+    parser.add_bytes(new_buf.get_buffer(), length);
   }
 
   if (!parser.headers_parsed())
@@ -676,7 +696,7 @@ mpeg_ps_reader_c::new_stream_v_avc(mpeg_ps_id_t id,
   track->v_avcc = parser.get_avcc();
 
   try {
-    mm_mem_io_c avcc(track->v_avcc->get(), track->v_avcc->get_size());
+    mm_mem_io_c avcc(track->v_avcc->get_buffer(), track->v_avcc->get_size());
     mm_mem_io_c new_avcc(NULL, track->v_avcc->get_size(), 1024);
     memory_cptr nalu(new memory_c());
     int num_sps, sps, sps_length;
@@ -698,7 +718,7 @@ mpeg_ps_reader_c::new_stream_v_avc(mpeg_ps_id_t id,
       avcc.read(nalu, sps_length);
 
       abort = false;
-      if ((0 < sps_length) && ((nalu->get()[0] & 0x1f) == 7)) {
+      if ((0 < sps_length) && ((nalu->get_buffer()[0] & 0x1f) == 7)) {
         nalu_to_rbsp(nalu);
         if (!mpeg4::p10::parse_sps(nalu, sps_info, true))
           throw false;
@@ -761,10 +781,10 @@ mpeg_ps_reader_c::new_stream_v_vc1(mpeg_ps_id_t id,
       break;
 
     memory_cptr new_buf = memory_c::alloc(length);
-    if (io->read(new_buf->get(), length) != length)
+    if (io->read(new_buf->get_buffer(), length) != length)
       break;
 
-    parser.add_bytes(new_buf->get(), length);
+    parser.add_bytes(new_buf->get_buffer(), length);
   }
 
   if (!parser.is_sequence_header_available())
@@ -839,10 +859,58 @@ mpeg_ps_reader_c::new_stream_a_dts(mpeg_ps_id_t id,
       continue;
 
     memory_cptr new_buf = memory_c::alloc(length);
-    if (io->read(new_buf->get(), length) != length)
+    if (io->read(new_buf->get_buffer(), length) != length)
       throw false;
 
-    buffer.add(new_buf->get(), length);
+    buffer.add(new_buf->get_buffer(), length);
+  }
+}
+
+void
+mpeg_ps_reader_c::new_stream_a_truehd(mpeg_ps_id_t id,
+                                      unsigned char *buf,
+                                      int length,
+                                      mpeg_ps_track_ptr &track) {
+  truehd_parser_c parser;
+
+  parser.add_data(buf, length);
+
+  while (1) {
+    while (parser.frame_available()) {
+      truehd_frame_cptr frame = parser.get_next_frame();
+      if (truehd_frame_t::sync != frame->m_type)
+        continue;
+
+      mxverb(2,
+             boost::format("first TrueHD header channels %1% sampling_rate %2% samples_per_frame %3%\n")
+             % frame->m_channels % frame->m_sampling_rate % frame->m_samples_per_frame);
+
+      track->a_channels    = frame->m_channels;
+      track->a_sample_rate = frame->m_sampling_rate;
+
+      return;
+    }
+
+    if (PS_PROBE_SIZE < io->getFilePointer())
+      throw false;
+
+    if (!find_next_packet_for_id(id, PS_PROBE_SIZE))
+      throw false;
+
+    int full_length;
+    int64_t timecode;
+    mpeg_ps_id_t new_id(id.id);
+    if (!parse_packet(new_id, timecode, length, full_length))
+      continue;
+
+    if (id.sub_id != new_id.sub_id)
+      continue;
+
+    memory_cptr new_buf = memory_c::alloc(length);
+    if (io->read(new_buf->get_buffer(), length) != length)
+      throw false;
+
+    parser.add_data(new_buf->get_buffer(), length);
   }
 }
 
@@ -855,7 +923,7 @@ mpeg_ps_reader_c::new_stream_a_dts(mpeg_ps_id_t id,
   . 0x88..0x8f DTS
   . 0x98..0x9f DTS
   . 0xa0..0xaf PCM
-  . 0xb0..0xbf LPCM (without 0xbd)
+  . 0xb0..0xbf TrueHD
   . 0xc0..0xc7 (E)AC3
   0xc0..0xdf   MP2 audio
   0xe0..0xef   MPEG-1/-2 video
@@ -913,6 +981,9 @@ mpeg_ps_reader_c::found_new_stream(mpeg_ps_id_t id) {
       else if ((0xa0 <= id.sub_id) && (0xa7 >= id.sub_id))
         track->fourcc = FOURCC('P', 'C', 'M', ' ');
 
+      else if ((0xb0 <= id.sub_id) && (0xbf >= id.sub_id))
+        track->fourcc = FOURCC('T', 'R', 'H', 'D');
+
       else
         track->type = '?';
 
@@ -933,7 +1004,7 @@ mpeg_ps_reader_c::found_new_stream(mpeg_ps_id_t id) {
       return;
 
     memory_c af_buf((unsigned char *)safemalloc(length), 0, true);
-    unsigned char *buf = af_buf.get();
+    unsigned char *buf = af_buf.get_buffer();
     if (io->read(buf, length) != length)
       throw false;
 
@@ -952,6 +1023,9 @@ mpeg_ps_reader_c::found_new_stream(mpeg_ps_id_t id) {
     else if (FOURCC('W', 'V', 'C', '1') == track->fourcc)
       new_stream_v_vc1(id, buf, length, track);
 
+    else if (FOURCC('T', 'R', 'H', 'D') == track->fourcc)
+      new_stream_a_truehd(id, buf, length, track);
+
     else
       // Unsupported track type
       throw false;
@@ -1107,6 +1181,11 @@ mpeg_ps_reader_c::create_packetizer(int64_t id) {
         mxinfo_tid(ti.fname, id, Y("Using the DTS output module.\n"));
       track->ptzr = add_packetizer(new dts_packetizer_c(this, ti, track->dts_header, true));
 
+    } else if (FOURCC('T', 'R', 'H', 'D') == track->fourcc) {
+      if (verbose)
+        mxinfo_tid(ti.fname, id, Y("Using the TrueHD output module.\n"));
+      track->ptzr = add_packetizer(new truehd_packetizer_c(this, ti, truehd_frame_t::truehd, track->a_sample_rate, track->a_channels));
+
     } else
       mxerror(boost::format(Y("mpeg_ps_reader: Should not have happened #1. %1%")) % BUGMSG);
 
@@ -1172,7 +1251,9 @@ mpeg_ps_reader_c::read(generic_packetizer_c *,
     while (find_next_packet(new_id)) {
       packet_pos = io->getFilePointer() - 4;
       if (!parse_packet(new_id, timecode, length, full_length)) {
-        mxverb(2, boost::format("mpeg_ps: packet_parse failed at %1%, skipping %2%\n") % packet_pos % full_length);
+        if (    (0xbe != new_id.id)       // padding stream
+             && (0xbf != new_id.id))      // private 2 stream (navigation data)
+          mxverb(2, boost::format("mpeg_ps: parse_packet failed at %1%, skipping %2%\n") % packet_pos % full_length);
         io->setFilePointer(packet_pos + 4 + 2 + full_length);
         continue;
       }
@@ -1182,7 +1263,7 @@ mpeg_ps_reader_c::read(generic_packetizer_c *,
         continue;
       }
 
-      mpeg_ps_track_t *track = tracks[id2idx[new_id.idx()]].get();
+      mpeg_ps_track_t *track = tracks[id2idx[new_id.idx()]].get_object();
 
       mxverb(3,
              boost::format("mpeg_ps: packet for 0x%|1$02x|(%|2$02x|) length %3% at %4% timecode %5%\n")
@@ -1249,7 +1330,7 @@ mpeg_ps_reader_c::finish() {
   if (file_done)
     return FILE_STATUS_DONE;
 
-  vector<mpeg_ps_track_ptr>::iterator track;
+  std::vector<mpeg_ps_track_ptr>::iterator track;
 
   mxforeach(track, tracks) {
     if (0 < (*track)->buffer_usage) {
@@ -1271,10 +1352,12 @@ mpeg_ps_reader_c::get_progress() {
 
 void
 mpeg_ps_reader_c::identify() {
-  vector<string> verbose_info;
+  std::vector<std::string> verbose_info;
   int i;
 
-  id_result_container((boost::format("MPEG %1% program stream (PS)") % version).str());
+  io->create_verbose_identification_info(verbose_info);
+
+  id_result_container((boost::format("MPEG %1% program stream (PS)") % version).str(), verbose_info);
 
   for (i = 0; i < tracks.size(); i++) {
     mpeg_ps_track_ptr &track = tracks[i];
@@ -1289,6 +1372,9 @@ mpeg_ps_reader_c::identify() {
 
     verbose_info.push_back((boost::format("stream_id:%|1$02x| sub_stream_id:%|2$02x|") % track->id.id % track->id.sub_id).str());
 
+    if ((0 != track->v_dwidth) && (0 != track->v_dheight))
+      verbose_info.push_back((boost::format("display_dimensions:%1%x%2%") % track->v_dwidth % track->v_dheight).str());
+
     id_result_track(i, 'a' == track->type ? ID_RESULT_TRACK_AUDIO : ID_RESULT_TRACK_VIDEO,
                       FOURCC('M', 'P', 'G', '1') == track->fourcc ? "MPEG-1"
                     : FOURCC('M', 'P', 'G', '2') == track->fourcc ? "MPEG-2"
@@ -1299,6 +1385,7 @@ mpeg_ps_reader_c::identify() {
                     : FOURCC('M', 'P', '3', ' ') == track->fourcc ? "MPEG-1 layer 3"
                     : FOURCC('A', 'C', '3', ' ') == track->fourcc ? (16 == track->a_bsid ? "EAC3" : "AC3")
                     : FOURCC('D', 'T', 'S', ' ') == track->fourcc ? "DTS"
+                    : FOURCC('T', 'R', 'H', 'D') == track->fourcc ? "TrueHD"
                     : FOURCC('P', 'C', 'M', ' ') == track->fourcc ? "PCM"
                     : FOURCC('L', 'P', 'C', 'M') == track->fourcc ? "LPCM"
                     :                                               Y("unknown"),
