@@ -14,16 +14,18 @@
 #ifndef __R_MPEG_PS_H
 #define __R_MPEG_PS_H
 
-#include "os.h"
+#include "common/os.h"
 
 #include <map>
 
-#include "bit_cursor.h"
-#include "dts_common.h"
-#include "mpeg4_common.h"
-#include "packet_extensions.h"
-#include "pr_generic.h"
-#include "smart_pointers.h"
+#include "common/bit_cursor.h"
+#include "common/dts.h"
+#include "common/mm_multi_file_io.h"
+#include "common/mpeg1_2.h"
+#include "merge/packet_extensions.h"
+#include "merge/pr_generic.h"
+
+namespace bfs = boost::filesystem;
 
 struct mpeg_ps_id_t {
   int id;
@@ -120,17 +122,17 @@ typedef counted_ptr<mpeg_ps_track_t> mpeg_ps_track_ptr;
 
 class mpeg_ps_reader_c: public generic_reader_c {
 private:
-  mm_io_c *io;
+  mm_multi_file_io_cptr io;
   int64_t bytes_processed, size, duration, global_timecode_offset;
 
-  map<int, int> id2idx;
-  map<int, bool> blacklisted_ids;
+  std::map<int, int> id2idx;
+  std::map<int, bool> blacklisted_ids;
 
   uint32_t es_map[NUM_ES_MAP_ENTRIES];
   int version;
   bool file_done;
 
-  vector<mpeg_ps_track_ptr> tracks;
+  std::vector<mpeg_ps_track_ptr> tracks;
 
 public:
   mpeg_ps_reader_c(track_info_c &_ti) throw (error_c);
@@ -163,6 +165,7 @@ private:
   virtual void new_stream_a_mpeg(mpeg_ps_id_t id, unsigned char *buf, int length, mpeg_ps_track_ptr &track);
   virtual void new_stream_a_ac3(mpeg_ps_id_t id, unsigned char *buf, int length, mpeg_ps_track_ptr &track);
   virtual void new_stream_a_dts(mpeg_ps_id_t id, unsigned char *buf, int length, mpeg_ps_track_ptr &track);
+  virtual void new_stream_a_truehd(mpeg_ps_id_t id, unsigned char *buf, int length, mpeg_ps_track_ptr &track);
   virtual bool resync_stream(uint32_t &header);
   virtual file_status_e finish();
   void sort_tracks();
