@@ -11,8 +11,8 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
-#include "config.h"
+#include "common/os.h"
+#include "common/os.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,27 +28,32 @@ extern "C" {                    // for BITMAPINFOHEADER
 #include "avilib.h"
 }
 
-#include "aac_common.h"
-#include "chapters.h"
-#include "common.h"
-#include "commonebml.h"
-#include "hacks.h"
-#include "iso639.h"
-#include "matroska.h"
-#include "ogmstreams.h"
-#include "output_control.h"
-#include "pr_generic.h"
-#include "p_aac.h"
-#include "p_ac3.h"
-#include "p_avc.h"
-#include "p_mp3.h"
-#include "p_pcm.h"
-#include "p_textsubs.h"
-#include "p_video.h"
-#include "p_vorbis.h"
-#include "p_kate.h"
-#include "r_ogm.h"
-#include "r_ogm_flac.h"
+#include "common/aac.h"
+#include "common/chapters/chapters.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/endian.h"
+#include "common/hacks.h"
+#include "common/iso639.h"
+#include "common/matroska.h"
+#include "common/mpeg4_p2.h"
+#include "common/locale.h"
+#include "common/ogmstreams.h"
+#include "input/r_ogm.h"
+#include "input/r_ogm_flac.h"
+#include "merge/output_control.h"
+#include "merge/pr_generic.h"
+#include "output/p_aac.h"
+#include "output/p_ac3.h"
+#include "output/p_avc.h"
+#include "output/p_kate.h"
+#include "output/p_mp3.h"
+#include "output/p_mpeg4_p2.h"
+#include "output/p_pcm.h"
+#include "output/p_textsubs.h"
+#include "output/p_theora.h"
+#include "output/p_video.h"
+#include "output/p_vorbis.h"
 
 #define BUFFER_SIZE 4096
 
@@ -69,7 +74,7 @@ public:
   virtual const char *get_type() {
     return ID_RESULT_TRACK_AUDIO;
   };
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "AAC";
   };
 };
@@ -85,7 +90,7 @@ public:
   virtual const char *get_type() {
     return ID_RESULT_TRACK_AUDIO;
   };
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "AC3";
   };
 };
@@ -101,7 +106,7 @@ public:
   virtual const char *get_type() {
     return ID_RESULT_TRACK_AUDIO;
   };
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "MP2/MP3";
   };
 };
@@ -117,7 +122,7 @@ public:
   virtual const char *get_type() {
     return ID_RESULT_TRACK_AUDIO;
   };
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "PCM";
   };
 };
@@ -135,7 +140,7 @@ public:
   virtual const char *get_type() {
     return ID_RESULT_TRACK_AUDIO;
   };
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "Vorbis";
   };
 };
@@ -153,32 +158,11 @@ public:
   virtual const char *get_type() {
     return ID_RESULT_TRACK_SUBTITLES;
   };
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "Text";
   };
 };
 
-class ogm_v_avc_demuxer_c: public ogm_demuxer_c {
-public:
-
-public:
-  ogm_v_avc_demuxer_c(ogm_reader_c *p_reader);
-
-  virtual const char *get_type() {
-    return ID_RESULT_TRACK_VIDEO;
-  };
-
-  virtual string get_codec() {
-    return "h.264/AVC";
-  };
-
-  virtual void initialize();
-  virtual generic_packetizer_c *create_packetizer(track_info_c &ti);
-
-private:
-  virtual memory_cptr extract_avcc();
-};
-
 class ogm_v_mscomp_demuxer_c: public ogm_demuxer_c {
 public:
   int64_t frames_since_granulepos_change;
@@ -190,13 +174,29 @@ public:
     return ID_RESULT_TRACK_VIDEO;
   };
 
-  virtual string get_codec();
+  virtual std::string get_codec();
 
   virtual void initialize();
   virtual generic_packetizer_c *create_packetizer(track_info_c &ti);
   virtual void process_page(int64_t granulepos);
 };
 
+class ogm_v_avc_demuxer_c: public ogm_v_mscomp_demuxer_c {
+public:
+
+public:
+  ogm_v_avc_demuxer_c(ogm_reader_c *p_reader);
+
+  virtual std::string get_codec() {
+    return "h.264/AVC";
+  };
+
+  virtual generic_packetizer_c *create_packetizer(track_info_c &ti);
+
+private:
+  virtual memory_cptr extract_avcc();
+};
+
 class ogm_v_theora_demuxer_c: public ogm_demuxer_c {
 public:
   theora_identification_header_t theora;
@@ -208,7 +208,7 @@ public:
     return ID_RESULT_TRACK_VIDEO;
   };
 
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "Theora";
   };
 
@@ -229,7 +229,7 @@ public:
     return ID_RESULT_TRACK_SUBTITLES;
   };
 
-  virtual string get_codec() {
+  virtual std::string get_codec() {
     return "Kate";
   };
 
@@ -241,10 +241,10 @@ public:
 
 // ---------------------------------------------------------------------------------
 
-static counted_ptr<vector<string> >
+static counted_ptr<std::vector<std::string> >
 extract_vorbis_comments(const memory_cptr &mem) {
-  counted_ptr<vector<string> > comments(new vector<string>);
-  mm_mem_io_c in(mem->get(), mem->get_size());
+  counted_ptr<std::vector<std::string> > comments(new std::vector<std::string>);
+  mm_mem_io_c in(mem->get_buffer(), mem->get_size());
   uint32_t i, n, len;
 
   try {
@@ -259,10 +259,10 @@ extract_vorbis_comments(const memory_cptr &mem) {
       len = in.read_uint32_le();
       memory_cptr buffer(new memory_c((unsigned char *)safemalloc(len + 1), len + 1));
 
-      if (in.read(buffer->get(), len) != len)
+      if (in.read(buffer->get_buffer(), len) != len)
         throw false;
 
-      comments->push_back(string((char *)buffer->get(), len));
+      comments->push_back(std::string((char *)buffer->get_buffer(), len));
     }
   } catch(...) {
   }
@@ -433,7 +433,7 @@ ogm_reader_c::handle_new_stream_and_packets(ogg_page *og) {
 
   handle_new_stream(og);
   dmx = find_demuxer(ogg_page_serialno(og));
-  if (dmx.get())
+  if (dmx.is_set())
     process_header_packets(dmx);
 }
 
@@ -535,11 +535,11 @@ ogm_reader_c::handle_new_stream(ogg_page *og) {
   if (!dmx)
     dmx = new ogm_demuxer_c(this);
 
-  string type   = dmx->get_type();
+  std::string type = dmx->get_type();
 
-  dmx->serialno = ogg_page_serialno(og);
-  dmx->track_id = sdemuxers.size();
-  dmx->in_use   = (type != "unknown") && demuxing_requested(type[0], dmx->track_id);
+  dmx->serialno    = ogg_page_serialno(og);
+  dmx->track_id    = sdemuxers.size();
+  dmx->in_use      = (type != "unknown") && demuxing_requested(type[0], dmx->track_id);
 
   dmx->packet_data.push_back(memory_cptr(new memory_c((unsigned char *)safememdup(op.packet, op.bytes), op.bytes, true)));
 
@@ -564,7 +564,7 @@ ogm_reader_c::process_page(ogg_page *og) {
   int64_t granulepos;
 
   dmx = find_demuxer(ogg_page_serialno(og));
-  if (!dmx.get() || !dmx->in_use)
+  if (!dmx.is_set() || !dmx->in_use)
     return;
 
   granulepos = ogg_page_granulepos(og);
@@ -587,7 +587,7 @@ ogm_reader_c::process_header_page(ogg_page *og) {
   ogm_demuxer_cptr dmx;
 
   dmx = find_demuxer(ogg_page_serialno(og));
-  if (!dmx.get() ||dmx->headers_read)
+  if (!dmx.is_set() ||dmx->headers_read)
     return;
 
   ogg_stream_pagein(&dmx->os, og);
@@ -689,14 +689,14 @@ ogm_reader_c::get_progress() {
 
 void
 ogm_reader_c::identify() {
-  vector<string> verbose_info;
+  std::vector<std::string> verbose_info;
   int i;
 
   // Check if a video track has a TITLE comment. If yes we use this as the
   // new segment title / global file title.
   for (i = 0; i < sdemuxers.size(); i++)
     if ((sdemuxers[i]->title != "") && (sdemuxers[i]->stype == OGM_STREAM_TYPE_V_MSCOMP)) {
-      verbose_info.push_back(string("title:") + escape(sdemuxers[i]->title));
+      verbose_info.push_back(std::string("title:") + escape(sdemuxers[i]->title));
       break;
     }
 
@@ -706,25 +706,28 @@ ogm_reader_c::identify() {
     verbose_info.clear();
 
     if (sdemuxers[i]->language != "")
-      verbose_info.push_back(string("language:") + escape(sdemuxers[i]->language));
+      verbose_info.push_back(std::string("language:") + escape(sdemuxers[i]->language));
 
     if ((sdemuxers[i]->title != "") && (sdemuxers[i]->stype != OGM_STREAM_TYPE_V_MSCOMP))
-      verbose_info.push_back(string("track_name:") + escape(sdemuxers[i]->title));
+      verbose_info.push_back(std::string("track_name:") + escape(sdemuxers[i]->title));
 
     if ((0 != sdemuxers[i]->display_width) && (0 != sdemuxers[i]->display_height))
       verbose_info.push_back((boost::format("display_dimensions:%1%x%2%") % sdemuxers[i]->display_width % sdemuxers[i]->display_height).str());
 
     id_result_track(i, sdemuxers[i]->get_type(), sdemuxers[i]->get_codec(), verbose_info);
   }
+
+  if (NULL != chapters)
+    id_result_chapters(count_chapter_atoms(*chapters));
 }
 
 void
 ogm_reader_c::handle_stream_comments() {
-  counted_ptr<vector<string> > comments;
-  string title;
+  counted_ptr<std::vector<std::string> > comments;
+  std::string title;
 
   bool charset_warning_printed = false;
-  int cch                      = utf8_init(ti.chapter_charset);
+  charset_converter_cptr cch   = charset_converter_c::init(ti.chapter_charset);
   int i;
 
   for (i = 0; i < sdemuxers.size(); i++) {
@@ -736,12 +739,12 @@ ogm_reader_c::handle_stream_comments() {
     if (comments->empty())
       continue;
 
-    vector<string> chapter_strings;
+    std::vector<std::string> chapter_strings;
 
     int j;
     for (j = 0; comments->size() > j; j++) {
       mxverb(2, boost::format("ogm_reader: commment for #%1% for %2%: %3%\n") % j % i % (*comments)[j]);
-      vector<string> comment = split((*comments)[j], "=", 2);
+      std::vector<std::string> comment = split((*comments)[j], "=", 2);
       if (comment.size() != 2)
         continue;
 
@@ -753,8 +756,8 @@ ogm_reader_c::handle_stream_comments() {
           dmx->language = iso639_languages[index].iso639_2_code;
         else {
 
-          string lang = comment[1];
-          int pos1    = lang.find("[");
+          std::string lang = comment[1];
+          int pos1         = lang.find("[");
           while (0 <= pos1) {
             int pos2 = lang.find("]", pos1);
             if (-1 == pos2)
@@ -784,39 +787,47 @@ ogm_reader_c::handle_stream_comments() {
         chapter_strings.push_back((*comments)[j]);
     }
 
-    if (((title != "") || (chapter_strings.size() > 0)) && !charset_warning_printed && (ti.chapter_charset == "")) {
-      mxwarn_fn(ti.fname,
-                Y("This Ogg/OGM file contains chapter or title information. Unfortunately the charset used to store this information in "
-                  "the file cannot be identified unambiguously. The program assumes that your system's current charset is appropriate. This can "
-                  "be overridden with the '--chapter-charset <charset>' switch.\n"));
-      charset_warning_printed = true;
-    }
-
+    bool segment_title_set = false;
     if (title != "") {
-      title = to_utf8(cch, title);
+      title = cch->utf8(title);
       if (!g_segment_title_set && g_segment_title.empty() && (OGM_STREAM_TYPE_V_MSCOMP == dmx->stype)) {
         g_segment_title     = title;
         g_segment_title_set = true;
+        segment_title_set   = true;
       }
       dmx->title = title.c_str();
       title      = "";
     }
 
-    if (!chapter_strings.empty() && !ti.no_chapters && (NULL == g_kax_chapters)) {
+    bool chapters_set = false;
+    if (!chapter_strings.empty() && !ti.no_chapters) {
       try {
-        auto_ptr<mm_mem_io_c> out(new mm_mem_io_c(NULL, 0, 1000));
+        counted_ptr<mm_mem_io_c> out(new mm_mem_io_c(NULL, 0, 1000));
 
         out->write_bom("UTF-8");
         for (j = 0; j < chapter_strings.size(); j++)
-          out->puts(to_utf8(cch, chapter_strings[j]) + string("\n"));
+          out->puts(cch->utf8(chapter_strings[j]) + std::string("\n"));
         out->set_file_name(ti.fname);
 
-        auto_ptr<mm_text_io_c> text_out(new mm_text_io_c(out.get(), false));
+        counted_ptr<mm_text_io_c> text_out(new mm_text_io_c(out.get_object(), false));
 
-        g_kax_chapters = parse_chapters(text_out.get());
+        chapters     = parse_chapters(text_out.get_object(), 0, -1, 0, ti.chapter_language);
+        chapters_set = true;
+
+        align_chapter_edition_uids(chapters);
       } catch (...) {
       }
     }
+
+    if (    (segment_title_set || chapters_set)
+         && !charset_warning_printed
+         && (ti.chapter_charset.empty())) {
+      mxwarn_fn(ti.fname,
+                Y("This Ogg/OGM file contains chapter or title information. Unfortunately the charset used to store this information in "
+                  "the file cannot be identified unambiguously. The program assumes that your system's current charset is appropriate. This can "
+                  "be overridden with the '--chapter-charset <charset>' switch.\n"));
+      charset_warning_printed = true;
+    }
   }
 }
 
@@ -872,7 +883,8 @@ ogm_demuxer_c::get_duration_and_len(ogg_packet &op,
     duration = 0;
 }
 
-void ogm_demuxer_c::process_page(int64_t granulepos) {
+void
+ogm_demuxer_c::process_page(int64_t granulepos) {
   ogg_packet op;
 
   while (ogg_stream_packetout(&os, &op) == 1) {
@@ -937,14 +949,14 @@ ogm_a_aac_demuxer_c::create_packetizer(track_info_c &ti) {
   bool sbr = false;
 
   if ((packet_data[0]->get_size() >= (sizeof(stream_header) + 5)) &&
-      (parse_aac_data(packet_data[0]->get()      + sizeof(stream_header) + 5,
-                      packet_data[0]->get_size() - sizeof(stream_header) - 5,
+      (parse_aac_data(packet_data[0]->get_buffer() + sizeof(stream_header) + 5,
+                      packet_data[0]->get_size()   - sizeof(stream_header) - 5,
                       profile, channels, sample_rate, output_sample_rate, sbr))) {
     if (sbr)
       profile = AAC_PROFILE_SBR;
 
   } else {
-    sth         = (stream_header *)&packet_data[0]->get()[1];
+    sth         = (stream_header *)&packet_data[0]->get_buffer()[1];
     channels    = get_uint16_le(&sth->sh.audio.channels);
     sample_rate = get_uint64_le(&sth->samples_per_unit);
     profile     = AAC_PROFILE_LC;
@@ -973,7 +985,7 @@ ogm_a_ac3_demuxer_c::ogm_a_ac3_demuxer_c(ogm_reader_c *p_reader)
 
 generic_packetizer_c *
 ogm_a_ac3_demuxer_c::create_packetizer(track_info_c &ti) {
-  stream_header        *sth      = (stream_header *)(packet_data[0]->get() + 1);
+  stream_header        *sth      = (stream_header *)(packet_data[0]->get_buffer() + 1);
   generic_packetizer_c *ptzr_obj = new ac3_packetizer_c(reader, ti, get_uint64_le(&sth->samples_per_unit), get_uint16_le(&sth->sh.audio.channels), 0);
 
   mxinfo_tid(ti.fname, ti.id, Y("Using the AC3 output module.\n"));
@@ -991,7 +1003,7 @@ ogm_a_mp3_demuxer_c::ogm_a_mp3_demuxer_c(ogm_reader_c *p_reader)
 
 generic_packetizer_c *
 ogm_a_mp3_demuxer_c::create_packetizer(track_info_c &ti) {
-  stream_header        *sth      = (stream_header *)(packet_data[0]->get() + 1);
+  stream_header        *sth      = (stream_header *)(packet_data[0]->get_buffer() + 1);
   generic_packetizer_c *ptzr_obj = new mp3_packetizer_c(reader, ti, get_uint64_le(&sth->samples_per_unit), get_uint16_le(&sth->sh.audio.channels), true);
 
   mxinfo_tid(ti.fname, ti.id, Y("Using the MPEG audio output module.\n"));
@@ -1009,7 +1021,7 @@ ogm_a_pcm_demuxer_c::ogm_a_pcm_demuxer_c(ogm_reader_c *p_reader)
 
 generic_packetizer_c *
 ogm_a_pcm_demuxer_c::create_packetizer(track_info_c &ti) {
-  stream_header        *sth      = (stream_header *)(packet_data[0]->get() + 1);
+  stream_header        *sth      = (stream_header *)(packet_data[0]->get_buffer() + 1);
   generic_packetizer_c *ptzr_obj = new pcm_packetizer_c(reader, ti, get_uint64_le(&sth->samples_per_unit), get_uint16_le(&sth->sh.audio.channels),
                                                         get_uint16_le(&sth->bits_per_sample));
 
@@ -1030,9 +1042,9 @@ ogm_a_vorbis_demuxer_c::ogm_a_vorbis_demuxer_c(ogm_reader_c *p_reader)
 generic_packetizer_c *
 ogm_a_vorbis_demuxer_c::create_packetizer(track_info_c &ti) {
   generic_packetizer_c *ptzr_obj = new vorbis_packetizer_c(reader, ti,
-                                                           packet_data[0]->get(), packet_data[0]->get_size(),
-                                                           packet_data[1]->get(), packet_data[1]->get_size(),
-                                                           packet_data[2]->get(), packet_data[2]->get_size());
+                                                           packet_data[0]->get_buffer(), packet_data[0]->get_size(),
+                                                           packet_data[1]->get_buffer(), packet_data[1]->get_size(),
+                                                           packet_data[2]->get_buffer(), packet_data[2]->get_size());
 
   mxinfo_tid(ti.fname, ti.id, Y("Using the Vorbis output module.\n"));
 
@@ -1096,26 +1108,18 @@ ogm_s_text_demuxer_c::process_page(int64_t granulepos) {
 // -----------------------------------------------------------
 
 ogm_v_avc_demuxer_c::ogm_v_avc_demuxer_c(ogm_reader_c *p_reader)
-  : ogm_demuxer_c(p_reader)
+  : ogm_v_mscomp_demuxer_c(p_reader)
 {
   stype                  = OGM_STREAM_TYPE_V_AVC;
   num_non_header_packets = 3;
 }
 
-void
-ogm_v_avc_demuxer_c::initialize() {
-  stream_header *sth = (stream_header *)(packet_data[0]->get() + 1);
-
-  if (0 > g_video_fps)
-    g_video_fps = 10000000.0 / (float)get_uint64_le(&sth->time_unit);
-}
-
 generic_packetizer_c *
 ogm_v_avc_demuxer_c::create_packetizer(track_info_c &ti) {
   mpeg4_p10_es_video_packetizer_c *vptzr = NULL;
 
   try {
-    stream_header *sth = (stream_header *)&packet_data[0]->get()[1];
+    stream_header *sth = (stream_header *)&packet_data[0]->get_buffer()[1];
 
     ti.private_data    = NULL;
     ti.private_size    = 0;
@@ -1145,8 +1149,8 @@ ogm_v_avc_demuxer_c::extract_avcc() {
   else if (map_has_key(reader->ti.nalu_size_lengths, -1))
     parser.set_nalu_size_length(reader->ti.nalu_size_lengths[-1]);
 
-  unsigned char *private_data = packet_data[0]->get()      + 1 + sizeof(stream_header);
-  int private_size            = packet_data[0]->get_size() - 1 - sizeof(stream_header);
+  unsigned char *private_data = packet_data[0]->get_buffer() + 1 + sizeof(stream_header);
+  int private_size            = packet_data[0]->get_size()   - 1 - sizeof(stream_header);
 
   while (4 < private_size) {
     if (get_uint32_be(private_data) == 0x00000001) {
@@ -1158,11 +1162,11 @@ ogm_v_avc_demuxer_c::extract_avcc() {
     --private_size;
   }
 
-  vector<memory_cptr>::iterator packet(nh_packet_data.begin());
+  std::vector<memory_cptr>::iterator packet(nh_packet_data.begin());
 
   while (packet != nh_packet_data.end()) {
     if ((*packet)->get_size()) {
-      parser.add_bytes((*packet)->get(), (*packet)->get_size());
+      parser.add_bytes((*packet)->get_buffer(), (*packet)->get_size());
       if (parser.headers_parsed())
         return parser.get_avcc();
     }
@@ -1182,9 +1186,9 @@ ogm_v_mscomp_demuxer_c::ogm_v_mscomp_demuxer_c(ogm_reader_c *p_reader)
   stype = OGM_STREAM_TYPE_V_MSCOMP;
 }
 
-string
+std::string
 ogm_v_mscomp_demuxer_c::get_codec() {
-  stream_header *sth = (stream_header *)(packet_data[0]->get() + 1);
+  stream_header *sth = (stream_header *)(packet_data[0]->get_buffer() + 1);
   char fourcc[5];
 
   memcpy(fourcc, sth->subtype, 4);
@@ -1195,7 +1199,7 @@ ogm_v_mscomp_demuxer_c::get_codec() {
 
 void
 ogm_v_mscomp_demuxer_c::initialize() {
-  stream_header *sth = (stream_header *)(packet_data[0]->get() + 1);
+  stream_header *sth = (stream_header *)(packet_data[0]->get_buffer() + 1);
 
   if (0 > g_video_fps)
     g_video_fps = 10000000.0 / (float)get_uint64_le(&sth->time_unit);
@@ -1206,7 +1210,7 @@ ogm_v_mscomp_demuxer_c::initialize() {
 generic_packetizer_c *
 ogm_v_mscomp_demuxer_c::create_packetizer(track_info_c &ti) {
   alBITMAPINFOHEADER bih;
-  stream_header *sth = (stream_header *)&packet_data[0]->get()[1];
+  stream_header *sth = (stream_header *)&packet_data[0]->get_buffer()[1];
 
   // AVI compatibility mode. Fill in the values we've got and guess
   // the others.
@@ -1245,7 +1249,7 @@ ogm_v_mscomp_demuxer_c::create_packetizer(track_info_c &ti) {
 
 void
 ogm_v_mscomp_demuxer_c::process_page(int64_t granulepos) {
-  vector<ogm_frame_t> frames;
+  std::vector<ogm_frame_t> frames;
   ogg_packet op;
   int64_t duration;
 
@@ -1305,7 +1309,7 @@ void
 ogm_v_theora_demuxer_c::initialize() {
   try {
     memory_cptr &mem = packet_data[0];
-    theora_parse_identification_header(mem->get(), mem->get_size(), theora);
+    theora_parse_identification_header(mem->get_buffer(), mem->get_size(), theora);
 
     display_width  = theora.display_width;
     display_height = theora.display_height;
@@ -1317,7 +1321,7 @@ ogm_v_theora_demuxer_c::initialize() {
 generic_packetizer_c *
 ogm_v_theora_demuxer_c::create_packetizer(track_info_c &ti) {
   memory_cptr codecprivate       = lace_memory_xiph(packet_data);
-  ti.private_data                = codecprivate->get();
+  ti.private_data                = codecprivate->get_buffer();
   ti.private_size                = codecprivate->get_size();
 
   double                fps      = (double)theora.frn / (double)theora.frd;
@@ -1337,10 +1341,13 @@ ogm_v_theora_demuxer_c::process_page(int64_t granulepos) {
   while (ogg_stream_packetout(&os, &op) == 1) {
     eos |= op.e_o_s;
 
-    if ((0 == op.bytes) || (0 != (op.packet[0] & 0x80)))
+    if ((0 != op.bytes) && (0 != (op.packet[0] & 0x80)))
+      // Header packets have the very first bit set. Skip those.
       continue;
 
-    bool is_keyframe = 0x00 == (op.packet[0] & 0x40);
+    // Zero-length frames are 'repeat previous frame' markers and
+    // cannot be I frames.
+    bool is_keyframe = (0 != op.bytes) && (0x00 == (op.packet[0] & 0x40));
     int64_t timecode = (int64_t)(1000000000.0 * units_processed * theora.frd / theora.frn);
     int64_t duration = (int64_t)(1000000000.0 *                   theora.frd / theora.frn);
     int64_t bref     = is_keyframe ? VFT_IFRAME : VFT_PFRAMEAUTOMATIC;
@@ -1357,7 +1364,7 @@ ogm_v_theora_demuxer_c::process_page(int64_t granulepos) {
 
 bool
 ogm_v_theora_demuxer_c::is_header_packet(ogg_packet &op) {
-  return ((0x80 <= op.packet[0]) && (0x82 >= op.packet[0]));
+  return (0 != op.bytes) && (0x80 <= op.packet[0]) && (0x82 >= op.packet[0]);
 }
 
 // -----------------------------------------------------------
@@ -1375,7 +1382,7 @@ void
 ogm_s_kate_demuxer_c::initialize() {
   try {
     memory_cptr &mem = packet_data[0];
-    kate_parse_identification_header(mem->get(), mem->get_size(), kate);
+    kate_parse_identification_header(mem->get_buffer(), mem->get_size(), kate);
     num_header_packets = kate.nheaders;
   } catch (error_c &e) {
     mxerror_tid(reader->ti.fname, track_id, boost::format(Y("The Kate identifaction header could not be parsed (%1%).\n")) % e.get_error());
@@ -1385,7 +1392,7 @@ ogm_s_kate_demuxer_c::initialize() {
 generic_packetizer_c *
 ogm_s_kate_demuxer_c::create_packetizer(track_info_c &ti) {
   memory_cptr codecprivate       = lace_memory_xiph(packet_data);
-  ti.private_data                = codecprivate->get();
+  ti.private_data                = codecprivate->get_buffer();
   ti.private_size                = codecprivate->get_size();
 
   generic_packetizer_c *ptzr_obj = new kate_packetizer_c(reader, ti, ti.private_data, ti.private_size);
