@@ -15,32 +15,40 @@
      the original authors.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <algorithm>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <boost/math/common_factor.hpp>
+#include <cstring>
 #if defined(HAVE_ZLIB_H)
 #include <zlib.h>
 #endif
 
-#include "aac_common.h"
+extern "C" {
 #include "avilib.h"
-#include "common.h"
-#include "hacks.h"
-#include "matroska.h"
-#include "output_control.h"
-#include "p_aac.h"
-#include "p_ac3.h"
-#include "p_mp3.h"
-#include "p_passthrough.h"
-#include "p_pcm.h"
-#include "p_video.h"
-#include "r_qtmp4.h"
+}
+
+#include "common/aac.h"
+#include "common/chapters/chapters.h"
+#include "common/common.h"
+#include "common/endian.h"
+#include "common/hacks.h"
+#include "common/iso639.h"
+#include "common/locale.h"
+#include "common/matroska.h"
+#include "common/strings/formatting.h"
+#include "input/r_qtmp4.h"
+#include "merge/output_control.h"
+#include "output/p_aac.h"
+#include "output/p_ac3.h"
+#include "output/p_mp3.h"
+#include "output/p_mpeg1_2.h"
+#include "output/p_mpeg4_p2.h"
+#include "output/p_mpeg4_p10.h"
+#include "output/p_passthrough.h"
+#include "output/p_pcm.h"
+#include "output/p_video.h"
 
-using namespace std;
 using namespace libmatroska;
 
 #if defined(ARCH_BIGENDIAN)
@@ -63,7 +71,7 @@ space(int num) {
   memset(s, ' ', num);
   s[num] = 0;
 
-  return string(s);
+  return std::string(s);
 }
 
 int
@@ -170,8 +178,8 @@ qtmp4_reader_c::read_atom(mm_io_c *read_from,
 
 void
 qtmp4_reader_c::parse_headers() {
-  uint32_t tmp, idx;
-  int i;
+  unsigned int idx;
+  uint32_t tmp;
 
   io->setFilePointer(0);
 
@@ -185,9 +193,10 @@ qtmp4_reader_c::parse_headers() {
       mxverb(2, boost::format("Quicktime/MP4 reader:   File type major brand: %1%%2%%3%%4%\n") % BE2STR(tmp));
       tmp = io->read_uint32_be();
       mxverb(2, boost::format("Quicktime/MP4 reader:   File type minor brand: 0x%|1$08x|\n") % tmp);
-      for (i = 0; i < ((atom.size - 16) / 4); ++i) {
+
+      for (idx = 0; idx < ((atom.size - 16) / 4); ++idx) {
         tmp = io->read_uint32_be();
-        mxverb(2, boost::format("Quicktime/MP4 reader:   File type compatible brands #%1%: %2%%3%%4%%5%\n") % i % BE2STR(tmp));
+        mxverb(2, boost::format("Quicktime/MP4 reader:   File type compatible brands #%1%: %2%%3%%4%%5%\n") % idx % BE2STR(tmp));
       }
 
     } else if (FOURCC('m', 'o', 'o', 'v') == atom.fourcc) {
@@ -222,14 +231,17 @@ qtmp4_reader_c::parse_headers() {
          && strncasecmp(dmx->fourcc, "cvid", 4)
          && strncasecmp(dmx->fourcc, "rle ", 4)
          && strncasecmp(dmx->fourcc, "mp4v", 4)
+         && strncasecmp(dmx->fourcc, "xvid", 4)
          && strncasecmp(dmx->fourcc, "avc1", 4))
         ||
         (   ('a' == dmx->type)
          && strncasecmp(dmx->fourcc, "QDM", 3)
          && strncasecmp(dmx->fourcc, "MP4A", 4)
+         && strncasecmp(dmx->fourcc, ".mp3", 4)
          && strncasecmp(dmx->fourcc, "twos", 4)
-         && strncasecmp(dmx->fourcc, "swot", 4)
-         && strncasecmp(dmx->fourcc, "ac-3", 4))) {
+         && strncasecmp(dmx->fourcc, "sowt", 4)
+         && strncasecmp(dmx->fourcc, "ac-3", 4)
+         && strncasecmp(dmx->fourcc, "sac3", 4))) {
       mxwarn(boost::format(Y("Quicktime/MP4 reader: Unknown/unsupported FourCC '%|1$.4s|' for track %2%.\n")) % dmx->fourcc % dmx->id);
       continue;
     }
@@ -304,6 +316,8 @@ qtmp4_reader_c::parse_headers() {
     dmx->ok = true;
   }
 
+  read_chapter_track();
+
   if (!g_identifying)
     calculate_timecodes();
 
@@ -312,7 +326,7 @@ qtmp4_reader_c::parse_headers() {
 
 void
 qtmp4_reader_c::calculate_timecodes() {
-  vector<qtmp4_demuxer_cptr>::iterator idmx;
+  std::vector<qtmp4_demuxer_cptr>::iterator idmx;
   int64_t min_timecode = 0;
 
   mxforeach(idmx, demuxers) {
@@ -346,7 +360,7 @@ qtmp4_reader_c::parse_video_header_priv_atoms(qtmp4_demuxer_cptr &dmx,
                                               unsigned char *mem,
                                               int size,
                                               int level) {
-  if (!dmx->v_is_avc && strncasecmp(dmx->fourcc, "mp4v", 4) && (0 != size)) {
+  if (!dmx->v_is_avc && strncasecmp(dmx->fourcc, "mp4v", 4) && strncasecmp(dmx->fourcc, "xvid", 4) && (0 != size)) {
     dmx->priv_size = size;
     dmx->priv      = (unsigned char *)safememdup(mem, size);
 
@@ -502,9 +516,9 @@ qtmp4_reader_c::handle_cmvd_atom(qt_atom_t atom,
   mm_io_c *old_io         = io;
   uint32_t cmov_size      = atom.size - atom.hsize;
   memory_cptr af_cmov_buf = memory_c::alloc(cmov_size);
-  unsigned char *cmov_buf = af_cmov_buf->get();
+  unsigned char *cmov_buf = af_cmov_buf->get_buffer();
   memory_cptr af_moov_buf = memory_c::alloc(moov_size + 16);
-  unsigned char *moov_buf = af_moov_buf->get();
+  unsigned char *moov_buf = af_moov_buf->get_buffer();
 
   if (io->read(cmov_buf, cmov_size) != cmov_size)
     throw error_c(Y("end-of-file"));
@@ -601,6 +615,10 @@ qtmp4_reader_c::handle_hdlr_atom(qtmp4_demuxer_cptr &new_dmx,
     case FOURCC('v', 'i', 'd', 'e'):
       new_dmx->type = 'v';
       break;
+
+    case FOURCC('t', 'e', 'x', 't'):
+      chapter_dmx = new_dmx;
+      break;
   }
 }
 
@@ -623,8 +641,7 @@ qtmp4_reader_c::handle_mdhd_atom(qtmp4_demuxer_cptr &new_dmx,
 
     new_dmx->time_scale      = get_uint32_be(&mdhd.time_scale);
     new_dmx->global_duration = get_uint32_be(&mdhd.duration);
-
-    mxverb(2, boost::format("Quicktime/MP4 reader:%1%Time scale: %2%, duration: %3%\n") % space(level * 2 + 1) % new_dmx->time_scale % new_dmx->global_duration);
+    new_dmx->language        = decode_and_verify_language(get_uint16_be(&mdhd.language));
 
   } else if (1 == version) {
     mdhd64_atom_t mdhd;
@@ -636,12 +653,13 @@ qtmp4_reader_c::handle_mdhd_atom(qtmp4_demuxer_cptr &new_dmx,
 
     new_dmx->time_scale      = get_uint32_be(&mdhd.time_scale);
     new_dmx->global_duration = get_uint64_be(&mdhd.duration);
-
-    mxverb(2, boost::format("Quicktime/MP4 reader:%1%Time scale: %2%, duration: %3%\n") % space(level * 2 + 1) % new_dmx->time_scale % new_dmx->global_duration);
+    new_dmx->language        = decode_and_verify_language(get_uint16_be(&mdhd.language));
 
   } else
     mxerror(boost::format(Y("Quicktime/MP4 reader: The 'media header' atom ('mdhd') uses the unsupported version %1%.\n")) % version);
 
+  mxverb(2, boost::format("Quicktime/MP4 reader:%1%Time scale: %2%, duration: %3%, language: %4%\n") % space(level * 2 + 1) % new_dmx->time_scale % new_dmx->global_duration % new_dmx->language);
+
   if (0 == new_dmx->time_scale)
     mxerror(Y("Quicktime/MP4 reader: The 'time scale' parameter was 0. This is not supported.\n"));
 }
@@ -701,6 +719,9 @@ qtmp4_reader_c::handle_moov_atom(qt_atom_t parent,
     } else if (FOURCC('m', 'v', 'h', 'd') == atom.fourcc)
       handle_mvhd_atom(atom.to_parent(), level + 1);
 
+    else if (FOURCC('u', 'd', 't', 'a') == atom.fourcc)
+      handle_udta_atom(atom.to_parent(), level + 1);
+
     else if (FOURCC('t', 'r', 'a', 'k') == atom.fourcc) {
       qtmp4_demuxer_cptr new_dmx(new qtmp4_demuxer_c);
 
@@ -729,6 +750,123 @@ qtmp4_reader_c::handle_mvhd_atom(qt_atom_t atom,
   mxverb(2, boost::format("Quicktime/MP4 reader:%1%Time scale: %2%\n") % space(level * 2 + 1) % time_scale);
 }
 
+void
+qtmp4_reader_c::handle_udta_atom(qt_atom_t parent,
+                                 int level) {
+  while (8 <= parent.size) {
+    qt_atom_t atom = read_atom();
+    print_basic_atom_info();
+
+    if (FOURCC('c', 'h', 'p', 'l') == atom.fourcc)
+      handle_chpl_atom(atom.to_parent(), level + 1);
+
+    skip_atom();
+    parent.size -= atom.size;
+  }
+}
+
+void
+qtmp4_reader_c::handle_chpl_atom(qt_atom_t atom,
+                                 int level) {
+  if (ti.no_chapters || (NULL != chapters))
+    return;
+
+  io->skip(1 + 3 + 4);          // Version, flags, zero
+
+  int count = io->read_uint8();
+  mxverb(2, boost::format("Quicktime/MP4 reader:%1%Chapter list: %2% entries\n") % space(level * 2 + 1) % count);
+
+  if (0 == count)
+    return;
+
+  std::vector<qtmp4_chapter_entry_t> entries;
+
+  int i;
+  for (i = 0; i < count; ++i) {
+    uint64_t timecode = io->read_uint64_be() * 100;
+    memory_cptr buf   = memory_c::alloc(io->read_uint8() + 1);
+    memset(buf->get_buffer(), 0, buf->get_size());
+
+    if (io->read(buf->get_buffer(), buf->get_size() - 1) != (buf->get_size() - 1))
+      break;
+
+    entries.push_back(qtmp4_chapter_entry_t(std::string(reinterpret_cast<char *>(buf->get_buffer())), timecode));
+  }
+
+  recode_chapter_entries(entries);
+  process_chapter_entries(level, entries);
+}
+
+void
+qtmp4_reader_c::read_chapter_track() {
+  if (ti.no_chapters || (NULL != chapters) || !chapter_dmx.is_set())
+    return;
+
+  chapter_dmx->update_tables(time_scale);
+
+  if (chapter_dmx->sample_table.empty())
+    return;
+
+  std::vector<qtmp4_chapter_entry_t> entries;
+  uint64_t pts_scale_gcd = boost::math::gcd(static_cast<uint64_t>(1000000000ull), static_cast<uint64_t>(chapter_dmx->time_scale));
+  uint64_t pts_scale_num = 1000000000ull                                  / pts_scale_gcd;
+  uint64_t pts_scale_den = static_cast<uint64_t>(chapter_dmx->time_scale) / pts_scale_gcd;
+
+  foreach(qt_sample_t &sample, chapter_dmx->sample_table) {
+    if (2 >= sample.size)
+      continue;
+
+    io->setFilePointer(sample.pos, seek_beginning);
+    memory_cptr chunk(memory_c::alloc(sample.size));
+    if (io->read(chunk->get_buffer(), sample.size) != sample.size)
+      continue;
+
+    unsigned int name_len = get_uint16_be(chunk->get_buffer());
+    if ((name_len + 2) > sample.size)
+      continue;
+
+    entries.push_back(qtmp4_chapter_entry_t(std::string(reinterpret_cast<char *>(chunk->get_buffer()) + 2, name_len),
+                                            sample.pts * pts_scale_num / pts_scale_den));
+  }
+
+  process_chapter_entries(0, entries);
+}
+
+void
+qtmp4_reader_c::process_chapter_entries(int level,
+                                        std::vector<qtmp4_chapter_entry_t> &entries) {
+  if (entries.empty())
+    return;
+
+  mxverb(3, boost::format("Quicktime/MP4 reader:%1%%2% chapter(s):\n") % space((level + 1) * 2 + 1) % entries.size());
+
+  stable_sort(entries.begin(), entries.end());
+
+  mm_mem_io_c out(NULL, 0, 1000);
+  out.set_file_name(ti.fname);
+  out.write_bom("UTF-8");
+
+  unsigned int i = 0;
+  for (; entries.size() > i; ++i) {
+    qtmp4_chapter_entry_t &chapter = entries[i];
+
+    mxverb(3, boost::format("Quicktime/MP4 reader:%1%%2%: start %4% name %3%\n") % space((level + 1) * 2 + 1) % i % chapter.m_name % format_timecode(chapter.m_timecode));
+
+    out.puts(boost::format("CHAPTER%|1$02d|=%|2$02d|:%|3$02d|:%|4$02d|.%|5$03d|\n"
+                           "CHAPTER%|1$02d|NAME=%6%\n")
+             % i
+             % (int)( chapter.m_timecode / 60 / 60 / 1000000000)
+             % (int)((chapter.m_timecode      / 60 / 1000000000) %   60)
+             % (int)((chapter.m_timecode           / 1000000000) %   60)
+             % (int)((chapter.m_timecode           /    1000000) % 1000)
+             % chapter.m_name);
+  }
+
+  mm_text_io_c text_out(&out, false);
+  chapters = parse_chapters(&text_out, 0, -1, 0, ti.chapter_language);
+  align_chapter_edition_uids(chapters);
+}
+
 void
 qtmp4_reader_c::handle_stbl_atom(qtmp4_demuxer_cptr &new_dmx,
                                  qt_atom_t parent,
@@ -833,26 +971,38 @@ qtmp4_reader_c::handle_stsd_atom(qtmp4_demuxer_cptr &new_dmx,
 
   int i;
   for (i = 0; i < count; ++i) {
-    int64_t pos         = io->getFilePointer();
-    uint32_t size       = io->read_uint32_be();
-    memory_cptr af_priv = memory_c::alloc(size);
-    unsigned char *priv = af_priv->get();
+    int64_t pos   = io->getFilePointer();
+    uint32_t size = io->read_uint32_be();
 
-    if (io->read(priv, size) != size)
+    if (4 > size)
+      mxerror(boost::format(Y("Quicktime/MP4 reader: The 'size' field is too small in the stream description atom for track ID %1%.\n")) % new_dmx->id);
+
+    memory_cptr af_priv = memory_c::alloc(size);
+    unsigned char *priv = af_priv->get_buffer();
+
+    put_uint32_be(priv, size);
+    if (io->read(priv + sizeof(uint32_t), size - sizeof(uint32_t)) != (size - sizeof(uint32_t)))
       mxerror(boost::format(Y("Quicktime/MP4 reader: Could not read the stream description atom for track ID %1%.\n")) % new_dmx->id);
 
     if ('a' == new_dmx->type) {
+      new_dmx->a_stsd = clone_memory(priv, size);
+
       if (sizeof(sound_v0_stsd_atom_t) > size)
         mxerror(boost::format(Y("Quicktime/MP4 reader: Could not read the sound description atom for track ID %1%.\n")) % new_dmx->id);
 
       sound_v1_stsd_atom_t sv1_stsd;
+      sound_v2_stsd_atom_t sv2_stsd;
       memcpy(&sv1_stsd, priv, sizeof(sound_v0_stsd_atom_t));
+      memcpy(&sv2_stsd, priv, sizeof(sound_v0_stsd_atom_t));
+
       if (0 != new_dmx->fourcc[0])
         mxwarn(boost::format(Y("Quicktime/MP4 reader: Track ID %1% has more than one FourCC. Only using the first one (%|2$.4s|) and not this one (%|3$.4s|).\n"))
                % new_dmx->id % new_dmx->fourcc % (const unsigned char *)sv1_stsd.v0.base.fourcc);
       else
         memcpy(new_dmx->fourcc, sv1_stsd.v0.base.fourcc, 4);
 
+      uint16_t version = get_uint16_be(&sv1_stsd.v0.version);
+
       mxverb(2,
              boost::format("Quicktime/MP4 reader:%1%FourCC: %|2$.4s|, channels: %3%, sample size: %4%, compression id: %5%, sample rate: 0x%|6$08x|, version: %7%")
              % space(level * 2 + 1)
@@ -863,17 +1013,37 @@ qtmp4_reader_c::handle_stsd_atom(qtmp4_demuxer_cptr &new_dmx,
              % get_uint32_be(&sv1_stsd.v0.sample_rate)
              % get_uint16_be(&sv1_stsd.v0.version));
 
-      if (get_uint16_be(&sv1_stsd.v0.version) == 1) {
-        if ((sizeof(sound_v1_stsd_atom_t) > size) || (io->read(&sv1_stsd.v1, sizeof(sv1_stsd.v1)) != sizeof(sv1_stsd.v1)))
+      if (1 == version) {
+        if (sizeof(sound_v1_stsd_atom_t) > size)
           mxerror(boost::format(Y("Quicktime/MP4 reader: Could not read the extended sound description atom for track ID %1%.\n")) % new_dmx->id);
 
+        memcpy(&sv1_stsd, priv, sizeof(sound_v1_stsd_atom_t));
+
         mxverb(2,
                boost::format(" samples per packet: %1% bytes per packet: %2% bytes per frame: %3% bytes_per_sample: %4%")
                % get_uint32_be(&sv1_stsd.v1.samples_per_packet)
                % get_uint32_be(&sv1_stsd.v1.bytes_per_packet)
                % get_uint32_be(&sv1_stsd.v1.bytes_per_frame)
                % get_uint32_be(&sv1_stsd.v1.bytes_per_sample));
+
+      } else if (2 == version) {
+        if (sizeof(sound_v2_stsd_atom_t) > size)
+          mxerror(boost::format(Y("Quicktime/MP4 reader: Could not read the extended sound description atom for track ID %1%.\n")) % new_dmx->id);
+
+        memcpy(&sv2_stsd, priv, sizeof(sound_v2_stsd_atom_t));
+
+        mxverb(2,
+               boost::format(" struct size: %1% sample rate: %|2$016x| channels: %3% const1: 0x%|4$08x| bits per channel: %5% flags: %6% bytes per frame: %7% samples per frame: %8%")
+               % get_uint32_be(&sv2_stsd.v2.v2_struct_size)
+               % get_uint64_be(&sv2_stsd.v2.sample_rate)
+               % get_uint32_be(&sv2_stsd.v2.channels)
+               % get_uint32_be(&sv2_stsd.v2.const1)
+               % get_uint32_be(&sv2_stsd.v2.bits_per_channel)
+               % get_uint32_be(&sv2_stsd.v2.flags)
+               % get_uint32_be(&sv2_stsd.v2.bytes_per_frame)
+               % get_uint32_be(&sv2_stsd.v2.samples_per_frame));
       }
+
       mxverb(2, "\n");
 
       new_dmx->a_channels   = get_uint16_be(&sv1_stsd.v0.channels);
@@ -881,25 +1051,24 @@ qtmp4_reader_c::handle_stsd_atom(qtmp4_demuxer_cptr &new_dmx,
       uint32_t tmp          = get_uint32_be(&sv1_stsd.v0.sample_rate);
       new_dmx->a_samplerate = (float)((tmp & 0xffff0000) >> 16) + (float)(tmp & 0x0000ffff) / 65536.0;
 
-      if (get_uint16_be(&sv1_stsd.v0.version) == 1)
-        stsd_size = sizeof(sound_v1_stsd_atom_t);
-
-      else if (get_uint16_be(&sv1_stsd.v0.version) == 0)
+      if (get_uint16_be(&sv1_stsd.v0.version) == 0)
         stsd_size = sizeof(sound_v0_stsd_atom_t);
 
-      else if (get_uint16_be(&sv1_stsd.v0.version) == 2)
-        stsd_size = 68;
+      else if (get_uint16_be(&sv1_stsd.v0.version) == 1)
+        stsd_size = sizeof(sound_v1_stsd_atom_t);
 
-      memcpy(&new_dmx->a_stsd, &sv1_stsd, sizeof(sound_v1_stsd_atom_t));
+      else if (get_uint16_be(&sv1_stsd.v0.version) == 2)
+        stsd_size = sizeof(sound_v2_stsd_atom_t);
 
     } else if ('v' == new_dmx->type) {
-      if (size < sizeof(video_stsd_atom_t))
+      new_dmx->v_stsd = clone_memory(priv, size);
+
+      if (sizeof(video_stsd_atom_t) > size)
         mxerror(boost::format(Y("Quicktime/MP4 reader: Could not read the video description atom for track ID %1%.\n")) % new_dmx->id);
 
       video_stsd_atom_t v_stsd;
       memcpy(&v_stsd, priv, sizeof(video_stsd_atom_t));
-      new_dmx->v_stsd      = (unsigned char *)safememdup(priv, size);
-      new_dmx->v_stsd_size = size;
+
       if (0 != new_dmx->fourcc[0])
         mxwarn(boost::format(Y("Quicktime/MP4 reader: Track ID %1% has more than one FourCC. Only using the first one (%|2$.4s|) and not this one (%|3$.4s|).\n"))
                % new_dmx->id % new_dmx->fourcc % (const unsigned char *)v_stsd.base.fourcc);
@@ -1048,7 +1217,7 @@ qtmp4_reader_c::handle_elst_atom(qtmp4_demuxer_cptr &new_dmx,
     editlist.speed    = io->read_uint32_be();
   }
 
-  mxverb(2, boost::format("Quicktime/MP4 reader:%1%Edit list table: %2% entries\n") % space(level * 2 + 1) % count);
+  mxverb(2, boost::format("Quicktime/MP4 reader:%1%Edit std::list table: %2% entries\n") % space(level * 2 + 1) % count);
   for (i = 0; i < count; ++i)
     mxverb(4,
            boost::format("Quicktime/MP4 reader:%1%%2%: duration %3% pos %4% speed %5%\n")
@@ -1126,7 +1295,11 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
   int buffer_offset = 0;
   unsigned char *buffer;
 
-  if (('v' == dmx->type) && (0 == dmx->pos) && !strncasecmp(dmx->fourcc, "mp4v", 4) && dmx->esds_parsed && (NULL != dmx->esds.decoder_config)) {
+  if (   ('v' == dmx->type)
+      && (0 == dmx->pos)
+      && (!strncasecmp(dmx->fourcc, "mp4v", 4) || !strncasecmp(dmx->fourcc, "xvid", 4))
+      && dmx->esds_parsed
+      && (NULL != dmx->esds.decoder_config)) {
     buffer        = (unsigned char *)safemalloc(index.size + dmx->esds.decoder_config_len);
     buffer_offset = dmx->esds.decoder_config_len;
 
@@ -1157,8 +1330,8 @@ qtmp4_reader_c::read(generic_packetizer_c *ptzr,
 
 uint32_t
 qtmp4_reader_c::read_esds_descr_len(mm_mem_io_c &memio) {
-  uint32_t len       = 0;
-  uint32_t num_bytes = 0;
+  uint32_t len           = 0;
+  unsigned int num_bytes = 0;
   uint8_t byte;
 
   do {
@@ -1260,7 +1433,7 @@ qtmp4_reader_c::create_bitmap_info_header(qtmp4_demuxer_cptr &dmx,
                                           const void *extra_data) {
   int full_size           = sizeof(alBITMAPINFOHEADER) + extra_size;
   memory_cptr bih_p       = memory_c::alloc(full_size);
-  alBITMAPINFOHEADER *bih = (alBITMAPINFOHEADER *)bih_p->get();
+  alBITMAPINFOHEADER *bih = (alBITMAPINFOHEADER *)bih_p->get_buffer();
 
   memset(bih, 0, full_size);
   put_uint32_le(&bih->bi_size,       full_size);
@@ -1281,7 +1454,7 @@ bool
 qtmp4_reader_c::create_audio_packetizer_ac3(qtmp4_demuxer_cptr &dmx) {
   memory_cptr buf = memory_c::alloc(64);
 
-  if (!dmx->read_first_bytes(buf, 64, io) || (-1 == find_ac3_header(buf->get(), buf->get_size(), &dmx->m_ac3_header, false))) {
+  if (!dmx->read_first_bytes(buf, 64, io) || (-1 == find_ac3_header(buf->get_buffer(), buf->get_size(), &dmx->m_ac3_header, false))) {
     mxwarn_tid(ti.fname, dmx->id, Y("No AC3 header found in first frame; track will be skipped.\n"));
     dmx->ok = false;
 
@@ -1299,7 +1472,7 @@ qtmp4_reader_c::create_video_packetizer_svq1(qtmp4_demuxer_cptr &dmx) {
   memory_cptr bih(create_bitmap_info_header(dmx, "SVQ1"));
 
   ti.private_size = bih->get_size();
-  ti.private_data = (unsigned char *)bih->get();
+  ti.private_data = (unsigned char *)bih->get_buffer();
 
   dmx->ptzr       = add_packetizer(new video_packetizer_c(this, ti, MKV_V_MSCOMP, 0.0, dmx->v_width, dmx->v_height));
   ti.private_data = NULL;
@@ -1307,9 +1480,97 @@ qtmp4_reader_c::create_video_packetizer_svq1(qtmp4_demuxer_cptr &dmx) {
   mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the video output module (FourCC: %|1$.4s|).\n")) % dmx->fourcc);
 }
 
+void
+qtmp4_reader_c::create_video_packetizer_mpeg4_p2(qtmp4_demuxer_cptr &dmx) {
+  memory_cptr bih(create_bitmap_info_header(dmx, "DIVX"));
+
+  ti.private_size = bih->get_size();
+  ti.private_data = (unsigned char *)bih->get_buffer();
+  dmx->ptzr       = add_packetizer(new mpeg4_p2_video_packetizer_c(this, ti, 0.0, dmx->v_width, dmx->v_height, false));
+  ti.private_data = NULL;
+
+  mxinfo_tid(ti.fname, dmx->id, Y("Using the MPEG-4 part 2 video output module.\n"));
+}
+
+void
+qtmp4_reader_c::create_video_packetizer_mpeg1_2(qtmp4_demuxer_cptr &dmx) {
+  int version = dmx->fourcc[3] - '0';
+  dmx->ptzr   = add_packetizer(new mpeg1_2_video_packetizer_c(this, ti, version, -1.0, dmx->v_width, dmx->v_height, 0, 0, false));
+
+  mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the MPEG-%1% video output module.\n")) % version);
+}
+
+void
+qtmp4_reader_c::create_video_packetizer_mpeg4_p10(qtmp4_demuxer_cptr &dmx) {
+  if (dmx->frame_offset_table.empty())
+    mxwarn_tid(ti.fname, dmx->id,
+               Y("The AVC video track is missing the 'CTTS' atom for frame timecode offsets. "
+                 "However, AVC/h.264 allows frames to have more than the traditional one (for P frames) or two (for B frames) references to other frames. "
+                 "The timecodes for such frames will be out-of-order, and the 'CTTS' atom is needed for getting the timecodes right. "
+                 "As it is missing the timecodes for this track might be wrong. "
+                 "You should watch the resulting file and make sure that it looks like you expected it to.\n"));
+
+  ti.private_size = dmx->priv_size;
+  ti.private_data = dmx->priv;
+  dmx->ptzr       = add_packetizer(new mpeg4_p10_video_packetizer_c(this, ti, dmx->fps, dmx->v_width, dmx->v_height));
+  ti.private_data = NULL;
+
+  mxinfo_tid(ti.fname, dmx->id, Y("Using the MPEG-4 part 10 (AVC) video output module.\n"));
+}
+
+void
+qtmp4_reader_c::create_video_packetizer_standard(qtmp4_demuxer_cptr &dmx) {
+  ti.private_size = dmx->v_stsd->get_size();
+  ti.private_data = dmx->v_stsd->get_buffer();
+  dmx->ptzr       = add_packetizer(new video_packetizer_c(this, ti, MKV_V_QUICKTIME, 0.0, dmx->v_width, dmx->v_height));
+  ti.private_data = NULL;
+
+  mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the video output module (FourCC: %|1$.4s|).\n")) % dmx->fourcc);
+}
+
+void
+qtmp4_reader_c::create_audio_packetizer_aac(qtmp4_demuxer_cptr &dmx) {
+  ti.private_data = dmx->esds.decoder_config;
+  ti.private_size = dmx->esds.decoder_config_len;
+  dmx->ptzr       = add_packetizer(new aac_packetizer_c(this, ti, AAC_ID_MPEG4, dmx->a_aac_profile, (int)dmx->a_samplerate, dmx->a_channels, false, true));
+  ti.private_data = NULL;
+  ti.private_size = 0;
+
+  if (dmx->a_aac_is_sbr)
+    PTZR(dmx->ptzr)->set_audio_output_sampling_freq(dmx->a_aac_output_sample_rate);
+
+  mxinfo_tid(ti.fname, dmx->id, Y("Using the AAC output module.\n"));
+}
+
+void
+qtmp4_reader_c::create_audio_packetizer_mp3(qtmp4_demuxer_cptr &dmx) {
+  dmx->ptzr = add_packetizer(new mp3_packetizer_c(this, ti, (int32_t)dmx->a_samplerate, dmx->a_channels, true));
+  mxinfo_tid(ti.fname, dmx->id, Y("Using the MPEG audio output module.\n"));
+}
+
+void
+qtmp4_reader_c::create_audio_packetizer_pcm(qtmp4_demuxer_cptr &dmx) {
+  dmx->ptzr = add_packetizer(new pcm_packetizer_c(this, ti, (int32_t)dmx->a_samplerate, dmx->a_channels, dmx->a_bitdepth, (8 < dmx->a_bitdepth) && ('t' == dmx->fourcc[0])));
+  mxinfo_tid(ti.fname, dmx->id, Y("Using the PCM output module.\n"));
+}
+
+void
+qtmp4_reader_c::create_audio_packetizer_passthrough(qtmp4_demuxer_cptr &dmx) {
+  passthrough_packetizer_c *ptzr = new passthrough_packetizer_c(this, ti);
+  dmx->ptzr                      = add_packetizer(ptzr);
+
+  ptzr->set_track_type(track_audio);
+  ptzr->set_codec_id(MKV_A_QUICKTIME);
+  ptzr->set_codec_private(dmx->a_stsd->get_buffer(), dmx->a_stsd->get_size());
+  ptzr->set_audio_sampling_freq(dmx->a_samplerate);
+  ptzr->set_audio_channels(dmx->a_channels);
+
+  mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the generic audio output module (FourCC: %|1$.4s|).\n")) % dmx->fourcc);
+}
+
 void
 qtmp4_reader_c::create_packetizer(int64_t tid) {
-  uint32_t i;
+  unsigned int i;
   qtmp4_demuxer_cptr dmx;
 
   for (i = 0; i < demuxers.size(); ++i)
@@ -1317,109 +1578,59 @@ qtmp4_reader_c::create_packetizer(int64_t tid) {
       dmx = demuxers[i];
       break;
     }
-  if (!dmx.get())
+
+  if (!dmx.is_set() || !dmx->ok || !demuxing_requested(dmx->type, dmx->id) || (-1 != dmx->ptzr))
     return;
 
-  if (!dmx->ok || !demuxing_requested(dmx->type, dmx->id) || (-1 != dmx->ptzr))
-    return;
+  ti.id              = dmx->id;
+  ti.language        = dmx->language;
 
-  ti.id = dmx->id;
+  bool packetizer_ok = true;
 
   if ('v' == dmx->type) {
-    if (!strncasecmp(dmx->fourcc, "mp4v", 4)) {
-      memory_cptr bih(create_bitmap_info_header(dmx, "DIVX"));
+    if (!strncasecmp(dmx->fourcc, "mp4v", 4) || !strncasecmp(dmx->fourcc, "xvid", 4))
+      create_video_packetizer_mpeg4_p2(dmx);
 
-      ti.private_size = bih->get_size();
-      ti.private_data = (unsigned char *)bih->get();
-      dmx->ptzr       = add_packetizer(new mpeg4_p2_video_packetizer_c(this, ti, 0.0, dmx->v_width, dmx->v_height, false));
-      ti.private_data = NULL;
+    else if (!strncasecmp(dmx->fourcc, "mpg1", 4) || !strncasecmp(dmx->fourcc, "mpg2", 4))
+      create_video_packetizer_mpeg1_2(dmx);
 
-      mxinfo_tid(ti.fname, dmx->id, Y("Using the MPEG-4 part 2 video output module.\n"));
+    else if (dmx->v_is_avc)
+      create_video_packetizer_mpeg4_p10(dmx);
 
-    } else if (!strncasecmp(dmx->fourcc, "mpg1", 4) || !strncasecmp(dmx->fourcc, "mpg2", 4)) {
-      int version = dmx->fourcc[3] - '0';
-      dmx->ptzr   = add_packetizer(new mpeg1_2_video_packetizer_c(this, ti, version, -1.0, dmx->v_width, dmx->v_height, 0, 0, false));
-
-      mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the MPEG-%1% video output module.\n")) % version);
-
-    } else if (dmx->v_is_avc) {
-      if (dmx->frame_offset_table.empty())
-        mxwarn_tid(ti.fname, dmx->id,
-                   Y("The AVC video track is missing the 'CTTS' atom for frame timecode offsets. "
-                     "However, AVC/h.264 allows frames to have more than the traditional one (for P frames) or two (for B frames) references to other frames. "
-                     "The timecodes for such frames will be out-of-order, and the 'CTTS' atom is needed for getting the timecodes right. "
-                     "As it is missing the timecodes for this track might be wrong. "
-                     "You should watch the resulting file and make sure that it looks like you expected it to.\n"));
-
-      ti.private_size = dmx->priv_size;
-      ti.private_data = dmx->priv;
-      dmx->ptzr       = add_packetizer(new mpeg4_p10_video_packetizer_c(this, ti, dmx->fps, dmx->v_width, dmx->v_height));
-      ti.private_data = NULL;
-
-      mxinfo_tid(ti.fname, dmx->id, Y("Using the MPEG-4 part 10 (AVC) video output module.\n"));
-
-    } else if (!strncasecmp(dmx->fourcc, "svq1", 4))
+    else if (!strncasecmp(dmx->fourcc, "svq1", 4))
       create_video_packetizer_svq1(dmx);
 
-    else {
-      ti.private_size = dmx->v_stsd_size;
-      ti.private_data = (unsigned char *)dmx->v_stsd;
-      dmx->ptzr       = add_packetizer(new video_packetizer_c(this, ti, MKV_V_QUICKTIME, 0.0, dmx->v_width, dmx->v_height));
-      ti.private_data = NULL;
+    else
+      create_video_packetizer_standard(dmx);
 
-      mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the video output module (FourCC: %|1$.4s|).\n")) % dmx->fourcc);
-    }
 
   } else {
-    if (!strncasecmp(dmx->fourcc, "QDMC", 4) || !strncasecmp(dmx->fourcc, "QDM2", 4)) {
-      passthrough_packetizer_c *ptzr = new passthrough_packetizer_c(this, ti);
-      dmx->ptzr                      = add_packetizer(ptzr);
+    if (!strncasecmp(dmx->fourcc, "MP4A", 4) && IS_AAC_OBJECT_TYPE_ID(dmx->esds.object_type_id))
+      create_audio_packetizer_aac(dmx);
 
-      ptzr->set_track_type(track_audio);
-      ptzr->set_codec_id('2' == dmx->fourcc[3] ? MKV_A_QDMC2 : MKV_A_QDMC);
-      ptzr->set_codec_private(dmx->priv, dmx->priv_size);
-      ptzr->set_audio_sampling_freq(dmx->a_samplerate);
-      ptzr->set_audio_channels(dmx->a_channels);
-      ptzr->set_audio_bit_depth(dmx->a_bitdepth);
+    else if (!strncasecmp(dmx->fourcc, "MP4A", 4) && ((MP4OTI_MPEG2AudioPart3 == dmx->esds.object_type_id) || (MP4OTI_MPEG1Audio == dmx->esds.object_type_id)))
+      create_audio_packetizer_mp3(dmx);
 
-      mxinfo_tid(ti.fname, dmx->id, boost::format(Y("Using the generic audio output module (FourCC: %|1$.4s|).\n")) % dmx->fourcc);
+    else if (!strncasecmp(dmx->fourcc, ".mp3", 4))
+      create_audio_packetizer_mp3(dmx);
 
-    } else if (!strncasecmp(dmx->fourcc, "MP4A", 4) && IS_AAC_OBJECT_TYPE_ID(dmx->esds.object_type_id)) {
-      ti.private_data = dmx->esds.decoder_config;
-      ti.private_size = dmx->esds.decoder_config_len;
-      dmx->ptzr       = add_packetizer(new aac_packetizer_c(this, ti, AAC_ID_MPEG4, dmx->a_aac_profile, (int)dmx->a_samplerate, dmx->a_channels, false, true));
-      ti.private_data = NULL;
-      ti.private_size = 0;
+    else if (!strncasecmp(dmx->fourcc, "twos", 4) || !strncasecmp(dmx->fourcc, "sowt", 4))
+      create_audio_packetizer_pcm(dmx);
 
-      if (dmx->a_aac_is_sbr)
-        PTZR(dmx->ptzr)->set_audio_output_sampling_freq(dmx->a_aac_output_sample_rate);
+    else if (!strncasecmp(dmx->fourcc, "ac-3", 4) || !strncasecmp(dmx->fourcc, "sac3", 4))
+      packetizer_ok = create_audio_packetizer_ac3(dmx);
 
-      mxinfo_tid(ti.fname, dmx->id, Y("Using the AAC output module.\n"));
-
-    } else if (!strncasecmp(dmx->fourcc, "MP4A", 4) && ((MP4OTI_MPEG2AudioPart3 == dmx->esds.object_type_id) || (MP4OTI_MPEG1Audio == dmx->esds.object_type_id))) {
-      dmx->ptzr = add_packetizer(new mp3_packetizer_c(this, ti, (int32_t)dmx->a_samplerate, dmx->a_channels, true));
-      mxinfo_tid(ti.fname, dmx->id, Y("Using the MPEG audio output module.\n"));
-
-    } else if (!strncasecmp(dmx->fourcc, "twos", 4) || !strncasecmp(dmx->fourcc, "swot", 4)) {
-      dmx->ptzr = add_packetizer(new pcm_packetizer_c(this, ti, (int32_t)dmx->a_samplerate, dmx->a_channels, dmx->a_bitdepth,
-                                                      (8 < dmx->a_bitdepth) && ('t' == dmx->fourcc[0])));
-      mxinfo_tid(ti.fname, dmx->id, Y("Using the PCM output module.\n"));
-
-    } else if (!strncasecmp(dmx->fourcc, "ac-3", 4)) {
-      if (!create_audio_packetizer_ac3(dmx))
-        return;
-
-    } else
-      mxerror(Y("Quicktime/MP4 reader: Should not have happened #1."));
+    else
+      create_audio_packetizer_passthrough(dmx);
   }
 
-  if (-1 == main_dmx)
+  if (packetizer_ok && (-1 == main_dmx))
     main_dmx = i;
 }
 
 void
 qtmp4_reader_c::create_packetizers() {
-  uint32_t i;
+  unsigned int i;
 
   main_dmx = -1;
 
@@ -1429,21 +1640,19 @@ qtmp4_reader_c::create_packetizers() {
 
 int
 qtmp4_reader_c::get_progress() {
-  uint32_t max_chunks;
+  if (-1 == main_dmx)
+    return 100;
 
   qtmp4_demuxer_cptr &dmx = demuxers[main_dmx];
-  if (dmx->sample_size == 0)
-    max_chunks = dmx->sample_table.size();
-  else
-    max_chunks = dmx->chunk_table.size();
+  unsigned int max_chunks = (0 == dmx->sample_size) ? dmx->sample_table.size() : dmx->chunk_table.size();
 
   return 100 * dmx->pos / max_chunks;
 }
 
 void
 qtmp4_reader_c::identify() {
-  vector<string> verbose_info;
-  int i;
+  std::vector<std::string> verbose_info;
+  unsigned int i;
 
   id_result_container("Quicktime/MP4");
 
@@ -1455,19 +1664,60 @@ qtmp4_reader_c::identify() {
     if (dmx->v_is_avc)
       verbose_info.push_back("packetizer:mpeg4_p10_video");
 
+    if (!dmx->language.empty())
+      verbose_info.push_back((boost::format("language:%1%") % dmx->language).str());
+
     id_result_track(dmx->id, dmx->type == 'v' ? ID_RESULT_TRACK_VIDEO : dmx->type == 'a' ? ID_RESULT_TRACK_AUDIO : ID_RESULT_TRACK_UNKNOWN,
                     (boost::format("%|1$.4s|") %  dmx->fourcc).str(), verbose_info);
   }
+
+  if (NULL != chapters)
+    id_result_chapters(count_chapter_atoms(*chapters));
 }
 
 void
 qtmp4_reader_c::add_available_track_ids() {
-  int i;
+  unsigned int i;
 
-  for (i =0 ; i < demuxers.size(); ++i)
+  for (i = 0; i < demuxers.size(); ++i)
     available_track_ids.push_back(demuxers[i]->id);
 }
 
+std::string
+qtmp4_reader_c::decode_and_verify_language(uint16_t coded_language) {
+  std::string language;
+  int i;
+
+  for (i = 0; 3 > i; ++i)
+    language += (char)(((coded_language >> ((2 - i) * 5)) & 0x1f) + 0x60);
+
+  language = downcase(language);
+
+  if (is_valid_iso639_2_code(language.c_str()))
+    return language;
+
+  return "";
+}
+
+void
+qtmp4_reader_c::recode_chapter_entries(std::vector<qtmp4_chapter_entry_t> &entries) {
+  int i;
+
+  if (g_identifying) {
+    for (i = 0; entries.size() > i; ++i)
+      entries[i].m_name = empty_string;
+    return;
+  }
+
+  std::string charset              = ti.chapter_charset.empty() ? "UTF-8" : ti.chapter_charset;
+  charset_converter_cptr converter = charset_converter_c::init(ti.chapter_charset);
+
+  for (i = 0; entries.size() > i; ++i)
+    entries[i].m_name = converter->utf8(entries[i].m_name);
+}
+
+// ----------------------------------------------------------------------
+
 void
 qtmp4_demuxer_c::calculate_fps() {
   fps = 0.0;
@@ -1478,7 +1728,7 @@ qtmp4_demuxer_c::calculate_fps() {
     mxverb(3, boost::format("Quicktime/MP4 reader: calculate_fps: case 1: %1%\n") % fps);
 
   } else {
-    map<int64_t, int> duration_map;
+    std::map<int64_t, int> duration_map;
 
     for (int i = 0; sample_table.size() > (i + 1); ++i) {
       int64_t this_duration = sample_table[i + 1].pts - sample_table[i].pts;
@@ -1488,8 +1738,8 @@ qtmp4_demuxer_c::calculate_fps() {
       ++duration_map[this_duration];
     }
 
-    map<int64_t, int>::const_iterator most_common = duration_map.begin();
-    map<int64_t, int>::const_iterator it;
+    std::map<int64_t, int>::const_iterator most_common = duration_map.begin();
+    std::map<int64_t, int>::const_iterator it;
     mxforeach(it, duration_map)
       if (it->second > most_common->second)
         most_common = it;
@@ -1534,7 +1784,7 @@ qtmp4_demuxer_c::calculate_timecodes() {
   }
 
   int64_t v_dts_offset = 0;
-  vector<int64_t> timecodes_before_offsets;
+  std::vector<int64_t> timecodes_before_offsets;
 
   if (('v' == type) && v_is_avc && !frame_offset_table.empty())
     v_dts_offset = to_nsecs(frame_offset_table[0]);
@@ -1726,7 +1976,7 @@ qtmp4_demuxer_c::update_editlist_table(int64_t global_time_scale) {
   if (('v' == type) && v_is_avc && !frame_offset_table.empty() && (frame_offset_table[0] <= min_editlist_pts))
     pts_offset = frame_offset_table[0];
 
-  mxverb(4, boost::format("qtmp4: Updating edit list table for track %1%; pts_offset = %2%\n") % id % pts_offset);
+  mxverb(4, boost::format("qtmp4: Updating edit std::list table for track %1%; pts_offset = %2%\n") % id % pts_offset);
 
   for (i = 0; editlist_table.size() > i; ++i) {
     qt_editlist_t &el = editlist_table[i];
@@ -1790,11 +2040,12 @@ qtmp4_demuxer_c::build_index_constant_sample_size_mode() {
       frame_size = chunk_table[frame_idx].size;
 
       if ('a' == type) {
-        if (get_uint16_be(&a_stsd.v0.version) == 1) {
-          frame_size *= get_uint32_be(&a_stsd.v1.bytes_per_frame);
-          frame_size /= get_uint32_be(&a_stsd.v1.samples_per_packet);
+        sound_v1_stsd_atom_t *sound_stsd_atom = (sound_v1_stsd_atom_t *)a_stsd->get_buffer();
+        if (get_uint16_be(&sound_stsd_atom->v0.version) == 1) {
+          frame_size *= get_uint32_be(&sound_stsd_atom->v1.bytes_per_frame);
+          frame_size /= get_uint32_be(&sound_stsd_atom->v1.samples_per_packet);
         } else
-          frame_size  = frame_size * a_channels * get_uint16_be(&a_stsd.v0.sample_size) / 8;
+          frame_size  = frame_size * a_channels * get_uint16_be(&sound_stsd_atom->v0.sample_size) / 8;
       }
     }
 
@@ -1843,7 +2094,7 @@ qtmp4_demuxer_c::read_first_bytes(memory_cptr &buf,
     int num_bytes_to_read = std::min((int64_t)num_bytes, index.size);
 
     io->setFilePointer(index.file_pos);
-    if (io->read(buf->get() + buf_pos, num_bytes_to_read) < num_bytes_to_read)
+    if (io->read(buf->get_buffer() + buf_pos, num_bytes_to_read) < num_bytes_to_read)
       return false;
 
     num_bytes -= num_bytes_to_read;
