@@ -14,18 +14,18 @@
 #ifndef __R_QTMP4_H
 #define __R_QTMP4_H
 
-#include "os.h"
+#include "common/os.h"
 
 #include <stdio.h>
 
 #include <vector>
 
-#include "ac3_common.h"
-#include "common.h"
-#include "mm_io.h"
-#include "p_video.h"
-#include "qtmp4_atoms.h"
-#include "smart_pointers.h"
+#include "common/ac3.h"
+#include "common/common.h"
+#include "common/mm_io.h"
+#include "output/p_video.h"
+#include "input/qtmp4_atoms.h"
+#include "common/smart_pointers.h"
 
 struct qt_durmap_t {
   uint32_t number;
@@ -133,18 +133,18 @@ struct qtmp4_demuxer_c {
 
   uint64_t duration;
 
-  vector<qt_sample_t> sample_table;
-  vector<qt_chunk_t> chunk_table;
-  vector<qt_chunkmap_t> chunkmap_table;
-  vector<qt_durmap_t> durmap_table;
-  vector<uint32_t> keyframe_table;
-  vector<qt_editlist_t> editlist_table;
-  vector<qt_frame_offset_t> raw_frame_offset_table;
-  vector<int32_t> frame_offset_table;
+  std::vector<qt_sample_t> sample_table;
+  std::vector<qt_chunk_t> chunk_table;
+  std::vector<qt_chunkmap_t> chunkmap_table;
+  std::vector<qt_durmap_t> durmap_table;
+  std::vector<uint32_t> keyframe_table;
+  std::vector<qt_editlist_t> editlist_table;
+  std::vector<qt_frame_offset_t> raw_frame_offset_table;
+  std::vector<int32_t> frame_offset_table;
 
-  vector<int64_t> timecodes, durations, frame_indices;
+  std::vector<int64_t> timecodes, durations, frame_indices;
 
-  vector<qt_index_t> m_index;
+  std::vector<qt_index_t> m_index;
 
   int64_t min_timecode, max_timecode;
   double fps;
@@ -152,14 +152,14 @@ struct qtmp4_demuxer_c {
   esds_t esds;
   bool esds_parsed;
 
-  unsigned char *v_stsd;
+  memory_cptr v_stsd;
   uint32_t v_stsd_size;
   uint32_t v_width, v_height, v_bitdepth;
-  deque<int64_t> references;
+  std::deque<int64_t> references;
   bool v_is_avc, avc_use_bframes;
+  memory_cptr a_stsd;
   uint32_t a_channels, a_bitdepth;
   float a_samplerate;
-  sound_v1_stsd_atom_t a_stsd;
   int a_aac_profile, a_aac_output_sample_rate;
   bool a_aac_is_sbr, a_aac_config_parsed;
   ac3_header_t m_ac3_header;
@@ -171,6 +171,8 @@ struct qtmp4_demuxer_c {
 
   int ptzr;
 
+  std::string language;
+
   qtmp4_demuxer_c():
     ok(false),
     type('?'),
@@ -184,8 +186,6 @@ struct qtmp4_demuxer_c {
     max_timecode(0),
     fps(0.0),
     esds_parsed(false),
-    v_stsd(NULL),
-    v_stsd_size(0),
     v_width(0),
     v_height(0),
     v_bitdepth(0),
@@ -205,11 +205,9 @@ struct qtmp4_demuxer_c {
 
     memset(fourcc, 0, 4);
     memset(&esds, 0, sizeof(esds_t));
-    memset(&a_stsd, 0, sizeof(sound_v1_stsd_atom_t));
   }
 
   ~qtmp4_demuxer_c() {
-    safefree(v_stsd);
     safefree(priv);
     safefree(esds.decoder_config);
     safefree(esds.sl_config);
@@ -255,10 +253,27 @@ struct qt_atom_t {
   }
 };
 
+struct qtmp4_chapter_entry_t {
+  std::string m_name;
+  int64_t m_timecode;
+
+  qtmp4_chapter_entry_t(const std::string &name,
+                        int64_t timecode)
+    : m_name(name)
+    , m_timecode(timecode)
+  {
+  }
+
+  bool operator <(const qtmp4_chapter_entry_t &cmp) const {
+    return m_timecode < cmp.m_timecode;
+  }
+};
+
 class qtmp4_reader_c: public generic_reader_c {
 private:
   mm_io_c *io;
-  vector<qtmp4_demuxer_cptr> demuxers;
+  std::vector<qtmp4_demuxer_cptr> demuxers;
+  qtmp4_demuxer_cptr chapter_dmx;
   int64_t file_size, mdat_pos, mdat_size;
   uint32_t time_scale, compression_algorithm;
   int main_dmx;
@@ -295,6 +310,8 @@ protected:
   virtual void handle_minf_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
   virtual void handle_moov_atom(qt_atom_t parent, int level);
   virtual void handle_mvhd_atom(qt_atom_t parent, int level);
+  virtual void handle_udta_atom(qt_atom_t parent, int level);
+  virtual void handle_chpl_atom(qt_atom_t parent, int level);
   virtual void handle_stbl_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
   virtual void handle_stco_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
   virtual void handle_co64_atom(qtmp4_demuxer_cptr &new_dmx, qt_atom_t parent, int level);
@@ -311,8 +328,22 @@ protected:
 
   virtual memory_cptr create_bitmap_info_header(qtmp4_demuxer_cptr &dmx, const char *fourcc, int extra_size = 0, const void *extra_data = NULL);
 
-  virtual void create_video_packetizer_svq1(qtmp4_demuxer_cptr &dmx);
+  virtual void create_audio_packetizer_aac(qtmp4_demuxer_cptr &dmx);
   virtual bool create_audio_packetizer_ac3(qtmp4_demuxer_cptr &dmx);
+  virtual void create_audio_packetizer_mp3(qtmp4_demuxer_cptr &dmx);
+  virtual void create_audio_packetizer_passthrough(qtmp4_demuxer_cptr &dmx);
+  virtual void create_audio_packetizer_pcm(qtmp4_demuxer_cptr &dmx);
+
+  virtual void create_video_packetizer_mpeg1_2(qtmp4_demuxer_cptr &dmx);
+  virtual void create_video_packetizer_mpeg4_p10(qtmp4_demuxer_cptr &dmx);
+  virtual void create_video_packetizer_mpeg4_p2(qtmp4_demuxer_cptr &dmx);
+  virtual void create_video_packetizer_standard(qtmp4_demuxer_cptr &dmx);
+  virtual void create_video_packetizer_svq1(qtmp4_demuxer_cptr &dmx);
+
+  virtual std::string decode_and_verify_language(uint16_t coded_language);
+  virtual void read_chapter_track();
+  virtual void recode_chapter_entries(std::vector<qtmp4_chapter_entry_t> &entries);
+  virtual void process_chapter_entries(int level, std::vector<qtmp4_chapter_entry_t> &entries);
 };
 
 #endif  // __R_QTMP4_H
