@@ -6,30 +6,25 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_vobsub.cpp 3952 2008-10-04 17:54:56Z mosu $
-
    VobSub stream reader
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
-#include <ctype.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
+#include "common/common.h"
+#include "common/hacks.h"
+#include "common/iso639.h"
+#include "common/endian.h"
+#include "common/mm_io.h"
+#include "common/strings/formatting.h"
+#include "common/strings/parsing.h"
+#include "input/r_vobsub.h"
+#include "input/subtitles.h"
+#include "merge/output_control.h"
+#include "output/p_vobsub.h"
 
-#include "common.h"
-#include "iso639.h"
-#include "mm_io.h"
-#include "output_control.h"
-#include "p_vobsub.h"
-#include "r_vobsub.h"
-#include "subtitles.h"
-
-using namespace std;
 
 #define hexvalue(c) (  isdigit(c)        ? (c) - '0' \
                      : tolower(c) == 'a' ? 10        \
@@ -40,7 +35,7 @@ using namespace std;
                      :                     15)
 #define ishexdigit(s) (isdigit(s) || (strchr("abcdefABCDEF", s) != NULL))
 
-const string vobsub_reader_c::id_string("# VobSub index file, v");
+const std::string vobsub_reader_c::id_string("# VobSub index file, v");
 
 bool
 vobsub_entry_c::operator < (const vobsub_entry_c &cmp) const {
@@ -76,8 +71,8 @@ vobsub_reader_c::vobsub_reader_c(track_info_c &_ti)
     throw error_c(Y("vobsub_reader: Cound not open the source file."));
   }
 
-  string sub_name = ti.fname;
-  int len         = sub_name.rfind(".");
+  std::string sub_name = ti.fname;
+  int len              = sub_name.rfind(".");
   if (0 <= len)
     sub_name.erase(len);
   sub_name += ".sub";
@@ -91,7 +86,7 @@ vobsub_reader_c::vobsub_reader_c(track_info_c &_ti)
   idx_data = "";
   len      = id_string.length();
 
-  string line;
+  std::string line;
   if (!idx_file->getline2(line) || !starts_with_case(line, id_string.c_str(), len) || (line.length() < (len + 1)))
     mxerror_fn(ti.fname, Y("No version number found.\n"));
 
@@ -115,7 +110,7 @@ vobsub_reader_c::~vobsub_reader_c() {
 
   for (i = 0; i < tracks.size(); i++) {
     mxverb(2,
-           boost::format(Y("r_vobsub track %1% SPU size: %2%, overall size: %3%, overhead: %4% (%|5$.3f|%%)\n"))
+           boost::format("r_vobsub track %1% SPU size: %2%, overall size: %3%, overhead: %4% (%|5$.3f|%%)\n")
            % i % tracks[i]->spu_size % (tracks[i]->spu_size + tracks[i]->overhead) % tracks[i]->overhead
            % (float)(100.0 * tracks[i]->overhead / (tracks[i]->overhead + tracks[i]->spu_size)));
     delete tracks[i];
@@ -166,7 +161,7 @@ vobsub_reader_c::create_packetizers() {
 
 void
 vobsub_reader_c::parse_headers() {
-  string language, line;
+  std::string language, line;
   vobsub_track_c *track  = NULL;
   int64_t line_no        = 0;
   int64_t last_timestamp = 0;
@@ -174,6 +169,8 @@ vobsub_reader_c::parse_headers() {
   num_indices            = 0;
   indices_processed      = 0;
 
+  idx_file->setFilePointer(0, seek_beginning);
+
   while (1) {
     if (!idx_file->getline2(line))
       break;
@@ -202,7 +199,7 @@ vobsub_reader_c::parse_headers() {
         else {
           tracks.push_back(track);
           if (sort_required) {
-            mxverb(2, boost::format(Y("vobsub_reader: Sorting track %1%\n"))  % tracks.size());
+            mxverb(2, boost::format("vobsub_reader: Sorting track %1%\n") % tracks.size());
             stable_sort(track->entries.begin(), track->entries.end());
           }
         }
@@ -232,7 +229,7 @@ vobsub_reader_c::parse_headers() {
 
       strip(line);
       shrink_whitespace(line);
-      vector<string> parts = split(line.c_str(), " ");
+      std::vector<std::string> parts = split(line.c_str(), " ");
 
       if ((4 != parts.size()) || (13 > parts[1].length()) || (downcase(parts[2]) != "filepos:")) {
         mxwarn_fn(ti.fname, boost::format(Y("Line %1%: The line seems to be a subtitle entry but the format couldn't be recognized. This entry will be skipped.\n")) % line_no);
@@ -279,10 +276,10 @@ vobsub_reader_c::parse_headers() {
           demuxing_requested('s', tracks.size())) {
         mxwarn_fn(ti.fname,
                   boost::format(Y("Line %1%: The current timestamp (%2%) is smaller than the previous one (%3%). "
-                                  "mkvmerge will sort the entries according to their timestamps. "
+                                  "The entries will be sorted according to their timestamps. "
                                   "This might result in the wrong order for some subtitle entries. "
                                   "If this is the case then you have to fix the .idx file manually.\n"))
-                  % line_no % format_timecode(entry.timestamp * 1000000, 3) % format_timecode(last_timestamp * 1000000, 3));
+                  % line_no % format_timecode(entry.timestamp, 3) % format_timecode(last_timestamp, 3));
         sort_required = true;
       }
       last_timestamp = entry.timestamp;
@@ -300,7 +297,7 @@ vobsub_reader_c::parse_headers() {
     else {
       tracks.push_back(track);
       if (sort_required) {
-        mxverb(2, boost::format(Y("vobsub_reader: Sorting track %1%\n")) % tracks.size());
+        mxverb(2, boost::format("vobsub_reader: Sorting track %1%\n") % tracks.size());
         stable_sort(track->entries.begin(), track->entries.end());
       }
     }
@@ -309,9 +306,9 @@ vobsub_reader_c::parse_headers() {
   if (!g_identifying && (1 < verbose)) {
     int i, k, tsize = tracks.size();
     for (i = 0; i < tsize; i++) {
-      mxinfo(boost::format(Y("vobsub_reader: Track number %1%\n")) % i);
+      mxinfo(boost::format("vobsub_reader: Track number %1%\n") % i);
       for (k = 0; k < tracks[i]->entries.size(); k++)
-        mxinfo(boost::format(Y("vobsub_reader:  %|1$04u| position: %|2$12d| (0x%|3$04x|%|4$08x|), timecode: %|5$12d| (%6%)\n"))
+        mxinfo(boost::format("vobsub_reader:  %|1$04u| position: %|2$12d| (0x%|3$04x|%|4$08x|), timecode: %|5$12d| (%6%)\n")
                % k % tracks[i]->entries[k].position % (tracks[i]->entries[k].position >> 32) % (tracks[i]->entries[k].position & 0xffffffff)
                % (tracks[i]->entries[k].timestamp / 1000000) % format_timecode(tracks[i]->entries[k].timestamp, 3));
     }
@@ -325,17 +322,73 @@ vobsub_reader_c::deliver_packet(unsigned char *buf,
                                 int64_t timecode,
                                 int64_t default_duration,
                                 generic_packetizer_c *ptzr) {
-  int64_t duration;
-
   if ((NULL == buf) || (0 == size)) {
     safefree(buf);
     return -1;
   }
 
-  duration = spu_extract_duration(buf, size, timecode);
+  int64_t duration = spu_extract_duration(buf, size, timecode);
   if (1 == -duration) {
-    mxverb_tid(2, ti.fname, ti.id, boost::format(Y("vobsub_reader: Could not extract the duration for a SPU packet (timecode: %1%.\n")) % format_timecode(timecode, 3));
     duration = default_duration;
+    mxverb(2, boost::format("vobsub_reader: Could not extract the duration for a SPU packet (timecode: %1%).") % format_timecode(timecode, 3));
+
+    int dcsq  =                   get_uint16_be(&buf[2]);
+    int dcsq2 = dcsq + 3 < size ? get_uint16_be(&buf[dcsq + 2]) : -1;
+
+    // Some players ignore sub-pictures if there is no stop display command.
+    // Add a stop display command only if 1 command chain exists and the hack is enabled.
+
+    if ((dcsq == dcsq2) && hack_engaged(ENGAGE_VOBSUB_SUBPIC_STOP_CMDS)) {
+      dcsq         += 2;        // points to first command chain
+      dcsq2        += 4;        // find end of first command chain
+      bool unknown  = false;
+      unsigned char type;
+      for (type = buf[dcsq2++]; 0xff != type; type = buf[dcsq2++]) {
+        switch(type) {
+          case 0x00:            // Menu ID, 1 byte
+            break;
+          case 0x01:            // Start display
+            break;
+          case 0x02:            // Stop display
+            unknown = true;     // Can only have one Stop Display command
+            break;
+          case 0x03:            // Palette
+            dcsq2 += 2;
+            break;
+          case 0x04:            // Alpha
+            dcsq2 += 2;
+            break;
+          case 0x05:            // Coords
+            dcsq2 += 6;
+            break;
+          case 0x06:            // Graphic lines
+            dcsq2 += 4;
+            break;
+          default:
+            unknown = true;
+        }
+      }
+
+      if (!unknown) {
+        size          += 6;
+        uint32_t len   = (buf[0] << 8 | buf[1]) + 6; // increase sub-picture length by 6
+        buf[0]         = (uint8_t)(len >> 8);
+        buf[1]         = (uint8_t)(len);
+        uint32_t stm   = duration / 1000000;         // calculate STM for Stop Display
+        stm            = ((stm - 1) * 90 >> 10) + 1;
+        buf[dcsq]      = (uint8_t)(dcsq2 >> 8);      // set pointer to 2nd chain
+        buf[dcsq + 1]  = (uint8_t)(dcsq2);
+        buf[dcsq2]     = (uint8_t)(stm >> 8);        // set DCSQ_STM
+        buf[dcsq2 + 1] = (uint8_t)(stm);
+        buf[dcsq2 + 2] = (uint8_t)(dcsq2 >> 8);      // last chain so point to itself
+        buf[dcsq2 + 3] = (uint8_t)(dcsq2);
+        buf[dcsq2 + 4] = 0x02;                       // stop display command
+        buf[dcsq2 + 5] = 0xff;                       // end command
+        mxverb(2, boost::format(" Added Stop Display cmd (SP_DCSQ_STM=0x%|1$04x|)") % stm);
+      }
+    }
+
+    mxverb(2, boost::format("\n"));
   }
 
   if (2 != -duration)
@@ -359,7 +412,7 @@ vobsub_reader_c::extract_one_spu_packet(int64_t track_id) {
   int64_t timecode             = track->entries[track->idx].timestamp;
   int64_t duration             = track->entries[track->idx].duration;
   int64_t extraction_start_pos = track->entries[track->idx].position;
-  int64_t extraction_end_pos   = track->idx >= track->entries.size() ? sub_file->get_size() : track->entries[track->idx + 1].position;
+  int64_t extraction_end_pos   = track->idx >= track->entries.size() - 1 ? sub_file->get_size() : track->entries[track->idx + 1].position;
 
   int64_t pts                  = 0;
   unsigned char *dst_buf       = NULL;
@@ -374,8 +427,11 @@ vobsub_reader_c::extract_one_spu_packet(int64_t track_id) {
     if ((spu_len >= 0) && ((dst_size >= spu_len) || (sub_file->getFilePointer() >= extraction_end_pos))) {
       if (dst_size != spu_len)
         mxverb(3,
-               boost::format(Y("r_vobsub.cpp: stddeliver spu_len %1% dst_size %2% curpos %3% endpos %4%\n"))
-               % spu_len % dst_size % sub_file->getFilePointer() % extraction_end_pos);
+               boost::format("r_vobsub.cpp: stddeliver spu_len different from dst_size; pts %5% spu_len %1% dst_size %2% curpos %3% endpos %4%\n")
+               % spu_len % dst_size % sub_file->getFilePointer() % extraction_end_pos % format_timecode(pts));
+      if (2 < dst_size)
+        put_uint16_be(dst_buf, dst_size);
+
       return deliver();
     }
     if (sub_file->read(buf, 4) != 4)
@@ -483,14 +539,22 @@ vobsub_reader_c::extract_one_spu_packet(int64_t track_id) {
           else if (track->aid != packet_aid) {
             // The packet does not belong to the current subtitle stream.
             mxverb(3,
-                   boost::format(Y("vobsub_reader: skipping sub packet with aid %1% (wanted aid: %2%) with size %3% at %4%\n"))
+                   boost::format("vobsub_reader: skipping sub packet with aid %1% (wanted aid: %2%) with size %3% at %4%\n")
                    % packet_aid % track->aid % packet_size % (sub_file->getFilePointer() - extraction_start_pos));
             sub_file->skip(packet_size);
             idx = len;
             break;
           }
-          dst_buf = (unsigned char *)saferealloc(dst_buf, dst_size + packet_size);
-          mxverb(3, boost::format(Y("vobsub_reader: sub packet data: aid: %1%, pts: %2%, packet_size: %3%\n")) % track->aid % format_timecode(pts, 3) % packet_size);
+
+          if (hack_engaged(ENGAGE_VOBSUB_SUBPIC_STOP_CMDS)) {
+            // Space for the stop display command and padding
+            dst_buf = (unsigned char *)saferealloc(dst_buf, dst_size + packet_size + 6);
+            memset(dst_buf + dst_size + packet_size, 0xff, 6);
+
+          } else
+            dst_buf = (unsigned char *)saferealloc(dst_buf, dst_size + packet_size);
+
+          mxverb(3, boost::format("vobsub_reader: sub packet data: aid: %1%, pts: %2%, packet_size: %3%\n") % track->aid % format_timecode(pts, 3) % packet_size);
           if (sub_file->read(&dst_buf[dst_size], packet_size) != packet_size) {
             mxwarn(Y("vobsub_reader: sub_file->read failure"));
             return deliver();
@@ -567,7 +631,7 @@ vobsub_reader_c::get_progress() {
 
 void
 vobsub_reader_c::identify() {
-  vector<string> verbose_info;
+  std::vector<std::string> verbose_info;
   int i;
 
   id_result_container("VobSub");
@@ -576,7 +640,7 @@ vobsub_reader_c::identify() {
     verbose_info.clear();
 
     if (!tracks[i]->language.empty())
-      verbose_info.push_back(string("language:") + tracks[i]->language);
+      verbose_info.push_back(std::string("language:") + tracks[i]->language);
 
     id_result_track(i, ID_RESULT_TRACK_SUBTITLES, "VobSub", verbose_info);
   }
