@@ -6,8 +6,6 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: r_wav.h 3770 2008-02-29 14:28:35Z mosu $
-
    class definitions for the WAV reader module
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
@@ -17,15 +15,15 @@
 #ifndef __R_WAV_H
 #define __R_WAV_H
 
-#include "os.h"
+#include "common/os.h"
 
 #include <stdio.h>
 
-#include "common.h"
-#include "dts_common.h"
-#include "error.h"
-#include "mm_io.h"
-#include "pr_generic.h"
+#include "common/common.h"
+#include "common/dts.h"
+#include "common/error.h"
+#include "common/mm_io.h"
+#include "merge/pr_generic.h"
 
 extern "C" {
 #include "avilib.h"
@@ -41,7 +39,9 @@ public:
 
 public:
   wav_demuxer_c(wav_reader_c *reader, wave_header *wheader):
-    m_reader(reader), m_wheader(wheader), m_ptzr(NULL) {
+    m_reader(reader),
+    m_wheader(wheader),
+    m_ptzr(NULL) {
   };
   virtual ~wav_demuxer_c() {};
 
@@ -51,7 +51,7 @@ public:
 
   virtual generic_packetizer_c *create_packetizer() = 0;
 
-  virtual string get_codec() = 0;
+  virtual std::string get_codec() = 0;
 
   virtual bool probe(mm_io_cptr &io) = 0;
 };
@@ -61,7 +61,7 @@ typedef counted_ptr<wav_demuxer_c> wav_demuxer_cptr;
 struct wav_chunk_t {
   int64_t pos;
   char id[4];
-  uint32_t len;
+  int64_t len;
 };
 
 class wav_reader_c: public generic_reader_c {
@@ -70,7 +70,7 @@ private:
   struct wave_header m_wheader;
   int64_t m_bytes_processed, m_bytes_in_data_chunks, m_remaining_bytes_in_current_data_chunk;
 
-  vector<wav_chunk_t> m_chunks;
+  std::vector<wav_chunk_t> m_chunks;
   int m_cur_data_chunk_idx;
 
   wav_demuxer_cptr m_demuxer;
@@ -91,6 +91,8 @@ protected:
   void scan_chunks();
   int find_chunk(const char *id, int start_idx = 0, bool allow_empty = true);
 
+  void dump_headers();
+
   void parse_file();
   void create_demuxer();
 };
