@@ -11,13 +11,17 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
-#include "common.h"
-#include "extern_data.h"
-#include "mm_io.h"
-#include "output_control.h"
-#include "subtitles.h"
+#include "common/common.h"
+#include "common/endian.h"
+#include "common/extern_data.h"
+#include "common/locale.h"
+#include "common/mm_io.h"
+#include "common/strings/formatting.h"
+#include "common/strings/parsing.h"
+#include "input/subtitles.h"
+#include "merge/output_control.h"
 
 // ------------------------------------------------------------
 
@@ -32,7 +36,7 @@ subtitles_c::process(generic_packetizer_c *p) {
 
 // ------------------------------------------------------------
 
-#define SRT_RE_TIMECODE      "\\s*(\\d+):\\s*(\\d+):\\s*(\\d+)[,\\.]\\s*(\\d+)?"
+#define SRT_RE_TIMECODE      "\\s*(-?)\\s*(\\d+):\\s*(\\d+):\\s*(\\d+)[,\\.]\\s*(\\d+)?"
 #define SRT_RE_TIMECODE_LINE "^" SRT_RE_TIMECODE "\\s*-+>\\s*" SRT_RE_TIMECODE "\\s*"
 #define SRT_RE_COORDINATES   "([XY]\\d+:\\d+\\s*){4}\\s*$"
 
@@ -40,7 +44,7 @@ bool
 srt_parser_c::probe(mm_text_io_c *io) {
   try {
     io->setFilePointer(0, seek_beginning);
-    string s = io->getline();
+    std::string s = io->getline();
     strip(s);
 
     int64_t dummy;
@@ -49,7 +53,7 @@ srt_parser_c::probe(mm_text_io_c *io) {
 
     s = io->getline();
     boost::regex timecode_re(SRT_RE_TIMECODE_LINE, boost::regex::perl);
-    boost::match_results<string::const_iterator> matches;
+    boost::match_results<std::string::const_iterator> matches;
     if (!boost::regex_search(s, timecode_re))
       return false;
 
@@ -85,12 +89,12 @@ srt_parser_c::parse() {
   bool timecode_warning_printed = false;
   parser_state_e state          = STATE_INITIAL;
   int line_number               = 0;
-  string subtitles;
+  std::string subtitles;
 
   m_io->setFilePointer(0, seek_beginning);
 
   while (1) {
-    string s;
+    std::string s;
     if (!m_io->getline2(s))
       break;
 
@@ -117,7 +121,7 @@ srt_parser_c::parse() {
       state = STATE_TIME;
 
     } else if (STATE_TIME == state) {
-      boost::match_results<string::const_iterator> matches;
+      boost::match_results<std::string::const_iterator> matches;
       if (!boost::regex_search(s, matches, timecode_re)) {
         mxwarn_tid(m_file_name, m_tid, boost::format(Y("Error in line %1%: expected a SRT timecode line but found something else. Aborting this file.\n")) % line_number);
         break;
@@ -125,15 +129,18 @@ srt_parser_c::parse() {
 
       int s_h, s_min, s_sec, e_h, e_min, e_sec;
 
-      parse_int(matches[1].str(), s_h);
-      parse_int(matches[2].str(), s_min);
-      parse_int(matches[3].str(), s_sec);
-      parse_int(matches[5].str(), e_h);
-      parse_int(matches[6].str(), e_min);
-      parse_int(matches[7].str(), e_sec);
+      parse_int(matches[2].str(), s_h);
+      parse_int(matches[3].str(), s_min);
+      parse_int(matches[4].str(), s_sec);
+      parse_int(matches[7].str(), e_h);
+      parse_int(matches[8].str(), e_min);
+      parse_int(matches[9].str(), e_sec);
 
-      string s_rest = matches[4].str();
-      string e_rest = matches[8].str();
+      std::string s_rest = matches[ 5].str();
+      std::string e_rest = matches[10].str();
+
+      int64_t s_neg      = matches[ 1].str() == "-" ? -1 : 1;
+      int64_t e_neg      = matches[ 6].str() == "-" ? -1 : 1;
 
       if (boost::regex_search(s, coordinates_re) && !m_coordinates_warning_shown) {
         mxwarn_tid(m_file_name, m_tid,
@@ -153,8 +160,8 @@ srt_parser_c::parse() {
       start  = (int64_t)s_h * 60 * 60 + s_min * 60 + s_sec;
       end    = (int64_t)e_h * 60 * 60 + e_min * 60 + e_sec;
 
-      start *= 1000000000ll;
-      end   *= 1000000000ll;
+      start *= 1000000000ll * s_neg;
+      end   *= 1000000000ll * e_neg;
 
       while (s_rest.length() < 9)
         s_rest += "0";
@@ -168,6 +175,15 @@ srt_parser_c::parse() {
         e_rest.erase(9);
       end += atol(e_rest.c_str());
 
+      if (0 > start) {
+        mxwarn_tid(m_file_name, m_tid,
+                   boost::format(Y("Line %1%: Negative timestamp encountered. The entry will be adjusted to start from 00:00:00.000.\n")) % line_number);
+        end   -= start;
+        start  = 0;
+        if (0 > end)
+          end *= -1;
+      }
+
       // There are files for which start timecodes overlap. Matroska requires
       // blocks to be sorted by their timecode. mkvmerge does this at the end
       // of this function, but warn the user that the original order is being
@@ -234,15 +250,17 @@ ssa_parser_c::probe(mm_text_io_c *io) {
   return false;
 }
 
-ssa_parser_c::ssa_parser_c(mm_text_io_c *io,
+ssa_parser_c::ssa_parser_c(generic_reader_c *reader,
+                           mm_text_io_c *io,
                            const std::string &file_name,
                            int64_t tid)
-  : m_io(io)
+  : m_reader(reader)
+  , m_io(io)
   , m_file_name(file_name)
   , m_tid(tid)
-  , m_cc_utf8(-1)
+  , m_cc_utf8(charset_converter_c::init("UTF-8"))
   , m_is_ass(false)
-  , m_ignore_attachments(true)
+  , m_attachment_id(0)
 {
 }
 
@@ -321,7 +339,7 @@ ssa_parser_c::parse() {
         // Split the line into fields.
         std::vector<std::string> fields = split(line.c_str(), ",", m_format.size());
         while (fields.size() < m_format.size())
-          fields.push_back(string(""));
+          fields.push_back(std::string(""));
 
         // Parse the start time.
         std::string stime = get_element("Start", fields);
@@ -396,27 +414,27 @@ ssa_parser_c::parse() {
   sort();
 }
 
-string
+std::string
 ssa_parser_c::get_element(const char *index,
-                          vector<string> &fields) {
+                          std::vector<std::string> &fields) {
   int i;
 
   for (i = 0; i < m_format.size(); i++)
     if (m_format[i] == index)
       return fields[i];
 
-  return string("");
+  return std::string("");
 }
 
 int64_t
-ssa_parser_c::parse_time(string &stime) {
+ssa_parser_c::parse_time(std::string &stime) {
   int64_t th, tm, ts, tds;
 
   int pos = stime.find(':');
   if (0 > pos)
     return -1;
 
-  string s = stime.substr(0, pos);
+  std::string s = stime.substr(0, pos);
   if (!parse_int(s, th))
     return -1;
   stime.erase(0, pos + 1);
@@ -445,16 +463,24 @@ ssa_parser_c::parse_time(string &stime) {
   return (tds * 10 + ts * 1000 + tm * 60 * 1000 + th * 60 * 60 * 1000) * 1000000;
 }
 
-string
-ssa_parser_c::recode_text(vector<string> &fields) {
-  return to_utf8(m_cc_utf8, get_element("Text", fields));
+std::string
+ssa_parser_c::recode_text(std::vector<std::string> &fields) {
+  return m_cc_utf8->utf8(get_element("Text", fields));
 }
 
 void
 ssa_parser_c::add_attachment_maybe(std::string &name,
                                    std::string &data_uu,
                                    ssa_section_e section) {
-  if (m_ignore_attachments || name.empty() || data_uu.empty() || ((SSA_SECTION_FONTS != section) && (SSA_SECTION_GRAPHICS != section))) {
+  if (name.empty() || data_uu.empty() || ((SSA_SECTION_FONTS != section) && (SSA_SECTION_GRAPHICS != section))) {
+    name    = "";
+    data_uu = "";
+    return;
+  }
+
+  ++m_attachment_id;
+
+  if (!m_reader->attachment_requested(m_attachment_id)) {
     name    = "";
     data_uu = "";
     return;
@@ -471,27 +497,28 @@ ssa_parser_c::add_attachment_maybe(std::string &name,
   if (0 < pos)
     short_name.erase(0, pos + 1);
 
-  attachment.name         = to_utf8(m_cc_utf8, name);
+  attachment.ui_id        = m_attachment_id;
+  attachment.name         = m_cc_utf8->utf8(name);
   attachment.description  = (boost::format(SSA_SECTION_FONTS == section ? Y("Imported font from %1%") : Y("Imported picture from %1%")) % short_name).str();
   attachment.to_all_files = true;
 
   int allocated           = 1024;
-  buffer_t *buffer        = new buffer_t((unsigned char *)safemalloc(allocated), 0);
+  attachment.data         = memory_c::alloc(allocated);
+  attachment.data->set_size(0);
 
   const unsigned char *p  = (const unsigned char *)data_uu.c_str();
   for (pos = 0; data_uu.length() > (pos + 4); pos += 4)
-    decode_chars(p[pos], p[pos + 1], p[pos + 2], p[pos + 3], *buffer, 3, allocated);
+    decode_chars(p[pos], p[pos + 1], p[pos + 2], p[pos + 3], attachment.data, 3, allocated);
 
   switch (data_uu.length() % 4) {
     case 2:
-      decode_chars(p[pos], p[pos + 1], 0, 0, *buffer, 1, allocated);
+      decode_chars(p[pos], p[pos + 1], 0, 0, attachment.data, 1, allocated);
       break;
     case 3:
-      decode_chars(p[pos], p[pos + 1], p[pos + 2], 0, *buffer, 2, allocated);
+      decode_chars(p[pos], p[pos + 1], p[pos + 2], 0, attachment.data, 2, allocated);
       break;
   }
 
-  attachment.data      = counted_ptr<buffer_t>(buffer);
   attachment.mime_type = guess_mime_type(name, false);
 
   if (attachment.mime_type == "")
@@ -508,7 +535,7 @@ ssa_parser_c::decode_chars(unsigned char c1,
                            unsigned char c2,
                            unsigned char c3,
                            unsigned char c4,
-                           buffer_t &buffer,
+                           memory_cptr &buffer,
                            int bytes_to_add,
                            int &allocated) {
   unsigned char bytes[3];
@@ -518,15 +545,15 @@ ssa_parser_c::decode_chars(unsigned char c1,
   bytes[1]       = (value & 0x00ff00) >>  8;
   bytes[0]       = (value & 0xff0000) >> 16;
 
-  if ((buffer.m_size + bytes_to_add) > allocated) {
-    allocated += 1024;
-    buffer.m_buffer = (unsigned char *)realloc(buffer.m_buffer, allocated);
+  if ((buffer->get_size() + bytes_to_add) > allocated) {
+    int old_size  = buffer->get_size();
+    allocated    += 1024;
+    buffer->resize(allocated);
+    buffer->set_size(old_size);
   }
 
-  int i;
-  for (i = 0; i < bytes_to_add; ++i)
-    buffer.m_buffer[buffer.m_size + i] = bytes[i];
-  buffer.m_size += bytes_to_add;
+  memcpy(buffer->get_buffer() + buffer->get_size(), bytes, bytes_to_add);
+  buffer->set_size(buffer->get_size() + bytes_to_add);
 }
 
 // ------------------------------------------------------------
