@@ -12,20 +12,22 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <algorithm>
 #include <cassert>
 #include <limits.h>
 #include <vector>
 
-#include "cluster_helper.h"
-#include "common.h"
-#include "commonebml.h"
-#include "hacks.h"
-#include "libmatroska_extensions.h"
-#include "output_control.h"
-#include "p_video.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/hacks.h"
+#include "common/math.h"
+#include "common/strings/formatting.h"
+#include "merge/cluster_helper.h"
+#include "merge/libmatroska_extensions.h"
+#include "merge/output_control.h"
+#include "output/p_video.h"
 
 #include <matroska/KaxBlock.h>
 #include <matroska/KaxBlockData.h>
@@ -110,7 +112,7 @@ cluster_helper_c::add_packet(packet_cptr packet) {
         additional_size = 21;
 
         // Add sizes for all frames.
-        vector<packet_cptr>::iterator p_it;
+        std::vector<packet_cptr>::iterator p_it;
         mxforeach(p_it, m_packets) {
           packet_cptr &p   = *p_it;
           additional_size += p->data->get_size() + ((-1 == p->bref) ? 10 : (-1 == p->fref) ? 13 : 16);
@@ -211,11 +213,11 @@ cluster_helper_c::set_output(mm_io_c *out) {
 }
 
 void
-cluster_helper_c::set_duration(render_groups_t *rg) {
+cluster_helper_c::set_duration(render_groups_c *rg) {
   if (rg->durations.empty())
     return;
 
-  kax_block_blob_c *group = rg->groups.back();
+  kax_block_blob_c *group = rg->groups.back().get_object();
   int64_t def_duration    = rg->source->get_track_default_duration();
   int64_t block_duration  = 0;
 
@@ -240,7 +242,7 @@ cluster_helper_c::set_duration(render_groups_t *rg) {
 }
 
 bool
-cluster_helper_c::must_duration_be_set(render_groups_t *rg,
+cluster_helper_c::must_duration_be_set(render_groups_c *rg,
                                        packet_cptr &new_packet) {
   int i;
   int64_t block_duration = 0;
@@ -274,10 +276,10 @@ cluster_helper_c::must_duration_be_set(render_groups_t *rg,
 
 int
 cluster_helper_c::render() {
-  vector<render_groups_t *> render_groups;
-  vector<render_groups_t *>::iterator rg_it;
+  std::vector<render_groups_cptr> render_groups;
+  std::vector<render_groups_cptr>::iterator rg_it;
 
-  bool use_simpleblock              = hack_engaged(ENGAGE_USE_SIMPLE_BLOCK);
+  bool use_simpleblock              = !hack_engaged(ENGAGE_NO_SIMPLE_BLOCKS);
 
   BlockBlobType std_block_blob_type = use_simpleblock ? BLOCK_BLOB_ALWAYS_SIMPLE : BLOCK_BLOB_NO_SIMPLE;
 
@@ -295,42 +297,38 @@ cluster_helper_c::render() {
 
   // Make sure that we don't have negative/wrapped around timecodes in the output file.
   // Can happend when we're splitting; so adjust timecode_offset accordingly.
-  vector<packet_cptr>::iterator pack_it;
+  std::vector<packet_cptr>::iterator pack_it;
   mxforeach(pack_it, m_packets)
     m_timecode_offset = std::min(m_timecode_offset, (*pack_it)->assigned_timecode);
 
   mxforeach(pack_it, m_packets) {
     packet_cptr &pack            = *pack_it;
     generic_packetizer_c *source = pack->source;
-    bool has_codec_state         = NULL != pack->codec_state.get();
+    bool has_codec_state         = pack->codec_state.is_set();
 
     if (source->contains_gap())
       m_cluster->SetSilentTrackUsed();
 
-    render_groups_t *render_group = NULL;
+    render_groups_c *render_group = NULL;
     mxforeach(rg_it, render_groups)
       if ((*rg_it)->source == source) {
-        render_group = *rg_it;
+        render_group = (*rg_it).get_object();
         break;
       }
 
     if (NULL == render_group) {
-      render_group                     = new render_groups_t;
-      render_group->source             = source;
-      render_group->more_data          = false;
-      render_group->duration_mandatory = false;
-
-      render_groups.push_back(render_group);
+      render_groups.push_back(render_groups_cptr(new render_groups_c(source)));
+      render_group = render_groups.back().get_object();
     }
 
     min_cl_timecode                        = std::min(pack->assigned_timecode, min_cl_timecode);
     max_cl_timecode                        = std::max(pack->assigned_timecode, max_cl_timecode);
 
-    DataBuffer *data_buffer                = new DataBuffer((binary *)pack->data->get(), pack->data->get_size());
+    DataBuffer *data_buffer                = new DataBuffer((binary *)pack->data->get_buffer(), pack->data->get_size());
 
     KaxTrackEntry &track_entry             = static_cast<KaxTrackEntry &>(*source->get_track_entry());
 
-    kax_block_blob_c *previous_block_group = !render_group->groups.empty() ? render_group->groups.back() : NULL;
+    kax_block_blob_c *previous_block_group = !render_group->groups.empty() ? render_group->groups.back().get_object() : NULL;
     kax_block_blob_c *new_block_group      = previous_block_group;
 
     if ((-1 != pack->bref) || has_codec_state)
@@ -349,23 +347,24 @@ cluster_helper_c::render() {
       if (has_codec_state)
         this_block_blob_type = BLOCK_BLOB_NO_SIMPLE;
 
-      new_block_group = new kax_block_blob_c(this_block_blob_type);
+      render_group->groups.push_back(kax_block_blob_cptr(new kax_block_blob_c(this_block_blob_type)));
+      new_block_group = render_group->groups.back().get_object();
       m_cluster->AddBlockBlob(new_block_group);
       new_block_group->SetParent(*m_cluster);
-      render_group->groups.push_back(new_block_group);
 
       added_to_cues = false;
     }
 
     // Now put the packet into the cluster.
     render_group->more_data = new_block_group->add_frame_auto(track_entry, pack->assigned_timecode - m_timecode_offset, *data_buffer, lacing_type,
-                                                              pack->bref - m_timecode_offset, pack->fref - m_timecode_offset);
+                                                              (-1 == pack->bref) ? -1 : pack->bref - m_timecode_offset,
+                                                              (-1 == pack->fref) ? -1 : pack->fref - m_timecode_offset);
 
     if (has_codec_state) {
       KaxBlockGroup &bgroup = (KaxBlockGroup &)*new_block_group;
       KaxCodecState *cstate = new KaxCodecState;
       bgroup.PushElement(*cstate);
-      cstate->CopyBuffer(pack->codec_state->get(), pack->codec_state->get_size());
+      cstate->CopyBuffer(pack->codec_state->get_buffer(), pack->codec_state->get_size());
     }
 
     if (-1 == m_first_timecode_in_file)
@@ -394,7 +393,7 @@ cluster_helper_c::render() {
           KaxBlockMore &block_more                                           = AddEmptyChild<KaxBlockMore>(additions);
           *static_cast<EbmlUInteger *>(&GetChild<KaxBlockAddID>(block_more)) = data_add_idx + 1;
 
-          GetChild<KaxBlockAdditional>(block_more).CopyBuffer((binary *)pack->data_adds[data_add_idx]->get(), pack->data_adds[data_add_idx]->get_size());
+          GetChild<KaxBlockAdditional>(block_more).CopyBuffer((binary *)pack->data_adds[data_add_idx]->get_buffer(), pack->data_adds[data_add_idx]->get_size());
         }
       }
     }
@@ -439,7 +438,7 @@ cluster_helper_c::render() {
 
   if (0 < elements_in_cluster) {
     mxforeach(rg_it, render_groups)
-      set_duration(*rg_it);
+      set_duration((*rg_it).get_object());
 
     m_cluster->SetPreviousTimecode(min_cl_timecode - m_timecode_offset - 1, (int64_t)g_timecode_scale);
     m_cluster->set_min_timecode(min_cl_timecode - m_timecode_offset);
@@ -455,9 +454,6 @@ cluster_helper_c::render() {
   } else
     m_previous_cluster_tc = 0;
 
-  mxforeach(rg_it, render_groups)
-    delete *rg_it;
-
   m_min_timecode_in_cluster = -1;
   m_max_timecode_in_cluster = -1;
 
