@@ -13,10 +13,7 @@
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-#include "os.h"
-#ifdef SYS_WINDOWS
-# include "os_windows.h"
-#endif
+#include "common/os.h"
 
 #include <errno.h>
 #include <ctype.h>
@@ -65,48 +62,55 @@
 #include <matroska/KaxTrackVideo.h>
 #include <matroska/KaxVersion.h>
 
-#include "chapters.h"
-#include "cluster_helper.h"
-#include "common.h"
-#include "commonebml.h"
-#include "hacks.h"
-#include "mkvmerge.h"
-#include "mm_io.h"
-#include "output_control.h"
-#include "r_aac.h"
-#include "r_ac3.h"
-#include "r_asf.h"
-#include "r_avc.h"
-#include "r_avi.h"
-#include "r_cdxa.h"
-#include "r_dirac.h"
-#include "r_dts.h"
-#include "r_flac.h"
-#include "r_flv.h"
-#include "r_matroska.h"
-#include "r_mp3.h"
-#include "r_mpeg_es.h"
-#include "r_mpeg_ps.h"
-#include "r_mpeg_ts.h"
-#include "r_ogm.h"
-#include "r_qtmp4.h"
-#include "r_real.h"
-#include "r_srt.h"
-#include "r_ssa.h"
-#include "r_tta.h"
-#include "r_usf.h"
-#include "r_vc1.h"
-#include "r_vobbtn.h"
-#include "r_vobsub.h"
-#include "r_wav.h"
-#include "r_wavpack.h"
-#include "r_corepicture.h"
-#include "tag_common.h"
-#include "translation.h"
-#include "xml_element_mapping.h"
+#include "common/chapters/chapters.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/fs_sys_helpers.h"
+#include "common/hacks.h"
+#include "common/locale.h"
+#include "common/math.h"
+#include "common/mm_io.h"
+#include "common/strings/formatting.h"
+#include "common/tags/tags.h"
+#include "common/translation.h"
+#include "common/unique_numbers.h"
+#include "common/version.h"
+#include "common/xml/element_mapping.h"
+#include "input/r_aac.h"
+#include "input/r_ac3.h"
+#include "input/r_asf.h"
+#include "input/r_avc.h"
+#include "input/r_avi.h"
+#include "input/r_cdxa.h"
+#include "input/r_corepicture.h"
+#include "input/r_dirac.h"
+#include "input/r_dts.h"
+#include "input/r_flac.h"
+#include "input/r_flv.h"
+#include "input/r_matroska.h"
+#include "input/r_mp3.h"
+#include "input/r_mpeg_es.h"
+#include "input/r_mpeg_ps.h"
+#include "input/r_mpeg_ts.h"
+#include "input/r_ogm.h"
+#include "input/r_qtmp4.h"
+#include "input/r_real.h"
+#include "input/r_srt.h"
+#include "input/r_ssa.h"
+#include "input/r_truehd.h"
+#include "input/r_tta.h"
+#include "input/r_usf.h"
+#include "input/r_vc1.h"
+#include "input/r_vobbtn.h"
+#include "input/r_vobsub.h"
+#include "input/r_wav.h"
+#include "input/r_wavpack.h"
+#include "merge/cluster_helper.h"
+#include "merge/debugging.h"
+#include "merge/mkvmerge.h"
+#include "merge/output_control.h"
 
 using namespace libmatroska;
-using namespace std;
 
 namespace libmatroska {
 
@@ -118,11 +122,11 @@ namespace libmatroska {
   };
 }
 
-vector<packetizer_t> g_packetizers;
-vector<filelist_t> g_files;
-vector<attachment_t> g_attachments;
-vector<track_order_t> g_track_order;
-vector<append_spec_t> g_append_mapping;
+std::vector<packetizer_t> g_packetizers;
+std::vector<filelist_t> g_files;
+std::vector<attachment_t> g_attachments;
+std::vector<track_order_t> g_track_order;
+std::vector<append_spec_t> g_append_mapping;
 family_uids_c g_segfamily_uids;
 
 int64_t g_attachment_sizes_first            = 0;
@@ -131,7 +135,7 @@ int64_t g_attachment_sizes_others           = 0;
 KaxInfo *g_kax_info_chap                    = NULL;
 
 // Variables set by the command line parser.
-string g_outfile;
+std::string g_outfile;
 int64_t g_file_sizes                        = 0;
 int g_max_blocks_per_cluster                = 65535;
 int64_t g_max_ns_per_cluster                = 2000000000;
@@ -164,13 +168,13 @@ KaxChapters *g_kax_chapters                 = NULL;
 
 KaxTags *g_tags_from_cue_chapters           = NULL;
 
-string g_chapter_file_name;
-string g_chapter_language;
-string g_chapter_charset;
+std::string g_chapter_file_name;
+std::string g_chapter_language;
+std::string g_chapter_charset;
 
-string g_segmentinfo_file_name;
+std::string g_segmentinfo_file_name;
 
-string g_segment_title;
+std::string g_segment_title;
 bool g_segment_title_set                    = false;
 
 int64_t g_tags_size                         = 0;
@@ -181,11 +185,11 @@ int g_split_max_num_files                   = 65535;
 
 append_mode_e g_append_mode                 = APPEND_MODE_FILE_BASED;
 
-string g_default_language                   = "und";
+std::string g_default_language              = "und";
 
 bitvalue_cptr g_seguid_link_previous;
 bitvalue_cptr g_seguid_link_next;
-vector<bitvalue_cptr> g_forced_seguids;
+std::deque<bitvalue_cptr> g_forced_seguids;
 
 static KaxInfo *s_kax_infos                 = NULL;
 static KaxMyDuration *s_kax_duration        = NULL;
@@ -293,8 +297,6 @@ sighandler(int signum) {
 */
 void
 get_file_type(filelist_t &file) {
-  static const int probe_sizes[] = {16 * 1024, 32 * 1024, 64 * 1024, 128 * 1024, 256 * 1024, 0};
-
   mm_io_c *io  = NULL;
   int64_t size = 0;
 
@@ -345,23 +347,26 @@ get_file_type(filelist_t &file) {
     type = FILE_TYPE_MPEG_ES;
   else {
     // File types which are the same in raw format and in other container formats.
-    // Detection requires five or more consecutive packets. 
+    // Detection requires 20 or more consecutive packets.
+    static const int s_probe_sizes[]                          = { 32 * 1024, 64 * 1024, 128 * 1024, 256 * 1024, 512 * 1024, 1024 * 1024, 0 };
+    static const int s_probe_num_required_consecutive_packets = 20;
+
     int i;
-    for (i = 0; (probe_sizes[i] != 0) && (type == FILE_TYPE_IS_UNKNOWN); i++)
-      if (mp3_reader_c::probe_file(io, size, probe_sizes[i], 5))
+    for (i = 0; (0 != s_probe_sizes[i]) && (FILE_TYPE_IS_UNKNOWN == type); ++i)
+      if (mp3_reader_c::probe_file(io, size, s_probe_sizes[i], s_probe_num_required_consecutive_packets))
         type = FILE_TYPE_MP3;
-      else if (ac3_reader_c::probe_file(io, size, probe_sizes[i], 5))
+      else if (ac3_reader_c::probe_file(io, size, s_probe_sizes[i], s_probe_num_required_consecutive_packets))
         type = FILE_TYPE_AC3;
+      else if (aac_reader_c::probe_file(io, size, s_probe_sizes[i], s_probe_num_required_consecutive_packets))
+        type = FILE_TYPE_AAC;
   }
   // More file types with detection issues.
   if (type != FILE_TYPE_IS_UNKNOWN)
     ;
-  else if (mp3_reader_c::probe_file(io, size, 2 * 1024 * 1024, 10))
-    type = FILE_TYPE_MP3;
+  else if (truehd_reader_c::probe_file(io, size))
+    type = FILE_TYPE_TRUEHD;
   else if (dts_reader_c::probe_file(io, size))
     type = FILE_TYPE_DTS;
-  else if (aac_reader_c::probe_file(io, size))
-    type = FILE_TYPE_AAC;
   else if (vobbtn_reader_c::probe_file(io, size))
     type = FILE_TYPE_VOBBTN;
   else if (avc_es_reader_c::probe_file(io, size))
@@ -408,7 +413,7 @@ display_progress() {
   static int s_previous_percentage      = -1;
 
   if (NULL == s_display_reader) {
-    vector<filelist_t>::const_iterator i;
+    std::vector<filelist_t>::const_iterator i;
 
     const filelist_t *winner = &g_files[0];
     for (i = g_files.begin() + 1; i != g_files.end(); ++i)
@@ -456,19 +461,17 @@ add_tags(KaxTag *tags) {
 */
 int64_t
 add_attachment(attachment_t attachment) {
-  vector<attachment_t>::iterator i;
-
   // If the attachment is coming from an existing file then we should
   // check if we already have another attachment stored. This can happen
   // if we're concatenating files.
   if (0 != attachment.id) {
-    mxforeach(i, g_attachments)
-      if ((   (i->id == attachment.id)
+    foreach(attachment_t &ex_attachment, g_attachments)
+      if ((   (ex_attachment.id == attachment.id)
            && !hack_engaged(ENGAGE_NO_VARIABLE_DATA))
           ||
-          (   (i->name             == attachment.name)
-           && (i->description      == attachment.description)
-           && (i->data->get_size() == attachment.data->get_size())))
+          (   (ex_attachment.name             == attachment.name)
+           && (ex_attachment.description      == attachment.description)
+           && (ex_attachment.data->get_size() == attachment.data->get_size())))
         return attachment.id;
 
     add_unique_uint32(attachment.id, UNIQUE_ATTACHMENT_IDS);
@@ -493,13 +496,14 @@ add_packetizer_globally(generic_packetizer_c *packetizer) {
   pack.old_status      = pack.status;
   pack.file            = -1;
 
-  vector<filelist_t>::iterator file;
-  mxforeach(file, g_files)
-    if (file->reader == packetizer->reader) {
-      pack.file      = distance(g_files.begin(), file);
+  int idx = 0;
+  foreach(filelist_t &file, g_files)
+    if (file.reader == packetizer->reader) {
+      pack.file      = idx;
       pack.orig_file = pack.file;
       break;
-    }
+    } else
+      ++idx;
 
   if (-1 == pack.file)
     mxerror(boost::format(Y("filelist_t not found for generic_packetizer_c. %1%\n")) % BUGMSG);
@@ -513,14 +517,13 @@ set_timecode_scale() {
   bool audio_present          = false;
   int64_t highest_sample_rate = 0;
 
-  vector<packetizer_t>::const_iterator ptzr;
-  mxforeach(ptzr, g_packetizers)
-    if ((*ptzr).packetizer->get_track_type() == track_video)
+  foreach(packetizer_t &ptzr, g_packetizers)
+    if (ptzr.packetizer->get_track_type() == track_video)
       video_present = true;
 
-    else if ((*ptzr).packetizer->get_track_type() == track_audio) {
+    else if (ptzr.packetizer->get_track_type() == track_audio) {
       audio_present       = true;
-      highest_sample_rate = std::max((int64_t)ptzr->packetizer->get_audio_sampling_freq(), highest_sample_rate);
+      highest_sample_rate = std::max(static_cast<int64_t>(ptzr.packetizer->get_audio_sampling_freq()), highest_sample_rate);
     }
 
   if (   (TIMECODE_SCALE_MODE_FIXED != g_timecode_scale_mode)
@@ -550,7 +553,7 @@ render_headers(mm_io_c *out) {
     bool first_file                                       = (1 == g_file_num);
 
     GetChildAs<EDocType, EbmlString>(head)                = "matroska";
-    if (!hack_engaged(ENGAGE_USE_SIMPLE_BLOCK)) {
+    if (hack_engaged(ENGAGE_NO_SIMPLE_BLOCKS)) {
       GetChildAs<EDocTypeVersion, EbmlUInteger>(head)     = 1;
       GetChildAs<EDocTypeReadVersion, EbmlUInteger>(head) = 1;
 
@@ -572,14 +575,14 @@ render_headers(mm_io_c *out) {
     s_kax_infos->PushElement(*s_kax_duration);
 
     if (!hack_engaged(ENGAGE_NO_VARIABLE_DATA)) {
-      string version                                            = string("libebml v") + EbmlCodeVersion + string(" + libmatroska v") + KaxCodeVersion;
-      GetChildAs<KaxMuxingApp, EbmlUnicodeString>(s_kax_infos)  = cstr_to_UTFstring(version.c_str());
-      GetChildAs<KaxWritingApp, EbmlUnicodeString>(s_kax_infos) = cstr_to_UTFstring(VERSIONINFO " built on " __DATE__ " " __TIME__);
+      std::string muxing_app                                    = std::string("libebml v") + EbmlCodeVersion + std::string(" + libmatroska v") + KaxCodeVersion;
+      GetChildAs<KaxMuxingApp, EbmlUnicodeString>(s_kax_infos)  = cstrutf8_to_UTFstring(muxing_app);
+      GetChildAs<KaxWritingApp, EbmlUnicodeString>(s_kax_infos) = cstrutf8_to_UTFstring(get_version_info("mkvmerge", true));
       GetChild<KaxDateUTC>(*s_kax_infos).SetEpochDate(time(NULL));
 
     } else {
-      GetChildAs<KaxMuxingApp, EbmlUnicodeString>(s_kax_infos)  = cstr_to_UTFstring(ENGAGE_NO_VARIABLE_DATA);
-      GetChildAs<KaxWritingApp, EbmlUnicodeString>(s_kax_infos) = cstr_to_UTFstring(ENGAGE_NO_VARIABLE_DATA);
+      GetChildAs<KaxMuxingApp, EbmlUnicodeString>(s_kax_infos)  = cstrutf8_to_UTFstring("no_variable_data");
+      GetChildAs<KaxWritingApp, EbmlUnicodeString>(s_kax_infos) = cstrutf8_to_UTFstring("no_variable_data");
       GetChild<KaxDateUTC>(*s_kax_infos).SetEpochDate(0);
     }
 
@@ -592,8 +595,8 @@ render_headers(mm_io_c *out) {
         if (g_forced_seguids.empty())
           s_seguid_current.generate_random();
         else {
-          s_seguid_current = *g_forced_seguids.back();
-          g_forced_seguids.pop_back();
+          s_seguid_current = *g_forced_seguids.front();
+          g_forced_seguids.pop_front();
         }
         s_seguid_next.generate_random();
 
@@ -602,8 +605,8 @@ render_headers(mm_io_c *out) {
         if (g_forced_seguids.empty())
           s_seguid_current = s_seguid_next;
         else {
-          s_seguid_current = *g_forced_seguids.back();
-          g_forced_seguids.pop_back();
+          s_seguid_current = *g_forced_seguids.front();
+          g_forced_seguids.pop_front();
         }
         s_seguid_next.generate_random();
       }
@@ -634,14 +637,14 @@ render_headers(mm_io_c *out) {
       }
     }
 
-    if (first_file && (NULL != g_seguid_link_previous.get()))
+    if (first_file && g_seguid_link_previous.is_set())
       GetChild<KaxPrevUID>(*s_kax_infos).CopyBuffer(g_seguid_link_previous->data(), 128 / 8);
 
     // The next segment UID is also set in finish_file(). This is not
     // redundant! It is set here as well in order to reserve enough space
     // for the KaxInfo structure in the file. If it is removed later then
     // an EbmlVoid element will be used for the freed space.
-    if (NULL != g_seguid_link_next.get())
+    if (g_seguid_link_next.is_set())
       GetChild<KaxNextUID>(*s_kax_infos).CopyBuffer(g_seguid_link_next->data(), 128 / 8);
 
     if (!g_no_linking && g_cluster_helper->splitting()) {
@@ -739,32 +742,31 @@ render_attachments(IOCallback *out) {
   s_kax_as           = new KaxAttachments();
   KaxAttached *kax_a = NULL;
 
-  vector<attachment_t>::iterator attch;
-  mxforeach(attch, g_attachments) {
-    if ((1 == g_file_num) || attch->to_all_files) {
+  foreach(attachment_t &attch, g_attachments) {
+    if ((1 == g_file_num) || attch.to_all_files) {
       kax_a = NULL == kax_a ? &GetChild<KaxAttached>(*s_kax_as) : &GetNextChild<KaxAttached>(*s_kax_as, *kax_a);
 
-      if (attch->description != "")
-        GetChildAs<KaxFileDescription, EbmlUnicodeString>(kax_a) = cstrutf8_to_UTFstring(attch->description);
+      if (attch.description != "")
+        GetChildAs<KaxFileDescription, EbmlUnicodeString>(kax_a) = cstrutf8_to_UTFstring(attch.description);
 
-      if (attch->mime_type != "")
-        GetChildAs<KaxMimeType, EbmlString>(kax_a)               = attch->mime_type;
+      if (attch.mime_type != "")
+        GetChildAs<KaxMimeType, EbmlString>(kax_a)               = attch.mime_type;
 
-      string name;
-      if (attch->stored_name == "") {
-        int path_sep_idx = attch->name.rfind(PATHSEP);
+      std::string name;
+      if (attch.stored_name == "") {
+        int path_sep_idx = attch.name.rfind(PATHSEP);
         if (-1 != path_sep_idx)
-          name = attch->name.substr(path_sep_idx + 1);
+          name = attch.name.substr(path_sep_idx + 1);
         else
-          name = attch->name;
+          name = attch.name;
 
       } else
-        name = attch->stored_name;
+        name = attch.stored_name;
 
-      GetChildAs<KaxFileName, EbmlUnicodeString>(kax_a) = cstr_to_UTFstring(name);
-      GetChildAs<KaxFileUID, EbmlUInteger>(kax_a)       = attch->id;
+      GetChildAs<KaxFileName, EbmlUnicodeString>(kax_a) = cstrutf8_to_UTFstring(name);
+      GetChildAs<KaxFileUID, EbmlUInteger>(kax_a)       = attch.id;
 
-      GetChild<KaxFileData>(*kax_a).CopyBuffer(attch->data->get(), attch->data->get_size());
+      GetChild<KaxFileData>(*kax_a).CopyBuffer(attch.data->get_buffer(), attch.data->get_size());
     }
   }
 
@@ -785,9 +787,9 @@ render_attachments(IOCallback *out) {
 */
 static void
 check_append_mapping() {
-  vector<append_spec_t>::iterator amap, cmp_amap, trav_amap;
-  vector<int64_t>::iterator id;
-  vector<filelist_t>::iterator src_file, dst_file;
+  std::vector<append_spec_t>::iterator amap, cmp_amap, trav_amap;
+  std::vector<int64_t>::iterator id;
+  std::vector<filelist_t>::iterator src_file, dst_file;
   int count, file_id;
 
   mxforeach(amap, g_append_mapping) {
@@ -829,7 +831,7 @@ check_append_mapping() {
       mxerror(boost::format(Y("Only partial append mappings were given for the file no. %1% ('%2%'). Either don't specify any mapping (in which case the "
                               "default mapping will be used) or specify a mapping for all tracks that are to be copied.\n")) % file_id % src_file->name);
     else if (0 == count) {
-      string missing_mappings;
+      std::string missing_mappings;
 
       // Default mapping.
       mxforeach(id, src_file->reader->used_track_ids) {
@@ -894,9 +896,9 @@ check_append_mapping() {
   // Finally see if the packetizers can be connected and connect them if they
   // can.
   mxforeach(amap, g_append_mapping) {
-    vector<generic_packetizer_c *>::const_iterator gptzr;
+    std::vector<generic_packetizer_c *>::const_iterator gptzr;
     generic_packetizer_c *src_ptzr, *dst_ptzr;
-    string error_message;
+    std::string error_message;
     int result;
 
     src_file = g_files.begin() + amap->src_file_id;
@@ -929,9 +931,9 @@ check_append_mapping() {
              % error_message);
 
     else if (CAN_CONNECT_YES != result) {
-      string reason(  result == CAN_CONNECT_NO_FORMAT     ? Y("The formats do not match.")
-                    : result == CAN_CONNECT_NO_PARAMETERS ? Y("The track parameters do not match.")
-                    :                                       Y("The reason is unknown."));
+      std::string reason(  result == CAN_CONNECT_NO_FORMAT     ? Y("The formats do not match.")
+                         : result == CAN_CONNECT_NO_PARAMETERS ? Y("The track parameters do not match.")
+                         :                                       Y("The reason is unknown."));
       mxerror(boost::format(Y("The track number %1% from the file '%2%' cannot be appended to the track number %3% from the file '%4%'. %5%\n"))
               % amap->src_track_id % g_files[amap->src_file_id].name
               % amap->dst_track_id % g_files[amap->dst_file_id].name
@@ -987,14 +989,12 @@ check_append_mapping() {
 */
 void
 calc_max_chapter_size() {
-  vector<filelist_t>::iterator file;
-  KaxChapters *chapters;
-
   // Step 1: Add all chapters from g_files that are not being appended.
-  mxforeach(file, g_files) {
-    if (file->appending)
+  foreach(filelist_t &file, g_files) {
+    if (file.appending)
       continue;
-    chapters = file->reader->chapters;
+
+    KaxChapters *chapters = file.reader->chapters;
     if (NULL == chapters)
       continue;
 
@@ -1003,7 +1003,7 @@ calc_max_chapter_size() {
 
     move_chapters_by_edition(*g_kax_chapters, *chapters);
     delete chapters;
-    file->reader->chapters = NULL;
+    file.reader->chapters = NULL;
   }
 
   // Step 2: Fix the mandatory elements and count the size of all chapters.
@@ -1014,8 +1014,8 @@ calc_max_chapter_size() {
     s_max_chapter_size += g_kax_chapters->ElementSize();
   }
 
-  mxforeach(file, g_files) {
-    chapters = file->reader->chapters;
+  foreach(filelist_t &file, g_files) {
+    KaxChapters *chapters = file.reader->chapters;
     if (NULL == chapters)
       continue;
 
@@ -1034,118 +1034,117 @@ calc_max_chapter_size() {
 */
 void
 create_readers() {
-  vector<filelist_t>::iterator file;
-
-  mxforeach(file, g_files) {
+  foreach(filelist_t &file, g_files) {
     try {
-      switch (file->type) {
+      switch (file.type) {
         case FILE_TYPE_AAC:
-          file->reader = new aac_reader_c(*file->ti);
+          file.reader = new aac_reader_c(*file.ti);
           break;
         case FILE_TYPE_AC3:
-          file->reader = new ac3_reader_c(*file->ti);
+          file.reader = new ac3_reader_c(*file.ti);
           break;
         case FILE_TYPE_AVC_ES:
-          file->reader = new avc_es_reader_c(*file->ti);
+          file.reader = new avc_es_reader_c(*file.ti);
           break;
         case FILE_TYPE_AVI:
-          file->reader = new avi_reader_c(*file->ti);
+          file.reader = new avi_reader_c(*file.ti);
           break;
         case FILE_TYPE_DIRAC:
-          file->reader = new dirac_es_reader_c(*file->ti);
+          file.reader = new dirac_es_reader_c(*file.ti);
           break;
         case FILE_TYPE_DTS:
-          file->reader = new dts_reader_c(*file->ti);
+          file.reader = new dts_reader_c(*file.ti);
           break;
 #if defined(HAVE_FLAC_FORMAT_H)
         case FILE_TYPE_FLAC:
-          file->reader = new flac_reader_c(*file->ti);
+          file.reader = new flac_reader_c(*file.ti);
           break;
 #endif
         case FILE_TYPE_MATROSKA:
-          file->reader = new kax_reader_c(*file->ti);
+          file.reader = new kax_reader_c(*file.ti);
           break;
         case FILE_TYPE_MP3:
-          file->reader = new mp3_reader_c(*file->ti);
+          file.reader = new mp3_reader_c(*file.ti);
           break;
         case FILE_TYPE_MPEG_ES:
-          file->reader = new mpeg_es_reader_c(*file->ti);
+          file.reader = new mpeg_es_reader_c(*file.ti);
           break;
         case FILE_TYPE_MPEG_PS:
-          file->reader = new mpeg_ps_reader_c(*file->ti);
+          file.reader = new mpeg_ps_reader_c(*file.ti);
           break;
         case FILE_TYPE_OGM:
-          file->reader = new ogm_reader_c(*file->ti);
+          file.reader = new ogm_reader_c(*file.ti);
           break;
         case FILE_TYPE_QTMP4:
-          file->reader = new qtmp4_reader_c(*file->ti);
+          file.reader = new qtmp4_reader_c(*file.ti);
           break;
         case FILE_TYPE_REAL:
-          file->reader = new real_reader_c(*file->ti);
+          file.reader = new real_reader_c(*file.ti);
           break;
         case FILE_TYPE_SSA:
-          file->reader = new ssa_reader_c(*file->ti);
+          file.reader = new ssa_reader_c(*file.ti);
           break;
         case FILE_TYPE_SRT:
-          file->reader = new srt_reader_c(*file->ti);
+          file.reader = new srt_reader_c(*file.ti);
+          break;
+        case FILE_TYPE_TRUEHD:
+          file.reader = new truehd_reader_c(*file.ti);
           break;
         case FILE_TYPE_TTA:
-          file->reader = new tta_reader_c(*file->ti);
+          file.reader = new tta_reader_c(*file.ti);
           break;
         case FILE_TYPE_USF:
-          file->reader = new usf_reader_c(*file->ti);
+          file.reader = new usf_reader_c(*file.ti);
           break;
         case FILE_TYPE_COREPICTURE:
-          file->reader = new corepicture_reader_c(*file->ti);
+          file.reader = new corepicture_reader_c(*file.ti);
           break;
         case FILE_TYPE_VC1:
-          file->reader = new vc1_es_reader_c(*file->ti);
+          file.reader = new vc1_es_reader_c(*file.ti);
           break;
         case FILE_TYPE_VOBBTN:
-          file->reader = new vobbtn_reader_c(*file->ti);
+          file.reader = new vobbtn_reader_c(*file.ti);
           break;
         case FILE_TYPE_VOBSUB:
-          file->reader = new vobsub_reader_c(*file->ti);
+          file.reader = new vobsub_reader_c(*file.ti);
           break;
         case FILE_TYPE_WAV:
-          file->reader = new wav_reader_c(*file->ti);
+          file.reader = new wav_reader_c(*file.ti);
           break;
         case FILE_TYPE_WAVPACK4:
-          file->reader = new wavpack_reader_c(*file->ti);
+          file.reader = new wavpack_reader_c(*file.ti);
           break;
         default:
           mxerror(boost::format(Y("EVIL internal bug! (unknown file type). %1%\n")) % BUGMSG);
           break;
       }
     } catch (error_c &error) {
-      mxerror(boost::format(Y("The demultiplexer for the file '%1%' failed to initialize:\n%2%\n")) % file->ti->fname % error.get_error());
+      mxerror(boost::format(Y("The demultiplexer for the file '%1%' failed to initialize:\n%2%\n")) % file.ti->fname % error.get_error());
     }
   }
 
   if (!g_identifying) {
-    vector<attachment_t>::const_iterator att;
-
     // Create the packetizers.
-    mxforeach(file, g_files) {
-      file->reader->appending = file->appending;
-      file->reader->create_packetizers();
+    foreach(filelist_t &file, g_files) {
+      file.reader->appending = file.appending;
+      file.reader->create_packetizers();
     }
     // Check if all track IDs given on the command line are actually
     // present.
-    mxforeach(file, g_files) {
-      file->reader->check_track_ids_and_packetizers();
-      file->num_unfinished_packetizers     = file->reader->reader_packetizers.size();
-      file->old_num_unfinished_packetizers = file->num_unfinished_packetizers;
+    foreach(filelist_t &file, g_files) {
+      file.reader->check_track_ids_and_packetizers();
+      file.num_unfinished_packetizers     = file.reader->reader_packetizers.size();
+      file.old_num_unfinished_packetizers = file.num_unfinished_packetizers;
     }
 
     // Check if the append mappings are ok.
     check_append_mapping();
 
     // Calculate the size of all attachments for split control.
-    mxforeach(att, g_attachments) {
-      g_attachment_sizes_first += att->data->get_size();
-      if (att->to_all_files)
-        g_attachment_sizes_others += att->data->get_size();
+    foreach(attachment_t &att, g_attachments) {
+      g_attachment_sizes_first += att.data->get_size();
+      if (att.to_all_files)
+        g_attachment_sizes_others += att.data->get_size();
     }
 
     calc_max_chapter_size();
@@ -1160,9 +1159,9 @@ create_readers() {
    \arg . ("-%03d" will be inserted before the .)
    \arg "-%03d" will be appended
 */
-string
+std::string
 create_output_name() {
-  string s = g_outfile;
+  std::string s = g_outfile;
   int p2   = 0;
   // First possibility: %d
   int p    = s.find("%d");
@@ -1182,7 +1181,7 @@ create_output_name() {
         if (!isdigit(s[i]))
           break;
 
-      string format(&s.c_str()[p]);
+      std::string format(&s.c_str()[p]);
       format.erase(p2 - p + 1);
       s.replace(p, format.size(), (boost::format(format) % g_file_num).str());
 
@@ -1190,7 +1189,7 @@ create_output_name() {
     }
   }
 
-  string buffer = (boost::format("-%|1$03d|") % g_file_num).str();
+  std::string buffer = (boost::format("-%|1$03d|") % g_file_num).str();
 
   // See if we can find a '.'.
   p = s.rfind(".");
@@ -1252,7 +1251,7 @@ add_tags_from_cue_chapters() {
 */
 void
 create_next_output_file() {
-  string this_outfile = g_cluster_helper->splitting() ? create_output_name() : g_outfile;
+  std::string this_outfile = g_cluster_helper->splitting() ? create_output_name() : g_outfile;
 
   g_kax_segment       = new KaxSegment();
   g_kax_cues          = new KaxCues();
@@ -1342,7 +1341,7 @@ finish_file(bool last_file) {
   int64_t info_size = s_kax_infos->ElementSize();
   int changed       = 0;
 
-  if (last_file && (NULL != g_seguid_link_next.get())) {
+  if (last_file && g_seguid_link_next.is_set()) {
     GetChild<KaxNextUID>(*s_kax_infos).CopyBuffer(g_seguid_link_next->data(), 128 / 8);
     changed = 1;
 
@@ -1516,7 +1515,7 @@ append_track(packetizer_t &ptzr,
 
   // Find the generic_packetizer_c that we will be appending to the one
   // stored in ptzr.
-  vector<generic_packetizer_c *>::const_iterator gptzr;
+  std::vector<generic_packetizer_c *>::const_iterator gptzr;
   mxforeach(gptzr, src_file.reader->reader_packetizers)
     if (amap.src_track_id == (*gptzr)->ti.id)
       break;
@@ -1544,12 +1543,12 @@ append_track(packetizer_t &ptzr,
       && (            -1 == src_file.deferred_max_timecode_seen)
       && (          NULL != g_video_packetizer)) {
 
-    vector<filelist_t>::iterator file;
+    std::vector<filelist_t>::iterator file;
     mxforeach(file, g_files) {
       if (file->done)
         continue;
 
-      vector<generic_packetizer_c *>::const_iterator vptzr;
+      std::vector<generic_packetizer_c *>::const_iterator vptzr;
       mxforeach(vptzr, file->reader->reader_packetizers)
         if ((*vptzr)->get_track_type() == track_video)
           break;
@@ -1615,7 +1614,7 @@ append_track(packetizer_t &ptzr,
       ptzr.status = ptzr.packetizer->read();
 
     if (src_file.reader->ptzr_first_packet != NULL) {
-      vector<append_spec_t>::const_iterator cmp_amap;
+      std::vector<append_spec_t>::const_iterator cmp_amap;
       mxforeach(cmp_amap, g_append_mapping)
         if (   (cmp_amap->src_file_id  == amap.src_file_id)
             && (cmp_amap->src_track_id == src_file.reader->ptzr_first_packet->ti.id)
@@ -1671,26 +1670,27 @@ bool
 append_tracks_maybe() {
   bool appended_a_track = false;
 
-  vector<packetizer_t>::iterator ptzr;
-  mxforeach(ptzr, g_packetizers) {
-    if (ptzr->deferred)
+  foreach(packetizer_t &ptzr, g_packetizers) {
+    if (ptzr.deferred)
       continue;
 
-    if (!g_files[ptzr->orig_file].appended_to)
+    if (!g_files[ptzr.orig_file].appended_to)
       continue;
 
-    if (FILE_STATUS_DONE_AND_DRY != ptzr->status)
+    if (FILE_STATUS_DONE_AND_DRY != ptzr.status)
       continue;
 
-    vector<append_spec_t>::const_iterator amap;
-    mxforeach(amap, g_append_mapping)
-      if ((amap->dst_file_id == ptzr->file) && (amap->dst_track_id == ptzr->packetizer->ti.id))
+    append_spec_t *amap = NULL;
+    foreach(append_spec_t &amap_idx, g_append_mapping)
+      if ((amap_idx.dst_file_id == ptzr.file) && (amap_idx.dst_track_id == ptzr.packetizer->ti.id)) {
+        amap = &amap_idx;
         break;
+      }
 
-    if (g_append_mapping.end() == amap)
+    if (NULL == amap)
       continue;
 
-    append_track(*ptzr, *amap);
+    append_track(ptzr, *amap);
     appended_a_track = true;
   }
 
@@ -1713,12 +1713,11 @@ append_tracks_maybe() {
 */
 static void
 establish_deferred_connections(filelist_t &file) {
-  vector<deferred_connection_t> def_cons = file.deferred_connections;
+  std::vector<deferred_connection_t> def_cons = file.deferred_connections;
   file.deferred_connections.clear();
 
-  vector<deferred_connection_t>::iterator def_con;
-  mxforeach(def_con, def_cons)
-    append_track(*def_con->ptzr, def_con->amap, &file);
+  foreach(deferred_connection_t &def_con, def_cons)
+    append_track(*def_con.ptzr, def_con.amap, &file);
 
   // \todo Select a new file that the subs will defer to.
 }
@@ -1733,39 +1732,37 @@ void
 main_loop() {
   // Let's go!
   while (1) {
-    packet_cptr pack;
-    bool appended_a_track;
+    debug_run_main_loop_hooks();
 
     // Step 1: Make sure a packet is available for each output
     // as long we haven't already processed the last one.
-    vector<packetizer_t>::iterator ptzr;
-    mxforeach(ptzr, g_packetizers) {
-      if (FILE_STATUS_HOLDING == ptzr->status)
-        ptzr->status = FILE_STATUS_MOREDATA;
+    foreach(packetizer_t &ptzr, g_packetizers) {
+      if (FILE_STATUS_HOLDING == ptzr.status)
+        ptzr.status = FILE_STATUS_MOREDATA;
 
-      ptzr->old_status = ptzr->status;
+      ptzr.old_status = ptzr.status;
 
-      while (   (NULL == ptzr->pack.get())
-             && (FILE_STATUS_MOREDATA == ptzr->status)
-             && !ptzr->packetizer->packet_available())
-        ptzr->status = ptzr->packetizer->read();
+      while (   !ptzr.pack.is_set()
+             && (FILE_STATUS_MOREDATA == ptzr.status)
+             && !ptzr.packetizer->packet_available())
+        ptzr.status = ptzr.packetizer->read();
 
-      if (   (FILE_STATUS_MOREDATA != ptzr->status)
-          && (FILE_STATUS_MOREDATA == ptzr->old_status))
-        ptzr->packetizer->force_duration_on_last_packet();
+      if (   (FILE_STATUS_MOREDATA != ptzr.status)
+          && (FILE_STATUS_MOREDATA == ptzr.old_status))
+        ptzr.packetizer->force_duration_on_last_packet();
 
-      if (NULL == ptzr->pack.get())
-        ptzr->pack = ptzr->packetizer->get_packet();
+      if (!ptzr.pack.is_set())
+        ptzr.pack = ptzr.packetizer->get_packet();
 
-      if ((NULL == ptzr->pack.get()) && (FILE_STATUS_DONE == ptzr->status))
-        ptzr->status = FILE_STATUS_DONE_AND_DRY;
+      if (!ptzr.pack.is_set() && (FILE_STATUS_DONE == ptzr.status))
+        ptzr.status = FILE_STATUS_DONE_AND_DRY;
 
       // Has this packetizer changed its status from "data available" to
       // "file done" during this loop? If so then decrease the number of
       // unfinished packetizers in the corresponding file structure.
-      if (   (FILE_STATUS_DONE_AND_DRY == ptzr->status)
-          && (ptzr->old_status != ptzr->status)) {
-        filelist_t &file = g_files[ptzr->file];
+      if (   (FILE_STATUS_DONE_AND_DRY == ptzr.status)
+          && (ptzr.old_status != ptzr.status)) {
+        filelist_t &file = g_files[ptzr.file];
         file.num_unfinished_packetizers--;
 
         // If all packetizers for a file have finished then establish the
@@ -1780,22 +1777,22 @@ main_loop() {
 
     // Step 2: Pick the packet with the lowest timecode and
     // stuff it into the Matroska file.
-    vector<packetizer_t>::iterator winner = g_packetizers.end();
-    mxforeach(ptzr, g_packetizers) {
-      if (NULL != ptzr->pack.get()) {
-        if ((g_packetizers.end() == winner) || (NULL == winner->pack.get()))
-          winner = ptzr;
+    packetizer_t * winner = NULL;
+    foreach(packetizer_t &ptzr, g_packetizers) {
+      if (ptzr.pack.is_set()) {
+        if ((NULL == winner) || !winner->pack.is_set())
+          winner = &ptzr;
 
-        else if ((NULL != ptzr->pack.get()) && (ptzr->pack->assigned_timecode < winner->pack->assigned_timecode))
-          winner = ptzr;
+        else if (ptzr.pack.is_set() && (ptzr.pack->assigned_timecode < winner->pack->assigned_timecode))
+          winner = &ptzr;
       }
     }
 
     // Append the next track if appending is wanted.
-    appended_a_track = append_tracks_maybe();
+    bool appended_a_track = append_tracks_maybe();
 
-    if ((g_packetizers.end() != winner) && (NULL != winner->pack.get())) {
-      pack = winner->pack;
+    if ((NULL != winner) && winner->pack.is_set()) {
+      packet_cptr pack = winner->pack;
 
       // Step 3: Add the winning packet to a cluster. Full clusters will be
       // rendered automatically.
@@ -1827,31 +1824,23 @@ main_loop() {
 */
 void
 setup() {
-  init_stdio();
-  init_locales();
+  mtx_common_init();
 
 #if defined(SYS_UNIX) || defined(COMP_CYGWIN) || defined(SYS_APPLE)
   signal(SIGUSR1, sighandler);
   signal(SIGINT, sighandler);
 #endif
 
-  mm_file_io_c::setup();
-  cc_local_utf8 = utf8_init("");
-  init_cc_stdio();
-
   g_cluster_helper = new cluster_helper_c();
-
-  xml_element_map_init();
 }
 
 /** \brief Deletes the file readers and other associated objects
 */
 static void
 destroy_readers() {
-  vector<filelist_t>::const_iterator file;
-  mxforeach(file, g_files) {
-    delete file->reader;
-    delete file->ti;
+  foreach(filelist_t &file, g_files) {
+    delete file.reader;
+    delete file.ti;
   }
 
   g_files.clear();
@@ -1886,6 +1875,4 @@ cleanup() {
   g_kax_info_chap = NULL;
 
   g_forced_seguids.clear();
-
-  utf8_done();
 }
