@@ -11,39 +11,41 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include "wx/wxprec.h"
+#include <wx/wxprec.h>
 
-#include "wx/wx.h"
-#include "wx/clipbrd.h"
-#include "wx/confbase.h"
-#include "wx/file.h"
-#include "wx/fileconf.h"
-#include "wx/listctrl.h"
-#include "wx/notebook.h"
-#include "wx/statusbr.h"
-#include "wx/statline.h"
+#include <wx/wx.h>
+#include <wx/clipbrd.h>
+#include <wx/confbase.h>
+#include <wx/file.h>
+#include <wx/fileconf.h>
+#include <wx/listctrl.h>
+#include <wx/notebook.h>
+#include <wx/statusbr.h>
+#include <wx/statline.h>
 
 #if defined(SYS_WINDOWS)
-#include <windows.h>
+# include <windows.h>
+# include "common/fs_sys_helpers.h"
 #endif
 
-#include "common.h"
-#include "jobs.h"
-#include "mmg.h"
-#include "mmg_dialog.h"
+#include "common/common.h"
+#include "mmg/jobs.h"
+#include "mmg/mmg.h"
+#include "mmg/mmg_dialog.h"
+#include "mmg/taskbar_progress.h"
 
 #define JOB_LOG_DIALOG_WIDTH 600
 #define JOB_RUN_DIALOG_WIDTH 500
 
 job_run_dialog::job_run_dialog(wxWindow *parent,
-                               vector<int> &n_jobs_to_start)
+                               std::vector<int> &n_jobs_to_start)
   : wxDialog(NULL, -1, Z("mkvmerge is running"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMINIMIZE_BOX | wxMAXIMIZE_BOX)
   , t_update(new wxTimer(this, 1))
   , out(NULL)
@@ -51,6 +53,9 @@ job_run_dialog::job_run_dialog(wxWindow *parent,
   , abort(false)
   , jobs_to_start(n_jobs_to_start)
   , current_job(-1)
+#if defined(SYS_WINDOWS)
+  , m_taskbar_progress(NULL)
+#endif
 {
   wxBoxSizer *siz_all      = new wxBoxSizer(wxVERTICAL);
   wxStaticBoxSizer *siz_sb = new wxStaticBoxSizer(new wxStaticBox(this, -1, Z("Status and progress")), wxVERTICAL);
@@ -95,6 +100,11 @@ job_run_dialog::job_run_dialog(wxWindow *parent,
 
   SetSizer(siz_all);
 
+#if defined(SYS_WINDOWS)
+  if (get_windows_version() >= WINDOWS_VERSION_7)
+    m_taskbar_progress = new taskbar_progress_c(this);
+#endif
+
   start_next_job();
 
   ShowModal();
@@ -110,9 +120,9 @@ job_run_dialog::start_next_job() {
     if (   abort
         || (   cb_abort_after_current->IsChecked()
             && (current_job < jobs_to_start.size())))
-      add_to_log(wxString::Format(Z("Aborted processing on %s"), format_date_time(time(NULL)).c_str()));
+      add_to_log(wxString::Format(Z("Aborted processing on %s"), format_date_time(wxGetUTCTime()).c_str()));
     else
-      add_to_log(wxString::Format(Z("Finished processing on %s"), format_date_time(time(NULL)).c_str()));
+      add_to_log(wxString::Format(Z("Finished processing on %s"), format_date_time(wxGetUTCTime()).c_str()));
 
     b_abort->Enable(false);
     cb_abort_after_current->Enable(false);
@@ -123,13 +133,20 @@ job_run_dialog::start_next_job() {
     return;
   }
 
+#if defined(SYS_WINDOWS)
+  if (NULL != m_taskbar_progress) {
+    m_taskbar_progress->set_state(TBPF_NORMAL);
+    m_taskbar_progress->set_value(current_job * 100, jobs_to_start.size() * 100);
+  }
+#endif
+
   int ndx = jobs_to_start[current_job];
   st_jobs->SetLabel(wxString::Format(Z("Processing job %d/%d"), current_job + 1, (int)jobs_to_start.size()));
   st_current->SetLabel(wxString::Format(Z("Current job ID %d:"), jobs[ndx].id));
 
-  mdlg->load(wxString::Format(wxT("%s/jobs/%d.mmg"), wxGetCwd().c_str(), jobs[ndx].id));
+  mdlg->load(wxString::Format(wxT("%s/%d.mmg"), app->get_jobs_folder().c_str(), jobs[ndx].id));
 
-  opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"), get_temp_dir().c_str(), (int)wxGetProcessId(), (int)time(NULL));
+  opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"), get_temp_dir().c_str(), (int)wxGetProcessId(), (int)wxGetUTCTime());
 
   wxFile *opt_file;
   try {
@@ -158,7 +175,7 @@ job_run_dialog::start_next_job() {
     if ((*arg_list)[i].Length() == 0)
       opt_file->Write(wxT("#EMPTY#"));
     else {
-      string arg_utf8 = to_utf8((*arg_list)[i]);
+      std::string arg_utf8 = wxMB((*arg_list)[i]);
       opt_file->Write(arg_utf8.c_str(), arg_utf8.length());
     }
     opt_file->Write(wxT("\n"));
@@ -167,11 +184,16 @@ job_run_dialog::start_next_job() {
 
   process = new wxProcess(this, 1);
   process->Redirect();
-  pid = wxExecute((*arg_list)[0] + wxT(" \"@") + opt_file_name + wxT("\""), wxEXEC_ASYNC, process);
+  wxString command_line = wxString::Format(wxT("\"%s\" \"@%s\""), (*arg_list)[0].c_str(), opt_file_name.c_str());
+  pid = wxExecute(command_line, wxEXEC_ASYNC, process);
+  if (0 == pid) {
+    wxLogError(wxT("Execution of '%s' failed."), command_line.c_str());
+    return;
+  }
   out = process->GetInputStream();
 
   *jobs[ndx].log        = wxEmptyString;
-  jobs[ndx].started_on  = time(NULL);
+  jobs[ndx].started_on  = wxGetUTCTime();
   jobs[ndx].finished_on = -1;
 
   add_to_log(wxString::Format(Z("Starting job ID %d (%s) on %s"), jobs[ndx].id, jobs[ndx].description->c_str(), format_date_time(jobs[ndx].started_on).c_str()));
@@ -194,12 +216,8 @@ job_run_dialog::process_input() {
     }
 
     if (got_char && ((c == '\n') || (c == '\r') || out->Eof())) {
-#if WXUNICODE
-      wxString wx_line = wxU(line.c_str());
-#else
-      wxString wx_line = from_utf8(cc_local_utf8, line).c_str();
-#endif
-      if (wx_line.Find(wxT("Progress")) == 0) {
+      wxString wx_line = wxU(line);
+      if (wx_line.Find(Z("Progress")) == 0) {
         int percent_pos = wx_line.Find(wxT("%"));
         if (0 < percent_pos) {
           wx_line.Remove(percent_pos);
@@ -208,7 +226,7 @@ job_run_dialog::process_input() {
           long value;
           tmp.ToLong(&value);
           if ((value >= 0) && (value <= 100))
-            g_progress->SetValue(value);
+            set_progress_value(value);
         }
       } else if (wx_line.Length() > 0)
         *jobs[jobs_to_start[current_job]].log += wx_line + wxT("\n");
@@ -221,6 +239,16 @@ job_run_dialog::process_input() {
   }
 }
 
+void
+job_run_dialog::set_progress_value(long value) {
+  g_progress->SetValue(value);
+
+#if defined(SYS_WINDOWS)
+  if (NULL != m_taskbar_progress)
+    m_taskbar_progress->set_value(current_job * 100 + value, jobs_to_start.size() * 100);
+#endif
+}
+
 void
 job_run_dialog::on_timer(wxTimerEvent &evt) {
   process_input();
@@ -236,6 +264,8 @@ job_run_dialog::on_abort(wxCommandEvent &evt) {
   abort = true;
 #if defined(SYS_WINDOWS)
   wxKill(pid, wxSIGKILL);
+  if (NULL != m_taskbar_progress)
+    m_taskbar_progress->set_state(TBPF_ERROR);
 #else
   wxKill(pid, wxSIGTERM);
 #endif
@@ -247,7 +277,7 @@ job_run_dialog::on_end_process(wxProcessEvent &evt) {
 
   int ndx       = jobs_to_start[current_job];
   int exit_code = evt.GetExitCode();
-  const wxChar *status;
+  wxString status;
 
   if (abort) {
     jobs[ndx].status = JOBS_ABORTED;
@@ -263,10 +293,9 @@ job_run_dialog::on_end_process(wxProcessEvent &evt) {
     status           = Z("failed");
   }
 
-  jobs[ndx].finished_on = time(NULL);
+  jobs[ndx].finished_on = wxGetUTCTime();
 
-  wxString format = Z("Finished job ID %d on %s: status '%s'");
-  add_to_log(wxString::Format(format, jobs[ndx].id, format_date_time(jobs[ndx].finished_on).c_str(), status));
+  add_to_log(wxString::Format(Z("Finished job ID %d on %s: status '%s'"), jobs[ndx].id, format_date_time(jobs[ndx].finished_on).c_str(), status.c_str()));
 
   mdlg->save_job_queue();
   delete process;
@@ -478,7 +507,7 @@ job_dialog::enable_buttons(bool enable,
 void
 job_dialog::on_start(wxCommandEvent &evt) {
   int i;
-  vector<int> jobs_to_start;
+  std::vector<int> jobs_to_start;
 
   for (i = 0; jobs.size() > i; ++i)
     if (JOBS_PENDING == jobs[i].status)
@@ -491,7 +520,7 @@ job_dialog::on_start(wxCommandEvent &evt) {
 void
 job_dialog::on_start_selected(wxCommandEvent &evt) {
   int i;
-  vector<int> jobs_to_start;
+  std::vector<int> jobs_to_start;
 
   for (i = 0; jobs.size() > i; ++i)
     if (lv_jobs->IsSelected(i))
@@ -504,7 +533,7 @@ job_dialog::on_start_selected(wxCommandEvent &evt) {
 void
 job_dialog::on_delete(wxCommandEvent &evt) {
   int i;
-  vector<bool> selected;
+  std::vector<bool> selected;
 
   for (i = 0; lv_jobs->GetItemCount() > i; ++i)
     selected.push_back(lv_jobs->IsSelected(i));
@@ -513,7 +542,7 @@ job_dialog::on_delete(wxCommandEvent &evt) {
   int k = 0;
   while (jobs.size() > i) {
     if (selected[k]) {
-      wxRemoveFile(wxString::Format(wxT("jobs/%d.mmg"), jobs[i].id));
+      wxRemoveFile(wxString::Format(wxT("%s/%d.mmg"), app->get_jobs_folder().c_str(), jobs[i].id));
       jobs.erase(jobs.begin() + i);
       lv_jobs->DeleteItem(i);
     } else
@@ -553,7 +582,7 @@ job_dialog::swap_rows(int lower,
 
 void
 job_dialog::on_up(wxCommandEvent &evt) {
-  vector<bool> selected;
+  std::vector<bool> selected;
 
   bool first = true;
   int i      = 0;
@@ -578,7 +607,7 @@ job_dialog::on_up(wxCommandEvent &evt) {
 
 void
 job_dialog::on_down(wxCommandEvent &evt) {
-  vector<bool> selected;
+  std::vector<bool> selected;
 
   bool first = true;
   int i      = lv_jobs->GetItemCount() - 1;
@@ -668,8 +697,8 @@ job_dialog::on_item_selected(wxListEvent &evt) {
 }
 
 void
-job_dialog::start_jobs(vector<int> &jobs_to_start) {
-  wxString temp_settings = wxGetCwd() + wxT("/jobs/temp.mmg");
+job_dialog::start_jobs(std::vector<int> &jobs_to_start) {
+  wxString temp_settings = app->get_jobs_folder() + wxT("/temp.mmg");
   mdlg->save(temp_settings, true);
 
   mdlg->Show(false);
@@ -691,6 +720,11 @@ job_dialog::start_jobs(vector<int> &jobs_to_start) {
 
 void
 job_dialog::on_key_pressed(wxKeyEvent &evt) {
+  if (WXK_RETURN == evt.GetKeyCode()) {
+    EndModal(0);
+    return;
+  }
+
   if (1 != evt.GetKeyCode()) {  // Ctrl-A
     evt.Skip(true);
     return;
@@ -710,7 +744,9 @@ jobdlg_list_view::jobdlg_list_view(wxWindow *parent,
 
 void
 jobdlg_list_view::on_key_pressed(wxKeyEvent &evt) {
-  if (1 == evt.GetKeyCode())    // Ctrl-A
+  int key_code = evt.GetKeyCode();
+  if (   (1 == key_code)        // Ctrl-A
+      || (WXK_RETURN == key_code))
     ((job_dialog *)GetParent())->on_key_pressed(evt);
   else
     evt.Skip(true);
