@@ -17,19 +17,20 @@
 #include <map>
 #include <vector>
 
-#include "os.h"
+#include "common/os.h"
 
-#include "wx/app.h"
-#include "wx/combobox.h"
-#include "wx/config.h"
-#include "wx/string.h"
+#include <wx/app.h>
+#include <wx/combobox.h>
+#include <wx/config.h>
+#include <wx/filename.h>
+#include <wx/string.h>
 
-#include "ebml/EbmlUnicodeString.h"
+#include <ebml/EbmlUnicodeString.h>
 
-#include "iso639.h"
-#include "smart_pointers.h"
-#include "translation_table.h"
-#include "wxcommon.h"
+#include "common/iso639.h"
+#include "common/smart_pointers.h"
+#include "common/wx.h"
+#include "mmg/translation_table.h"
 
 #ifdef SYS_WINDOWS
 # define ALLFILES Z("All Files (*.*)|*.*")
@@ -45,6 +46,10 @@
 # define LEFTRIGHTSPACING 5
 #endif
 
+#define TRACK_ID_CHAPTERS    0x10000000
+#define TRACK_ID_GLOBAL_TAGS 0x10000001
+#define TRACK_ID_TAGS_BASE   0x10000002
+
 // Config file versions and their differences
 //
 // Version 1: base settings
@@ -57,7 +62,6 @@
 
 #define MMG_CONFIG_FILE_VERSION_MAX 3
 
-using namespace std;
 using namespace libebml;
 
 struct mmg_track_t {
@@ -68,10 +72,10 @@ struct mmg_track_t {
   bool enabled, display_dimensions_selected;
 
   int default_track;
-  bool aac_is_sbr, aac_is_sbr_detected;
+  bool aac_is_sbr, aac_is_sbr_detected, forced_track;
   bool track_name_was_present;
   wxString language, track_name, cues, delay, stretch, sub_charset;
-  wxString tags, fourcc, aspect_ratio, compression, timecodes, fps;
+  wxString tags, fourcc, aspect_ratio, cropping, compression, timecodes, fps;
   wxString packetizer;
   int nalu_size_length;
   wxString dwidth, dheight;
@@ -81,15 +85,50 @@ struct mmg_track_t {
 
   bool appending;
 
-  mmg_track_t():
-    type(0), id(0), source(0),
-    enabled(false), display_dimensions_selected(false),
-    default_track(0), aac_is_sbr(false), aac_is_sbr_detected(false),
-    track_name_was_present(false),
-    language(wxT("und")), cues(wxT("default")), sub_charset(wxT("default")),
-    nalu_size_length(0),
-    stereo_mode(0),
-    appending(false) {};
+  // For chapters and tags:
+  int num_entries;
+
+  mmg_track_t()
+    : type(0)
+    , id(0)
+    , source(0)
+    , enabled(false)
+    , display_dimensions_selected(false)
+    , default_track(0)
+    , aac_is_sbr(false)
+    , aac_is_sbr_detected(false)
+    , forced_track(false)
+    , track_name_was_present(false)
+    , language(wxT("und"))
+    , cues(wxT("default"))
+    , sub_charset(wxT("default"))
+    , nalu_size_length(0)
+    , stereo_mode(0)
+    , appending(false)
+    , num_entries(0)
+  {
+  }
+
+  wxString create_label();
+
+  bool is_video() {
+    return 'v' == type;
+  }
+  bool is_audio() {
+    return 'a' == type;
+  }
+  bool is_subtitles() {
+    return 's' == type;
+  }
+  bool is_buttons() {
+    return 'b' == type;
+  }
+  bool is_chapters() {
+    return 'c' == type;
+  }
+  bool is_tags() {
+    return 't' == type;
+  }
 };
 typedef counted_ptr<mmg_track_t> mmg_track_cptr;
 
@@ -106,7 +145,8 @@ struct mmg_attached_file_t {
     , id(0)
     , size(0)
     , source(NULL)
-  { };
+  {
+  }
 };
 typedef counted_ptr<mmg_attached_file_t> mmg_attached_file_cptr;
 
@@ -114,15 +154,17 @@ struct mmg_file_t {
   wxString file_name, title;
   bool title_was_present;
   int container;
-  vector<mmg_track_cptr> tracks;
-  vector<mmg_attached_file_cptr> attached_files;
-  bool no_chapters, no_attachments, no_tags;
+  std::vector<mmg_track_cptr> tracks;
+  std::vector<mmg_attached_file_cptr> attached_files;
   bool appending;
+  std::vector<wxFileName> other_files;
 
-  mmg_file_t():
-    title_was_present(false), container(0),
-    no_chapters(false), no_attachments(false), no_tags(false),
-    appending(false) {};
+  mmg_file_t()
+    : title_was_present(false)
+    , container(0)
+    , appending(false)
+  {
+  }
 };
 typedef counted_ptr<mmg_file_t> mmg_file_cptr;
 
@@ -130,8 +172,10 @@ struct mmg_attachment_t {
   wxString file_name, stored_name, description, mime_type;
   int style;
 
-  mmg_attachment_t():
-    style(0) {};
+  mmg_attachment_t()
+    : style(0)
+  {
+  }
 };
 typedef counted_ptr<mmg_attachment_t> mmg_attachment_cptr;
 
@@ -149,51 +193,63 @@ struct mmg_options_t {
   bool ask_before_overwriting;
   bool on_top;
   bool filenew_after_add_to_jobqueue;
+  bool filenew_after_successful_mux;
   bool warn_usage;
   bool gui_debugging;
-  bool always_use_simpleblock;
   bool set_delay_from_filename;
   wxString priority;
+  wxArrayString popular_languages;
 
-  mmg_options_t():
-    autoset_output_filename(false), output_directory_mode(ODM_FROM_FIRST_INPUT_FILE),
-    ask_before_overwriting(false),
-    on_top(false), filenew_after_add_to_jobqueue(false),
-    warn_usage(false), gui_debugging(false), always_use_simpleblock(false),
-    set_delay_from_filename(false) {
+  mmg_options_t()
+    : autoset_output_filename(false)
+    , output_directory_mode(ODM_FROM_FIRST_INPUT_FILE)
+    , ask_before_overwriting(false)
+    , on_top(false)
+    , filenew_after_add_to_jobqueue(false)
+    , filenew_after_successful_mux(false)
+    , warn_usage(false)
+    , gui_debugging(false)
+    , set_delay_from_filename(false)
+  {
+    init_popular_languages();
   }
 
   void validate();
+  void init_popular_languages(const wxString &list = wxEmptyString);
 };
 
+typedef std::pair<const wxString, bool> wxstring_bool_pair_t;
+typedef std::pair<const wxString, wxString> wxstring_wxstring_pair_t;
+
 extern wxString last_open_dir;
-extern vector<wxString> last_settings;
-extern vector<wxString> last_chapters;
-extern vector<mmg_file_cptr> files;
-extern vector<mmg_track_t *> tracks;
-extern vector<mmg_attachment_cptr> attachments;
-extern vector<mmg_attached_file_cptr> attached_files;
+extern std::vector<wxString> last_settings;
+extern std::vector<wxString> last_chapters;
+extern std::vector<mmg_file_cptr> files;
+extern std::vector<mmg_track_t *> tracks;
+extern std::vector<mmg_attachment_cptr> attachments;
+extern std::vector<mmg_attached_file_cptr> attached_files;
 extern wxArrayString sorted_charsets;
 extern wxArrayString sorted_iso_codes;
 extern bool title_was_present;
-extern map<wxString, wxString> capabilities;
+extern std::map<wxString, wxString> capabilities;
 
 wxString &break_line(wxString &line, int break_after = 80);
 wxString extract_language_code(wxString source);
-wxString shell_escape(wxString source);
-vector<wxString> split(const wxString &src, const wxString &pattern,
-                       int max_num = -1);
-wxString join(const wxString &pattern, vector<wxString> &strings);
+wxString shell_escape(wxString source, bool cmd_exe_mode = false);
+std::vector<wxString> split(const wxString &src, const wxString &pattern, int max_num = -1);
+wxString join(const wxString &pattern, std::vector<wxString> &strings);
+wxString join(const wxString &pattern, wxArrayString &strings);
 wxString &strip(wxString &s, bool newlines = false);
-vector<wxString> & strip(vector<wxString> &v, bool newlines = false);
-string to_utf8(const wxString &src);
-wxString from_utf8(const wxString &src);
+std::vector<wxString> & strip(std::vector<wxString> &v, bool newlines = false);
+wxString no_cr(wxString source);
 wxString UTFstring_to_wxString(const UTFstring &u);
 wxString unescape(const wxString &src);
 wxString format_date_time(time_t date_time);
 wxString get_temp_dir();
+wxString get_installation_dir();
 
 wxString create_track_order(bool all);
+wxString create_append_mapping();
 
 int default_track_checked(char type);
 
@@ -201,6 +257,8 @@ void set_combobox_selection(wxComboBox *cb, const wxString wanted);
 #if defined(USE_WXBITMAPCOMBOBOX)
 void set_combobox_selection(wxBitmapComboBox *cb, const wxString wanted);
 #endif  // USE_WXBITMAPCOMBOBOX
+void set_menu_item_strings(wxFrame *frame, int id, const wxString &title, const wxString &help_text);
+void set_menu_label(wxFrame *frame, int pos, const wxString &label);
 
 void wxdie(const wxString &errmsg);
 
@@ -217,7 +275,11 @@ public:
 public:
   virtual bool OnInit();
   virtual int OnExit();
-  virtual void init_ui_locale(wxConfigBase *cfg);
+  virtual void init_ui_locale();
+  virtual void handle_command_line_arguments();
+  virtual wxString get_config_file_name();
+  virtual wxString get_jobs_folder();
+  virtual void prepare_mmg_data_folder();
 };
 
 extern mmg_app *app;
