@@ -11,33 +11,33 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include "wx/wxprec.h"
+#include <wx/wxprec.h>
 
-#include "wx/wx.h"
-#include "wx/clipbrd.h"
-#include "wx/file.h"
-#include "wx/confbase.h"
-#include "wx/fileconf.h"
-#include "wx/notebook.h"
-#include "wx/listctrl.h"
-#include "wx/statusbr.h"
-#include "wx/statline.h"
+#include <wx/wx.h>
+#include <wx/clipbrd.h>
+#include <wx/file.h>
+#include <wx/confbase.h>
+#include <wx/fileconf.h>
+#include <wx/notebook.h>
+#include <wx/listctrl.h>
+#include <wx/statusbr.h>
+#include <wx/statline.h>
 
 #if defined(SYS_WINDOWS)
 #include <windows.h>
 #endif
 
-#include "common.h"
-#include "mmg.h"
-#include "mmg_dialog.h"
-#include "mux_dialog.h"
+#include "common/common.h"
+#include "mmg/mmg.h"
+#include "mmg/mmg_dialog.h"
+#include "mmg/mux_dialog.h"
 
 mux_dialog::mux_dialog(wxWindow *parent):
   wxDialog(parent, -1, Z("mkvmerge is running"), wxDefaultPosition,
@@ -46,9 +46,16 @@ mux_dialog::mux_dialog(wxWindow *parent):
 #else
            wxSize(700, 520),
 #endif
-           wxDEFAULT_FRAME_STYLE) {
+           wxDEFAULT_FRAME_STYLE)
+#if defined(SYS_WINDOWS)
+  , pid(0)
+  , m_taskbar_progress(NULL)
+  , m_abort_button_changed(false)
+#endif  // SYS_WINDOWS
+  , m_exit_code(0)
+{
   char c;
-  string arg_utf8, line;
+  std::string arg_utf8, line;
   long value;
   wxString wx_line, tmp;
   wxInputStream *out;
@@ -58,6 +65,8 @@ mux_dialog::mux_dialog(wxWindow *parent):
   wxBoxSizer *siz_all, *siz_buttons;
   wxStaticBoxSizer *siz_status, *siz_output;
 
+  m_window_disabler = new wxWindowDisabler(this);
+
   c = 0;
   siz_status = new wxStaticBoxSizer(new wxStaticBox(this, -1, Z("Status and progress")), wxVERTICAL);
   st_label = new wxStaticText(this, -1, wxEmptyString);
@@ -100,7 +109,7 @@ mux_dialog::mux_dialog(wxWindow *parent):
 
   process = new mux_process(this);
 
-  opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"), get_temp_dir().c_str(), (int)wxGetProcessId(), (int)time(NULL));
+  opt_file_name.Printf(wxT("%smmg-mkvmerge-options-%d-%d"), get_temp_dir().c_str(), (int)wxGetProcessId(), (int)wxGetUTCTime());
   try {
     const unsigned char utf8_bom[3] = {0xef, 0xbb, 0xbf};
     opt_file = new wxFile(opt_file_name, wxFile::write);
@@ -116,15 +125,28 @@ mux_dialog::mux_dialog(wxWindow *parent):
     if ((*arg_list)[i].Length() == 0)
       opt_file->Write(wxT("#EMPTY#"));
     else {
-      arg_utf8 = to_utf8((*arg_list)[i]);
+      arg_utf8 = wxMB((*arg_list)[i]);
       opt_file->Write(arg_utf8.c_str(), arg_utf8.length());
     }
     opt_file->Write(wxT("\n"));
   }
   delete opt_file;
 
-  pid = wxExecute((*arg_list)[0] + wxT(" \"@") + opt_file_name + wxT("\""),
-                  wxEXEC_ASYNC, process);
+#if defined(SYS_WINDOWS)
+  if (get_windows_version() >= WINDOWS_VERSION_7) {
+    m_taskbar_progress = new taskbar_progress_c(mdlg);
+    m_taskbar_progress->set_state(TBPF_NORMAL);
+    m_taskbar_progress->set_value(0, 100);
+  }
+#endif  // SYS_WINDOWS
+
+  wxString command_line = wxString::Format(wxT("\"%s\" \"@%s\""), (*arg_list)[0].c_str(), opt_file_name.c_str());
+  pid = wxExecute(command_line, wxEXEC_ASYNC, process);
+  if (0 == pid) {
+    wxLogError(wxT("Execution of '%s' failed."), command_line.c_str());
+    done(2);
+    return;
+  }
   out = process->GetInputStream();
 
   line = "";
@@ -144,11 +166,7 @@ mux_dialog::mux_dialog(wxWindow *parent):
       c = '\n';
 
     if ((c == '\n') || (c == '\r') || out->Eof()) {
-#if WXUNICODE
-      wx_line = wxU(line.c_str());
-#else
-      wx_line = from_utf8(cc_local_utf8, line).c_str();
-#endif
+      wx_line = wxU(line);
       log += wx_line;
       if (c != '\r')
         log += wxT("\n");
@@ -176,6 +194,12 @@ mux_dialog::mux_dialog(wxWindow *parent):
 }
 
 mux_dialog::~mux_dialog() {
+#if defined(SYS_WINDOWS)
+  if (NULL != m_taskbar_progress)
+    m_taskbar_progress->set_state(TBPF_NOPROGRESS);
+  delete m_taskbar_progress;
+#endif  // SYS_WINDOWS
+
   process->dlg = NULL;
   delete process;
   wxRemoveFile(opt_file_name);
@@ -189,6 +213,10 @@ mux_dialog::update_window(wxString text) {
 void
 mux_dialog::update_gauge(long value) {
   g_progress->SetValue(value);
+#if defined(SYS_WINDOWS)
+  if (NULL != m_taskbar_progress)
+    m_taskbar_progress->set_value(value, 100);
+#endif  // SYS_WINDOWS
 }
 
 void
@@ -216,44 +244,78 @@ mux_dialog::on_save_log(wxCommandEvent &evt) {
 void
 mux_dialog::on_abort(wxCommandEvent &evt) {
 #if defined(SYS_WINDOWS)
+  if (0 == pid) {
+    wxFileName output_file_name(static_cast<mmg_dialog *>(GetParent())->tc_output->GetValue());
+    wxExecute(wxString::Format(wxT("explorer \"%s\""), output_file_name.GetPath().c_str()));
+    return;
+  }
+
   wxKill(pid, wxSIGKILL);
+  change_abort_button();
 #else
+  if (0 == pid)
+    return;
+
   wxKill(pid, wxSIGTERM);
-#endif
   b_abort->Enable(false);
+#endif
 }
 
 void
 mux_dialog::on_close(wxCloseEvent &evt) {
-  mdlg->muxing_has_finished();
+  mdlg->muxing_has_finished(m_exit_code);
+  delete m_window_disabler;
   Destroy();
 }
 
 void
-mux_dialog::done() {
+mux_dialog::done(int status) {
   SetTitle(Z("mkvmerge has finished"));
 
+  pid = 0;
   b_ok->Enable(true);
-  b_abort->Enable(false);
   b_ok->SetFocus();
+
+#if defined(SYS_WINDOWS)
+  change_abort_button();
+  if ((NULL != m_taskbar_progress) && (0 != status))
+    m_taskbar_progress->set_state(TBPF_ERROR);
+#else  // SYS_WINDOWS
+  b_abort->Enable(false);
+#endif  // SYS_WINDOWS
 }
 
+#if defined(SYS_WINDOWS)
+void
+mux_dialog::change_abort_button() {
+  if (m_abort_button_changed)
+    return;
+
+  b_abort->SetLabel(Z("Open folder"));
+  m_abort_button_changed = true;
+
+  Layout();
+}
+#endif  // SYS_WINDOWS
+
 mux_process::mux_process(mux_dialog *mux_dlg):
   wxProcess(wxPROCESS_REDIRECT),
   dlg(mux_dlg) {
 }
 
 void
-mux_process::OnTerminate(int pid,
+mux_process::OnTerminate(int terminated_pid,
                          int status) {
   if (NULL == dlg)
     return;
 
+  dlg->set_exit_code(status);
+
   wxString format;
   if ((status != 0) && (status != 1))
-    format = Z("mkvmerge FAILED with a return code of %d. %s\n");
+    format = Z("mkvmerge FAILED with a return code of %d. %s");
   else
-    format = Z("mkvmerge finished with a return code of %d. %s\n");
+    format = Z("mkvmerge finished with a return code of %d. %s");
 
 #if defined(SYS_WINDOWS)
   wxString status_1 = Z("There were warnings, or the process was terminated.");
@@ -267,7 +329,7 @@ mux_process::OnTerminate(int pid,
                       :               wxString(wxT(""));
 
   dlg->update_window(wxString::Format(format, status, status_str.c_str()));
-  dlg->done();
+  dlg->done((0 == status) || (1 == status) ? status : 2);
 }
 
 IMPLEMENT_CLASS(mux_dialog, wxDialog);
