@@ -14,8 +14,15 @@
 #ifndef __MUX_DIALOG_H
 #define __MUX_DIALOG_H
 
-#include "wx/dialog.h"
-#include "wx/process.h"
+#include "common/os.h"
+
+#include <wx/dialog.h>
+#include <wx/process.h>
+
+#if defined(SYS_WINDOWS)
+# include "common/fs_sys_helpers.h"
+# include "mmg/taskbar_progress.h"
+#endif  // SYS_WINDOWS
 
 #define ID_B_MUX_OK                       17000
 #define ID_B_MUX_SAVELOG                  17001
@@ -34,6 +41,13 @@ protected:
   wxString log, opt_file_name;
   wxButton *b_ok, *b_save_log, *b_abort;
   wxTextCtrl *tc_output, *tc_warnings, *tc_errors;
+  wxWindowDisabler *m_window_disabler;
+#if defined(SYS_WINDOWS)
+  taskbar_progress_c *m_taskbar_progress;
+  bool m_abort_button_changed;
+#endif  // SYS_WINDOWS
+
+  int m_exit_code;
 public:
 
   mux_dialog(wxWindow *parent);
@@ -46,7 +60,15 @@ public:
   void on_save_log(wxCommandEvent &evt);
   void on_abort(wxCommandEvent &evt);
   void on_close(wxCloseEvent &evt);
-  void done();
+  void done(int status);
+
+  void set_exit_code(int exit_code) {
+    m_exit_code = exit_code;
+  }
+
+#if defined(SYS_WINDOWS)
+  void change_abort_button();
+#endif  // SYS_WINDOWS
 };
 
 class mux_process: public wxProcess {
@@ -54,7 +76,7 @@ public:
   mux_dialog *dlg;
 
   mux_process(mux_dialog *mdlg);
-  virtual void OnTerminate(int pid, int status);
+  virtual void OnTerminate(int terminated_pid, int status);
 };
 
 #endif // __MUX_DIALOG_H
