@@ -6,24 +6,20 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_vorbis.cpp 3978 2008-10-09 07:50:55Z mosu $
-
    Vorbis packetizer
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
-
-#include <string.h>
+#include "common/os.h"
 
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
 
-#include "common.h"
-#include "matroska.h"
-#include "output_control.h"
-#include "p_vorbis.h"
+#include "common/common.h"
+#include "common/matroska.h"
+#include "merge/output_control.h"
+#include "output/p_vorbis.h"
 
 using namespace libmatroska;
 
@@ -49,9 +45,9 @@ vorbis_packetizer_c::vorbis_packetizer_c(generic_reader_c *p_reader,
 
   ogg_packet ogg_headers[3];
   memset(ogg_headers, 0, 3 * sizeof(ogg_packet));
-  ogg_headers[0].packet   = m_headers[0]->get();
-  ogg_headers[1].packet   = m_headers[1]->get();
-  ogg_headers[2].packet   = m_headers[2]->get();
+  ogg_headers[0].packet   = m_headers[0]->get_buffer();
+  ogg_headers[1].packet   = m_headers[1]->get_buffer();
+  ogg_headers[2].packet   = m_headers[2]->get_buffer();
   ogg_headers[0].bytes    = l_header;
   ogg_headers[1].bytes    = l_comments;
   ogg_headers[2].bytes    = l_codecsetup;
@@ -83,7 +79,7 @@ vorbis_packetizer_c::set_headers() {
   set_codec_id(MKV_A_VORBIS);
 
   memory_cptr codec_private = lace_memory_xiph(m_headers);
-  set_codec_private(codec_private->get(), codec_private->get_size());
+  set_codec_private(codec_private->get_buffer(), codec_private->get_size());
 
   set_audio_sampling_freq((float)m_vi.rate);
   set_audio_channels(m_vi.channels);
@@ -101,7 +97,7 @@ vorbis_packetizer_c::process(packet_cptr packet) {
 
   // Update the number of samples we have processed so that we can
   // calculate the timecode on the next call.
-  op.packet                  = packet->data->get();
+  op.packet                  = packet->data->get_buffer();
   op.bytes                   = packet->data->get_size();
   int64_t this_bs            = vorbis_packet_blocksize(&m_vi, &op);
   int64_t samples_here       = (this_bs + m_previous_bs) / 4;
@@ -125,7 +121,7 @@ vorbis_packetizer_c::process(packet_cptr packet) {
   m_previous_samples_sum += samples_here;
 
   mxverb(2,
-         boost::format(Y("Vorbis: samples_here at %1% (orig %2% expected %3%): %4% (m_previous_samples_sum: %5%)\n"))
+         boost::format("Vorbis: samples_here at %1% (orig %2% expected %3%): %4% (m_previous_samples_sum: %5%)\n")
          % chosen_timecode % packet->timecode % expected_timecode % samples_here % m_previous_samples_sum);
   packet->timecode = chosen_timecode;
   add_packet(packet);
@@ -135,7 +131,7 @@ vorbis_packetizer_c::process(packet_cptr packet) {
 
 connection_result_e
 vorbis_packetizer_c::can_connect_to(generic_packetizer_c *src,
-                                    string &error_message) {
+                                    std::string &error_message) {
   vorbis_packetizer_c *vsrc = dynamic_cast<vorbis_packetizer_c *>(src);
   if (NULL == vsrc)
     return CAN_CONNECT_NO_FORMAT;
@@ -143,7 +139,7 @@ vorbis_packetizer_c::can_connect_to(generic_packetizer_c *src,
   connect_check_a_samplerate(m_vi.rate,   vsrc->m_vi.rate);
   connect_check_a_channels(m_vi.channels, vsrc->m_vi.channels);
 
-  if ((m_headers[2]->get_size() != vsrc->m_headers[2]->get_size()) || memcmp(m_headers[2]->get(), vsrc->m_headers[2]->get(), m_headers[2]->get_size())) {
+  if ((m_headers[2]->get_size() != vsrc->m_headers[2]->get_size()) || memcmp(m_headers[2]->get_buffer(), vsrc->m_headers[2]->get_buffer(), m_headers[2]->get_size())) {
     error_message = Y("The Vorbis codebooks are different; such tracks cannot be concatenated without reencoding");
     return CAN_CONNECT_NO_FORMAT;
   }
